//+------------------------------------------------------------------+
//|                                                         Tree.mqh |
//|                        Copyright 2010, MetaQuotes Software Corp. |
//|                                        http://www.metaquotes.net |
//|                                              Revision 2010.02.22 |
//+------------------------------------------------------------------+
#include  "TreeNode.mqh"
//+------------------------------------------------------------------+
//| Class CTree.                                                     |
//| Appointment: Building a binary search tree.                      |
//|              Derives from class CTreeNode.                       |
//+------------------------------------------------------------------+
class CTree : public CTreeNode
  {
protected:
   CTreeNode        *m_root_node;            // root node of the tree
//---
   void              Balance(CTreeNode *node);

public:
   void              CTree(void);
   void             ~CTree(void);
   //--- methods of access to protected data
   CTreeNode*        Root()       { return(m_root_node); }
   //--- method of identifying the object
   virtual int       Type() const { return(0x9999); }
   //--- method of filling the tree
   CTreeNode*        Insert(CTreeNode *new_node);
   //--- methods of removing tree nodes
   bool              Detach(CTreeNode *node);
   bool              Delete(CTreeNode *node);
   void              Clear(void);
   //--- method of searching data in a tree
   CTreeNode*        Find(CTreeNode *node);
   //--- method to create the element tree
   virtual CTreeNode *CreateElement() { return(NULL); }
   //--- methods for working with files
   virtual bool      Save(int file_handle);
   virtual bool      Load(int file_handle);
  };

//+------------------------------------------------------------------+
//| Constructor CMQL5List.                                           |
//| INPUT:  no.                                                      |
//| OUTPUT: no.                                                      |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
void CTree::CTree()
  {
//--- initialize protected data
   m_root_node  =NULL;
  }
//+------------------------------------------------------------------+
//| Destructor CMQL5List.                                            |
//| INPUT:  no.                                                      |
//| OUTPUT: no.                                                      |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
void CTree::~CTree()
  {
   Clear();
  }  
//+------------------------------------------------------------------+
//| Method of adding tree node.                                      |
//| INPUT:  new_node-node to add.                                    |
//| OUTPUT: no.                                                      |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
CTreeNode* CTree::Insert(CTreeNode *new_node)
  {
   CTreeNode *p_node;
   CTreeNode *result=m_root_node;
//--- checking
   if(!CheckPointer(new_node)) return(NULL);
//--- add
   if(result!=NULL)
     {
      p_node=NULL;
      result=m_root_node;
      while(result!=NULL && result.Compare(new_node)!=0)
        {
         p_node=result;
         result=result.GetNext(new_node);
        }
      if(result!=NULL) return(result);
      if(p_node.Compare(new_node)>0) p_node.Left(new_node);
      else                           p_node.Right(new_node);
      new_node.Parent(p_node);
      Balance(p_node);
     }
   else
      m_root_node=new_node;
//---
   return(result);
  }
//+------------------------------------------------------------------+
//| Method of removing tree node.                                    |
//| INPUT:  node-node to remove.                                     |
//| OUTPUT: no.                                                      |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CTree::Delete(CTreeNode *node)
  {
//--- checking
   if(!CheckPointer(node)) return(false);
//--- delete
   if(Detach(node)) delete node;
//---
   return(true);
  }
//+------------------------------------------------------------------+
//| Method of detaching tree node.                                   |
//| INPUT:  node-node to detach.                                     |
//| OUTPUT: no.                                                      |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CTree::Detach(CTreeNode *node)
  {
   bool result;
   CTreeNode *curr_node,*tmp_node;
   CTreeNode *nodeA,*nodeB;
//--- checking
   curr_node=node;
   if(!CheckPointer(curr_node)) return(false);
//--- detach
   if(curr_node.BalanceL()>curr_node.BalanceR())
     {
      nodeA=curr_node.Left();
      while(nodeA.Right()!=NULL) nodeA=nodeA.Right();
      nodeB=nodeA.Parent();
      if(nodeB!=curr_node)
        {
         nodeB.Right(nodeA.Left());
         tmp_node=nodeB.Right();
         if(tmp_node!=NULL) tmp_node.Parent(nodeB);
         tmp_node=curr_node.Left();
         nodeA.Left(tmp_node);
         tmp_node.Parent(nodeA);
        }
      //--- left us curr_node already installed as it should
      curr_node.Left(NULL);
      //--- transferring the right link curr_node in nodeA
      nodeA.Right(curr_node.Right());
      tmp_node=curr_node.Right();
      if(tmp_node!=NULL) tmp_node.Parent(nodeA);
      curr_node.Right(NULL);
      //--- transferring the root link curr_node in nodeA
      tmp_node=curr_node.Parent();
      nodeA.Parent(tmp_node);
      if(tmp_node!=NULL)
        {
         if(tmp_node.Left()==curr_node) tmp_node.Left(nodeA);
         else                           tmp_node.Right(nodeA);
        }
      else
        {
         curr_node.Parent(NULL);
         m_root_node=nodeA;
         tmp_node=nodeA;
        }
      Balance(tmp_node);
     }
    else
      {
       if(curr_node.BalanceR()>0)
         {
          nodeA=curr_node.Right();
          while(nodeA.Left()!=NULL) nodeA=nodeA.Left();
          nodeB=nodeA.Parent();
          if(nodeB!=curr_node)
            {
             nodeB.Left(nodeA.Right());
             tmp_node=nodeB.Left();
             if(tmp_node!=NULL) tmp_node.Parent(nodeB);
             tmp_node=curr_node.Right();
             nodeA.Right(tmp_node);
             tmp_node.Parent(nodeA);
            }
         //--- right us curr_node already installed as it should
         curr_node.Right(NULL);
         //--- transferring the left link curr_node in nodeA
         nodeA.Left(curr_node.Left());
         tmp_node=curr_node.Left();
         if(tmp_node!=NULL) tmp_node.Parent(nodeA);
         curr_node.Left(NULL);
         //--- transferring the root link curr_node in nodeA
         tmp_node=curr_node.Parent();
         nodeA.Parent(tmp_node);
         if(tmp_node!=NULL)
           {
            if(tmp_node.Left()==curr_node) tmp_node.Left(nodeA);
            else                           tmp_node.Right(nodeA);
           }
         else
           {
            curr_node.Parent(NULL);
            m_root_node=nodeA;
            tmp_node=nodeA;
           }
          Balance(tmp_node);
         }
       else
         {
          //--- node-leaf
          if(curr_node.Parent()==NULL) m_root_node=NULL;
          else
            {
             tmp_node=curr_node.Parent();
             if(tmp_node.Left()==curr_node) tmp_node.Left(NULL);
             else                           tmp_node.Right(NULL);
             curr_node.Parent(NULL);
            }
          Balance(curr_node.Parent());
         }
      }
//---
   return(true);
  }
//+------------------------------------------------------------------+
//| Method of cleaning tree.                                         |
//| INPUT:  no.                                                      |
//| OUTPUT: no.                                                      |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
void CTree::Clear(void)
  {
   if(m_root_node!=NULL) delete m_root_node;
   m_root_node=NULL;
  }
//+------------------------------------------------------------------+
//| Method of find node in the tree.                                 |
//| INPUT:  key-sample data to find.                                 |
//| OUTPUT: no.                                                      |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
CTreeNode* CTree::Find(CTreeNode *node)
  {
  CTreeNode *result=m_root_node;
//--- find
  while(result!=NULL && result.Compare(node)!=0)
    result=result.GetNext(node);
//---
   return(result);
  }
//+------------------------------------------------------------------+
//| Method of balancing the tree.                                    |
//| INPUT:  no.                                                      |
//| OUTPUT: no.                                                      |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
void CTree::Balance(CTreeNode *node)
  {
   CTreeNode *nodeA,*nodeB,*nodeC,*curr_node,*tmp_node;
//---
   curr_node=node;
   while(curr_node!=NULL)
     {
      curr_node.RefreshBalance();
      if(MathAbs(curr_node.BalanceL()-curr_node.BalanceR())<=1)
         curr_node=curr_node.Parent();
      else
        {
         if(curr_node.BalanceR()>curr_node.BalanceL())
           {
            //--- right rotation
            tmp_node=curr_node.Right();
            if(tmp_node.BalanceL() > tmp_node.BalanceR())
              {
               //--- great right rotation
               nodeA=curr_node;
               nodeB=nodeA.Right();
               nodeC=nodeB.Left();
               nodeC.Parent(nodeA.Parent());
               tmp_node=nodeC.Parent();
               if(tmp_node!=NULL)
                 {
                  if(tmp_node.Right()==nodeA) tmp_node.Right(nodeC);
                  else                        tmp_node.Left(nodeC);
                 }
               else
                  m_root_node=nodeC;
               nodeA.Parent(nodeC);
               nodeB.Parent(nodeC);
               nodeA.Right(nodeC.Left());
               tmp_node=nodeA.Right();
               if(tmp_node!=NULL) tmp_node.Parent(nodeA);
               nodeC.Left(nodeA);
               nodeB.Left(nodeC.Right());
               tmp_node=nodeB.Left();
               if(tmp_node!=NULL) tmp_node.Parent(nodeB);
               nodeC.Right(nodeB);
               if(m_root_node==nodeA) m_root_node=nodeC;
               curr_node=nodeC.Parent();
              }
            else
              {
               //--- small right rotation
               nodeA=curr_node;
               nodeB=nodeA.Right();
               nodeB.Parent(nodeA.Parent());
               tmp_node=nodeB.Parent();
               if(tmp_node!=NULL)
                 {
                  if(tmp_node.Right()==nodeA) tmp_node.Right(nodeB);
                  else                        tmp_node.Left(nodeB);
                 }
               else
                  m_root_node=nodeB;
               nodeA.Parent(nodeB);
               nodeA.Right(nodeB.Left());
               tmp_node=nodeA.Right();
               if(tmp_node!=NULL) tmp_node.Parent(nodeA);
               nodeB.Left(nodeA);
               if(m_root_node==nodeA) m_root_node=nodeB;
               curr_node=nodeB.Parent();
              }
           }
         else
           {
            //--- left rotation
            tmp_node=curr_node.Left();
            if(tmp_node.BalanceR()>tmp_node.BalanceL())
              {
               //--- great left rotation
               nodeA=curr_node;
               nodeB=nodeA.Left();
               nodeC=nodeB.Right();
               nodeC.Parent(nodeA.Parent());
               tmp_node=nodeC.Parent();
               if(tmp_node!=NULL)
                 {
                  if(tmp_node.Right()==nodeA) tmp_node.Right(nodeC);
                  else                        tmp_node.Left(nodeC);
                 }
               else
                  m_root_node=nodeC;
               nodeA.Parent(nodeC);
               nodeB.Parent(nodeC);
               nodeA.Left(nodeC.Right());
               tmp_node=nodeA.Left();
               if(tmp_node!=NULL) tmp_node.Parent(nodeA);
               nodeC.Right(nodeA);
               nodeB.Right(nodeC.Left());
               tmp_node=nodeB.Right();
               if(tmp_node!=NULL) tmp_node.Parent(nodeB);
               nodeC.Left(nodeB);
               if(m_root_node==nodeA) m_root_node=nodeC;
               curr_node=nodeC.Parent();
              }
            else
              {
               //--- small left rotation
              nodeA=curr_node;
              nodeB=nodeA.Left();
              nodeB.Parent(nodeA.Parent());
              tmp_node=nodeB.Parent();
              if(tmp_node!=NULL)
                {
                 if(tmp_node.Right()==nodeA) tmp_node.Right(nodeB);
                 else                        tmp_node.Left(nodeB);
                }
              else
                 m_root_node=nodeB;
              nodeA.Parent(nodeB);
              nodeA.Left(nodeB.Right());
              tmp_node=nodeA.Left();
              if(tmp_node!=NULL) tmp_node.Parent(nodeA);
              nodeB.Right(nodeA);
              if(m_root_node==nodeA) m_root_node=nodeB;
              curr_node=nodeB.Parent();
              }
           }
        }
     }
  }
//+------------------------------------------------------------------+
//| Writing tree to file.                                            |
//| INPUT:  file_handle -handle previously opened for writing file.  |
//| OUTPUT: true if OK, else false.                                  |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CTree::Save(int file_handle)
  {
//--- checking
   if(file_handle<0)     return(false);
   if(m_root_node==NULL) return(true);
//---
   return(m_root_node.SaveNode(file_handle));
  }
//+------------------------------------------------------------------+
//| Reading tree from file.                                          |
//| INPUT:  file_handle -handle previously opened for reading file.  |
//| OUTPUT: true if OK, else false.                                  |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CTree::Load(int file_handle)
  {
//--- checking
   if(file_handle<0) return(false);
//--- create root node only
   Clear();
   Insert(CreateElement());
//---
   return(m_root_node.LoadNode(file_handle,m_root_node));
  }
//+------------------------------------------------------------------+

