//+------------------------------------------------------------------+
//|                                                  ChartObject.mqh |
//|                        Copyright 2010, MetaQuotes Software Corp. |
//|                                        http://www.metaquotes.net |
//|                                              Revision 2010.02.22 |
//+------------------------------------------------------------------+
#include <Object.mqh>
//+------------------------------------------------------------------+
//| Class CChartObject.                                              |
//| Appointment: Base class chart object.                            |
//|              Derives from class CObject.                         |
//+------------------------------------------------------------------+
class CChartObject : public CObject
  {
protected:
   long              m_chart_id;           // idintifier chart on which "lies" object
   int               m_window;             // number of subwindows (0 - main window)
   string            m_name;               // unique object name
   int               m_num_points;         // number of points "anchor" object
public:
                     CChartObject();
                    ~CChartObject();
   //--- methods of access to protected data
   long              ChartId() const       { return(m_chart_id);   }
   int               Window() const        { return(m_window);     }
   string            Name() const          { return(m_name);       }
   bool              Name(const string name);
   int               NumPoints() const     { return(m_num_points); }
   //--- method of identifying the object
   virtual int       Type() const          { return(0x8888);       }
   //--- methods of filling an object
   bool              Attach(long chart_id,string name,int window,int points);
   bool              SetPoint(int point,datetime time,double price);
   //--- methods deleting
   bool              Delete();
   void              Detach();
   //--- methods access to properties object
   datetime          Time(int point) const;
   bool              Time(int point,datetime time);
   double            Price(int point) const;
   bool              Price(int point,double price);
   color             Color() const;
   bool              Color(color new_color);
   ENUM_LINE_STYLE   Style() const;
   bool              Style(ENUM_LINE_STYLE new_style);
   int               Width() const;
   bool              Width(int new_width);
   bool              Background() const;
   bool              Background(bool new_back);
   bool              Selected() const;
   bool              Selected(bool new_sel);
   bool              Selectable() const;
   bool              Selectable(bool new_sel);
   string            Description() const;
   bool              Description(const string new_text);
   int               Timeframes() const;
   virtual bool      Timeframes(int timeframes);
   datetime          CreateTime() const;
   int               LevelsCount() const;
   bool              LevelsCount(int new_count);
   //--- methods access to properties objects levels
   color             LevelColor(int level) const;
   bool              LevelColor(int level,color new_color);
   ENUM_LINE_STYLE   LevelStyle(int level) const;
   bool              LevelStyle(int level,ENUM_LINE_STYLE new_style);
   int               LevelWidth(int level) const;
   bool              LevelWidth(int level,int new_width);
   double            LevelValue(int level) const;
   bool              LevelValue(int level,double new_value);
   string            LevelDescription(int level) const;
   bool              LevelDescription(int level,const string new_text);
   //--- access methods to the API MQL5 functions
   long              GetInteger(ENUM_OBJECT_PROPERTY_INTEGER prop_id,int modifier=-1) const;
   bool              GetInteger(ENUM_OBJECT_PROPERTY_INTEGER prop_id,int modifier,long& value) const;
   bool              SetInteger(ENUM_OBJECT_PROPERTY_INTEGER prop_id,int modifier,long value);
   bool              SetInteger(ENUM_OBJECT_PROPERTY_INTEGER prop_id,long value);
   double            GetDouble(ENUM_OBJECT_PROPERTY_DOUBLE prop_id,int modifier=-1) const;
   bool              GetDouble(ENUM_OBJECT_PROPERTY_DOUBLE prop_id,int modifier,double& value) const;
   bool              SetDouble(ENUM_OBJECT_PROPERTY_DOUBLE prop_id,int modifier,double value);
   bool              SetDouble(ENUM_OBJECT_PROPERTY_DOUBLE prop_id,double value);
   string            GetString(ENUM_OBJECT_PROPERTY_STRING prop_id,int modifier=-1) const;
   bool              GetString(ENUM_OBJECT_PROPERTY_STRING prop_id,int modifier,string& value) const;
   bool              SetString(ENUM_OBJECT_PROPERTY_STRING prop_id,int modifier,string value);
   bool              SetString(ENUM_OBJECT_PROPERTY_STRING prop_id,string value);
   //--- methods move
   bool              ShiftObject(datetime d_time,double d_price);
   bool              ShiftPoint(int point,datetime d_time,double d_price);
   //--- methods for working with files
   virtual bool      Save(int file_handle);
   virtual bool      Load(int file_handle);
  };
//+------------------------------------------------------------------+
//| Constructor CChartObject.                                        |
//| INPUT:  no.                                                      |
//| OUTPUT: no.                                                      |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
void CChartObject::CChartObject()
  {
//--- initialize protected data
   Detach();
  }
//+------------------------------------------------------------------+
//| Destructor CChartObject.                                         |
//| INPUT:  no.                                                      |
//| OUTPUT: no.                                                      |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
void CChartObject::~CChartObject()
  {
   if(m_chart_id!=-1) ObjectDelete(m_chart_id,m_name);
  }
//+------------------------------------------------------------------+
//| Changing the name of the object.                                 |
//| INPUT:  name -new name of the object.                            |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CChartObject::Name(const string name)
  {
//--- checking
   if(m_chart_id==-1) return(false);
//--- change
   if(ObjectSetString(m_chart_id,m_name,OBJPROP_NAME,name))
     {
      m_name=name;
      return(true);
     }
//---
   return(false);
  };
//+------------------------------------------------------------------+
//| Attach object.                                                   |
//| INPUT:  chart_id-chart idintifier,                               |
//|         name    -object name,                                    |
//|         window  -number of subwindows,                           |
//|         points  -number of points "anchor" object.               |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CChartObject::Attach(long chart_id,string name,int window,int points)
  {
//--- checking
   if(ObjectFind(chart_id,name)<0) return(false);
//--- attach
   if(chart_id==0) chart_id=ChartID();
   m_chart_id  =chart_id;
   m_window    =window;
   m_name      =name;
   m_num_points=points;
//---
   return(true);
  }
//+------------------------------------------------------------------+
//| Setting the object point.                                        |
//| INPUT:  point -number object point,                              |
//|         time  -new time,                                         |
//|         price -new price.                                        |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CChartObject::SetPoint(int point,datetime time,double price)
  {
//--- checking
   if(m_chart_id==-1)      return(false);
   if(point>=m_num_points) return(false);
//---
   bool res=ObjectMove(m_chart_id,m_name,point,time,price);
//---
   return(res);
  }
//+------------------------------------------------------------------+
//| Delete object.                                                   |
//| INPUT:  no.                                                      |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CChartObject::Delete()
  {
//--- checking
   if(m_chart_id==-1) return(false);
//---
   bool result=ObjectDelete(m_chart_id,m_name);
   Detach();
//---
   return(result);
  }
//+------------------------------------------------------------------+
//| Detach object.                                                   |
//| INPUT:  no.                                                      |
//| OUTPUT: no.                                                      |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
void CChartObject::Detach()
  {
   m_chart_id=-1;
   m_window=-1;
   m_name=NULL;
   m_num_points=0;
  }
//+------------------------------------------------------------------+
//| Get the time coordinates of the point.                           |
//| INPUT:  point -number of point.                                  |
//| OUTPUT: date/time                                                |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
datetime CChartObject::Time(int point) const
  {
//--- checking
   if(m_chart_id==-1)      return(0);
   if(point>=m_num_points) return(0);
//---
   return(ObjectGetInteger(m_chart_id,m_name,OBJPROP_TIME,point));
  }
//+------------------------------------------------------------------+
//| Set the time coordinates of the point.                           |
//| INPUT:  point -number of point,                                  |
//|         time  -new date/time.                                    |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CChartObject::Time(int point,datetime time)
  {
//--- checking
   if(m_chart_id==-1)      return(false);
   if(point>=m_num_points) return(false);
//---
   return(ObjectSetInteger(m_chart_id,m_name,OBJPROP_TIME,time));
  }
//+------------------------------------------------------------------+
//| Get the price coordinates of the point.                          |
//| INPUT:  point -number of point.                                  |
//| OUTPUT: price.                                                   |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
double CChartObject::Price(int point) const
  {
//--- checking
   if(m_chart_id==-1)      return(EMPTY_VALUE);
   if(point>=m_num_points) return(EMPTY_VALUE);
//---
   return(ObjectGetDouble(m_chart_id,m_name,OBJPROP_PRICE,point));
  }
//+------------------------------------------------------------------+
//| Set the price coordinates of the point.                          |
//| INPUT:  point -number of point,                                  |
//|         price -new price.                                        |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CChartObject::Price(int point,double price)
  {
//--- checking
   if(m_chart_id==-1)      return(false);
   if(point>=m_num_points) return(false);
//---
   return(ObjectSetDouble(m_chart_id,m_name,OBJPROP_PRICE,price));
  }
//+------------------------------------------------------------------+
//| Get object color.                                                |
//| INPUT:  no.                                                      |
//| OUTPUT: color.                                                   |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
color CChartObject::Color() const
  {
//--- checking
   if(m_chart_id==-1) return(CLR_NONE);
//---
   return((color)ObjectGetInteger(m_chart_id,m_name,OBJPROP_COLOR));
  }
//+------------------------------------------------------------------+
//| Set object color.                                                |
//| INPUT:  new_color -new color.                                    |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CChartObject::Color(color new_color)
  {
//--- checking
   if(m_chart_id==-1) return(false);
//---
   return(ObjectSetInteger(m_chart_id,m_name,OBJPROP_COLOR,new_color));
  }
//+------------------------------------------------------------------+
//| Get object line style.                                           |
//| INPUT:  no.                                                      |
//| OUTPUT: line style.                                              |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
ENUM_LINE_STYLE CChartObject::Style() const
  {
//--- checking
   if(m_chart_id==-1) return(WRONG_VALUE);
//---
   return((ENUM_LINE_STYLE)ObjectGetInteger(m_chart_id,m_name,OBJPROP_STYLE));
  }
//+------------------------------------------------------------------+
//| Set object line style.                                           |
//| INPUT:  new_style -new line style.                               |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CChartObject::Style(ENUM_LINE_STYLE new_style)
  {
//--- checking
   if(m_chart_id==-1) return(false);
//---
   return(ObjectSetInteger(m_chart_id,m_name,OBJPROP_STYLE,new_style));
  }
//+------------------------------------------------------------------+
//| Get object line width.                                           |
//| INPUT:  no.                                                      |
//| OUTPUT: line width.                                              |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
int CChartObject::Width() const
  {
//--- checking
   if(m_chart_id==-1) return(-1);
//---
   return((int)ObjectGetInteger(m_chart_id,m_name,OBJPROP_WIDTH));
  }
//+------------------------------------------------------------------+
//| Set object line width.                                           |
//| INPUT:  new_width -new line width.                               |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CChartObject::Width(int new_width)
  {
//--- checking
   if(m_chart_id==-1) return(false);
//---
   return(ObjectSetInteger(m_chart_id,m_name,OBJPROP_WIDTH,new_width));
  }
//+------------------------------------------------------------------+
//| Get flag "Background".                                           |
//| INPUT:  no.                                                      |
//| OUTPUT: flag "Background".                                       |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CChartObject::Background() const
  {
//--- checking
   if(m_chart_id==-1) return(false);
//---
   return(ObjectGetInteger(m_chart_id,m_name,OBJPROP_BACK));
  }
//+------------------------------------------------------------------+
//| Set flag "Background".                                           |
//| INPUT:  new_back -new flag "Background".                         |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CChartObject::Background(bool new_back)
  {
//--- checking
   if(m_chart_id==-1) return(false);
//---
   return(ObjectSetInteger(m_chart_id,m_name,OBJPROP_BACK,new_back));
  }
//+------------------------------------------------------------------+
//| Get flag "Selected".                                             |
//| INPUT:  no.                                                      |
//| OUTPUT: flag "Selected".                                         |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CChartObject::Selected() const
  {
//--- checking
   if(m_chart_id==-1) return(false);
//---
   return(ObjectGetInteger(m_chart_id,m_name,OBJPROP_SELECTED));
  }
//+------------------------------------------------------------------+
//| Set flag "Selected".                                             |
//| INPUT:  new_sel -new flag "Selected".                            |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CChartObject::Selected(bool new_sel)
  {
//--- checking
   if(m_chart_id==-1) return(false);
//---
   return(ObjectSetInteger(m_chart_id,m_name,OBJPROP_SELECTED,new_sel));
  }
//+------------------------------------------------------------------+
//| Get flag "Selectable".                                           |
//| INPUT:  no.                                                      |
//| OUTPUT: flag "Selectable".                                       |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CChartObject::Selectable() const
  {
//--- checking
   if(m_chart_id==-1) return(false);
//---
   return(ObjectGetInteger(m_chart_id,m_name,OBJPROP_SELECTABLE));
  }
//+------------------------------------------------------------------+
//| Set flag "Selectable".                                           |
//| INPUT:  new_sel -new flag "Selectable".                          |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CChartObject::Selectable(bool new_sel)
  {
//--- checking
   if(m_chart_id==-1) return(false);
//---
   return(ObjectSetInteger(m_chart_id,m_name,OBJPROP_SELECTABLE,new_sel));
  }
//+------------------------------------------------------------------+
//| Get object comment.                                              |
//| INPUT:  no.                                                      |
//| OUTPUT: object comment.                                          |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
string CChartObject::Description() const
  {
//--- checking
   if(m_chart_id==-1) return("");
//---
   return(ObjectGetString(m_chart_id,m_name,OBJPROP_TEXT));
  }
//+------------------------------------------------------------------+
//| Set object comment.                                              |
//| INPUT:  new_text -new comment.                                   |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CChartObject::Description(const string new_text)
  {
//--- checking
   if(m_chart_id==-1) return(false);
//--- tuning
   if(new_text=="") 
      return(ObjectSetString(m_chart_id,m_name,OBJPROP_TEXT," "));
//---
   return(ObjectSetString(m_chart_id,m_name,OBJPROP_TEXT,new_text));
  }
//+------------------------------------------------------------------+
//| Get flags "Timeframes".                                          |
//| INPUT:  no.                                                      |
//| OUTPUT: flags "Timeframes".                                      |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
int CChartObject::Timeframes() const
  {
//--- checking
   if(m_chart_id==-1) return(0);
//---
   return((int)ObjectGetInteger(m_chart_id,m_name,OBJPROP_TIMEFRAMES));
  }
//+------------------------------------------------------------------+
//| Set flags "Timeframes".                                          |
//| INPUT:  timeframes -new flags "Timeframes".                      |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CChartObject::Timeframes(int timeframes)
  {
//--- checking
   if(m_chart_id==-1) return(false);
//---
   return(ObjectSetInteger(m_chart_id,m_name,OBJPROP_TIMEFRAMES,timeframes));
  }
//+------------------------------------------------------------------+
//| Get object create time.                                          |
//| INPUT:  no.                                                      |
//| OUTPUT: crate time.                                              |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
datetime CChartObject::CreateTime() const
  {
//--- checking
   if(m_chart_id==-1) return(0);
//---
   return(ObjectGetInteger(m_chart_id,m_name,OBJPROP_CREATETIME));
  }
//+------------------------------------------------------------------+
//| Get object number of levels.                                     |
//| INPUT:  no.                                                      |
//| OUTPUT: object number of levels.                                 |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
int CChartObject::LevelsCount() const
  {
//--- checking
   if(m_chart_id==-1) return(0);
//---
   return((int)ObjectGetInteger(m_chart_id,m_name,OBJPROP_LEVELS));
  }
//+------------------------------------------------------------------+
//| Set object number of levels.                                     |
//| INPUT:  new_count -new object number of levels.                  |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CChartObject::LevelsCount(int new_count)
  {
//--- checking
   if(m_chart_id==-1) return(false);
//---
   return(ObjectSetInteger(m_chart_id,m_name,OBJPROP_LEVELS,new_count));
  }
//+------------------------------------------------------------------+
//| Get level color.                                                 |
//| INPUT:  level -level number.                                     |
//| OUTPUT: level color.                                             |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
color CChartObject::LevelColor(int level) const
  {
//--- checking
   if(m_chart_id==-1)       return(CLR_NONE);
   if(level>=LevelsCount()) return(CLR_NONE);
//---
   return((color)ObjectGetInteger(m_chart_id,m_name,OBJPROP_LEVELCOLOR,level));
  }
//+------------------------------------------------------------------+
//| Set level color.                                                 |
//| INPUT:  level     -level number,                                 |
//|         new_color -new level color.                              |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CChartObject::LevelColor(int level,color new_color)
  {
//--- checking
   if(m_chart_id==-1)       return(false);
   if(level>=LevelsCount()) return(false);
//---
   return(ObjectSetInteger(m_chart_id,m_name,OBJPROP_LEVELCOLOR,level,new_color));
  }
//+------------------------------------------------------------------+
//| Get level line style.                                            |
//| INPUT:  level -level number.                                     |
//| OUTPUT: level line style.                                        |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
ENUM_LINE_STYLE CChartObject::LevelStyle(int level) const
  {
//--- checking
   if(m_chart_id==-1)       return(WRONG_VALUE);
   if(level>=LevelsCount()) return(WRONG_VALUE);
//---
   return((ENUM_LINE_STYLE)ObjectGetInteger(m_chart_id,m_name,OBJPROP_LEVELSTYLE,level));
  }
//+------------------------------------------------------------------+
//| Set level line style.                                            |
//| INPUT:  level     -level number,                                 |
//|         new_style -new level line style.                         |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CChartObject::LevelStyle(int level,ENUM_LINE_STYLE new_style)
  {
//--- checking
   if(m_chart_id==-1)       return(false);
   if(level>=LevelsCount()) return(false);
//---
   return(ObjectSetInteger(m_chart_id,m_name,OBJPROP_LEVELSTYLE,level,new_style));
  }
//+------------------------------------------------------------------+
//| Get level line width.                                            |
//| INPUT:  level -level number.                                     |
//| OUTPUT: level line width.                                        |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
int CChartObject::LevelWidth(int level) const
  {
//--- checking
   if(m_chart_id==-1)       return(-1);
   if(level>=LevelsCount()) return(-1);
//---
   return((int)ObjectGetInteger(m_chart_id,m_name,OBJPROP_LEVELWIDTH,level));
  }
//+------------------------------------------------------------------+
//| Set level line width.                                            |
//| INPUT:  level     -level number,                                 |
//|         new_width -new level line width.                         |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CChartObject::LevelWidth(int level,int new_width)
  {
//--- checking
   if(m_chart_id==-1)       return(false);
   if(level>=LevelsCount()) return(false);
//---
   return(ObjectSetInteger(m_chart_id,m_name,OBJPROP_LEVELWIDTH,level,new_width));
  }
//+------------------------------------------------------------------+
//| Get level value.                                                 |
//| INPUT:  level -level number.                                     |
//| OUTPUT: level value.                                             |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
double CChartObject::LevelValue(int level) const
  {
//--- checking
   if(m_chart_id==-1)       return(EMPTY_VALUE);
   if(level>=LevelsCount()) return(EMPTY_VALUE);
//---
   return(ObjectGetDouble(m_chart_id,m_name,OBJPROP_LEVELVALUE,level));
  }
//+------------------------------------------------------------------+
//| Set level value.                                                 |
//| INPUT:  level     -level number,                                 |
//|         new_value -new level value.                              |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CChartObject::LevelValue(int level,double new_value)
  {
//--- checking
   if(m_chart_id==-1)       return(false);
   if(level>=LevelsCount()) return(false);
//---
   return(ObjectSetDouble(m_chart_id,m_name,OBJPROP_LEVELVALUE,level,new_value));
  }
//+------------------------------------------------------------------+
//| Get level comment.                                               |
//| INPUT:  level -level number.                                     |
//| OUTPUT: level comment.                                           |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
string CChartObject::LevelDescription(int level) const
  {
//--- checking
   if(m_chart_id==-1)       return("");
   if(level>=LevelsCount()) return("");
//---
   return(ObjectGetString(m_chart_id,m_name,OBJPROP_LEVELTEXT,level));
  }
//+------------------------------------------------------------------+
//| Set level comment.                                               |
//| INPUT:  level    -level number,                                  |
//|         new_text -new level comment.                             |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CChartObject::LevelDescription(int level,const string new_text)
  {
//--- checking
   if(m_chart_id==-1)       return(false);
   if(level>=LevelsCount()) return(false);
//---
   return(ObjectSetString(m_chart_id,m_name,OBJPROP_LEVELTEXT,level,new_text));
  }
//+------------------------------------------------------------------+
//| Access functions long ObjectGetInteger(...).                     |
//| INPUT:  prop_id  -property idintifier,                           |
//|         modifier -property modifier.                             |
//| OUTPUT: the property value.                                      |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
long CChartObject::GetInteger(ENUM_OBJECT_PROPERTY_INTEGER prop_id,int modifier) const
  {
//--- checking
   if(m_chart_id==-1) return(0);
//---
   if(modifier==-1) return(ObjectGetInteger(m_chart_id,m_name,prop_id));
//---
   return(ObjectGetInteger(m_chart_id,m_name,prop_id,modifier));
  }
//+------------------------------------------------------------------+
//| Access functions bool ObjectGetInteger(...).                     |
//| INPUT:  prop_id  -property idintifier,                           |
//|         modifier -property modifier,                             |
//|         value    -variable for the property value.               |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CChartObject::GetInteger(ENUM_OBJECT_PROPERTY_INTEGER prop_id,int modifier,long& value) const
  {
//--- checking
   if(m_chart_id==-1) return(false);
//---
   return(ObjectGetInteger(m_chart_id,m_name,prop_id,modifier,value));
  }
//+------------------------------------------------------------------+
//| Access functions ObjectSetInteger(.,modifier,.).                 |
//| INPUT:  prop_id  -property idintifier,                           |
//|         modifier -property modifier,                             |
//|         value    -new the property value.                        |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CChartObject::SetInteger(ENUM_OBJECT_PROPERTY_INTEGER prop_id,int modifier,long value)
  {
//--- checking
   if(m_chart_id==-1) return(false);
//---
   return(ObjectSetInteger(m_chart_id,m_name,prop_id,modifier,value));
  }
//+------------------------------------------------------------------+
//| Access functions ObjectSetInteger(...).                          |
//| INPUT:  prop_id  -property idintifier,                           |
//|         value    -new the property value.                        |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CChartObject::SetInteger(ENUM_OBJECT_PROPERTY_INTEGER prop_id,long value)
  {
//--- checking
   if(m_chart_id==-1) return(false);
//---
   return(ObjectSetInteger(m_chart_id,m_name,prop_id,value));
  }
//+------------------------------------------------------------------+
//| Access functions double ObjectGetDouble(...).                    |
//| INPUT:  prop_id  -property idintifier,                           |
//|         modifier -property modifier.                             |
//| OUTPUT: the property value.                                      |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
double CChartObject::GetDouble(ENUM_OBJECT_PROPERTY_DOUBLE prop_id,int modifier) const
  {
//--- checking
   if(m_chart_id==-1) return(EMPTY_VALUE);
//---
   if(modifier==-1) return(ObjectGetDouble(m_chart_id,m_name,prop_id));
//---
   return(ObjectGetDouble(m_chart_id,m_name,prop_id,modifier));
  }
//+------------------------------------------------------------------+
//| Access functions bool ObjectGetDouble(...).                      |
//| INPUT:  prop_id  -property idintifier,                           |
//|         modifier -property modifier,                             |
//|         value    -variable for the property value.               |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CChartObject::GetDouble(ENUM_OBJECT_PROPERTY_DOUBLE prop_id,int modifier,double& value) const
  {
//--- checking
   if(m_chart_id==-1) return(false);
//---
   return(ObjectGetDouble(m_chart_id,m_name,prop_id,modifier,value));
  }
//+------------------------------------------------------------------+
//| Access functions ObjectSetDouble(.,modifier,.).                  |
//| INPUT:  prop_id  -property idintifier,                           |
//|         modifier -property modifier,                             |
//|         value    -new the property value.                        |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CChartObject::SetDouble(ENUM_OBJECT_PROPERTY_DOUBLE prop_id,int modifier,double value)
  {
//--- checking
   if(m_chart_id==-1) return(false);
//---
   return(ObjectSetDouble(m_chart_id,m_name,prop_id,modifier,value));
  }
//+------------------------------------------------------------------+
//| Access functions ObjectSetDouble(...).                           |
//| INPUT:  prop_id  -property idintifier,                           |
//|         value    -new the property value.                        |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CChartObject::SetDouble(ENUM_OBJECT_PROPERTY_DOUBLE prop_id,double value)
  {
//--- checking
   if(m_chart_id==-1) return(false);
//---
   return(ObjectSetDouble(m_chart_id,m_name,prop_id,value));
  }
//+------------------------------------------------------------------+
//| Access functions string ObjectGetString (...).                   |
//| INPUT:  prop_id  -property idintifier,                           |
//|         modifier -property modifier.                             |
//| OUTPUT: the property value.                                      |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
string CChartObject::GetString(ENUM_OBJECT_PROPERTY_STRING prop_id,int modifier) const
  {
//--- checking
   if(m_chart_id==-1) return("");
//---
   if(modifier==-1) return(ObjectGetString(m_chart_id,m_name,prop_id));
//---
   return(ObjectGetString(m_chart_id,m_name,prop_id,modifier));
  }
//+------------------------------------------------------------------+
//| Access functions bool ObjectGetString(...).                      |
//| INPUT:  prop_id  -property idintifier,                           |
//|         modifier -property modifier,                             |
//|         value    -variable for the property value.               |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CChartObject::GetString(ENUM_OBJECT_PROPERTY_STRING prop_id,int modifier,string& value) const
  {
//--- checking
   if(m_chart_id==-1) return(false);
//---
   return(ObjectGetString(m_chart_id,m_name,prop_id,modifier,value));
  }
//+------------------------------------------------------------------+
//| Access functions ObjectSetString(.,modifier,.).                  |
//| INPUT:  prop_id  -property idintifier,                           |
//|         modifier -property modifier,                             |
//|         value    -new the property value.                        |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CChartObject::SetString(ENUM_OBJECT_PROPERTY_STRING prop_id,int modifier,string value)
  {
//--- checking
   if(m_chart_id==-1) return(false);
//---
   return(ObjectSetString(m_chart_id,m_name,prop_id,modifier,value));
  }
//+------------------------------------------------------------------+
//| Access functions ObjectSetString(...).                           |
//| INPUT:  prop_id -property idintifier,                            |
//|         value   -new the property value.                         |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CChartObject::SetString(ENUM_OBJECT_PROPERTY_STRING prop_id,string value)
  {
//--- checking
   if(m_chart_id==-1) return(false);
//---
   return(ObjectSetString(m_chart_id,m_name,prop_id,value));
  }
//+------------------------------------------------------------------+
//| Object relative move.                                            |
//| INPUT:  d_time  -time offset,                                    |
//|         d_price -price offset.                                   |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CChartObject::ShiftObject(datetime d_time,double d_price)
  {
   bool resutl=true;
   int  i;
//--- checking
   if(m_chart_id==-1) return(false);
//--- move
   for(i=0;i<m_num_points;i++)
      resutl&=ShiftPoint(i,d_time,d_price);
//---
   return(resutl);
  }
//+------------------------------------------------------------------+
//| Point relative move.                                             |
//| INPUT:  point   -point number,                                   |
//|         d_time  -time offset,                                    |
//|         d_price -price offset.                                   |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CChartObject::ShiftPoint(int point,datetime d_time,double d_price)
  {
//--- checking
   if(m_chart_id==-1)      return(false);
   if(point>=m_num_points) return(false);
//--- move
   datetime time=ObjectGetInteger(m_chart_id,m_name,OBJPROP_TIME,point);
   double   price=ObjectGetDouble(m_chart_id,m_name,OBJPROP_PRICE,point);
//---
   return(ObjectMove(m_chart_id,m_name,point,time+d_time,price+d_price));
  }
//+------------------------------------------------------------------+
//| Writing object parameters to file.                               |
//| INPUT:  file_handle -handle previously opened for writing file.  |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CChartObject::Save(int file_handle)
  {
   int    i,len;
   int    levels;
   string str;
//--- checking
   if(file_handle<=0) return(false);
   if(m_chart_id==-1) return(false);
//--- writing
//--- writing begin marker - 0xFFFFFFFFFFFFFFFF
   if(FileWriteLong(file_handle,-1)!=sizeof(long)) return(false);
//--- writing object type
   if(FileWriteInteger(file_handle,Type(),INT_VALUE)!=INT_VALUE) return(false);
//--- writing object name
   str=ObjectGetString(m_chart_id,m_name,OBJPROP_NAME);
   len=StringLen(str);
   if(FileWriteInteger(file_handle,len,INT_VALUE)!=INT_VALUE) return(false);
   if(len!=0) if(FileWriteString(file_handle,str,len)!=len) return(false);
//--- writing object color
   if(FileWriteLong(file_handle,ObjectGetInteger(m_chart_id,m_name,OBJPROP_COLOR))!=sizeof(long)) return(false);
//--- writing object line style
   if(FileWriteInteger(file_handle,(int)ObjectGetInteger(m_chart_id,m_name,OBJPROP_STYLE))!=sizeof(int)) return(false);
//--- writing object line width
   if(FileWriteInteger(file_handle,(int)ObjectGetInteger(m_chart_id,m_name,OBJPROP_WIDTH))!=sizeof(int)) return(false);
//--- writing the property value "Background"
   if(FileWriteInteger(file_handle,(int)ObjectGetInteger(m_chart_id,m_name,OBJPROP_BACK),CHAR_VALUE)!=sizeof(char)) return(false);
//--- writing the property value "Selectable"
   if(FileWriteInteger(file_handle,(int)ObjectGetInteger(m_chart_id,m_name,OBJPROP_SELECTABLE),CHAR_VALUE)!=sizeof(char)) return(false);
//--- writing the property value "Timeframes"
   if(FileWriteInteger(file_handle,(int)ObjectGetInteger(m_chart_id,m_name,OBJPROP_TIMEFRAMES),INT_VALUE)!=sizeof(int)) return(false);
//--- writing comment
   str=ObjectGetString(m_chart_id,m_name,OBJPROP_TEXT);
   len=StringLen(str);
   if(FileWriteInteger(file_handle,len,INT_VALUE)!=INT_VALUE) return(false);
   if(len!=0) if(FileWriteString(file_handle,str,len)!=len) return(false);
//--- writing number of points
   if(FileWriteInteger(file_handle,m_num_points,INT_VALUE)!=INT_VALUE) return(false);
//--- writing points
   for(i=0;i<m_num_points;i++)
     {
      if(FileWriteLong(file_handle,ObjectGetInteger(m_chart_id,m_name,OBJPROP_TIME,i))!=sizeof(long)) return(false);
      if(FileWriteDouble(file_handle,ObjectGetDouble(m_chart_id,m_name,OBJPROP_PRICE,i))!=sizeof(double)) return(false);
     }
//--- writing number of levels
   levels=(int)ObjectGetInteger(m_chart_id,m_name,OBJPROP_LEVELS);
   if(FileWriteInteger(file_handle,levels,INT_VALUE)!=INT_VALUE) return(false);
//--- writing levels
   for(i=0;i<levels;i++)
     {
      //--- level color
      if(FileWriteLong(file_handle,ObjectGetInteger(m_chart_id,m_name,OBJPROP_LEVELCOLOR,i))!=sizeof(long)) return(false);
      //--- level line style
      if(FileWriteInteger(file_handle,(int)ObjectGetInteger(m_chart_id,m_name,OBJPROP_LEVELSTYLE,i))!=sizeof(int)) return(false);
      //--- level line width
      if(FileWriteInteger(file_handle,(int)ObjectGetInteger(m_chart_id,m_name,OBJPROP_LEVELWIDTH,i))!=sizeof(int)) return(false);
      //--- level value
      if(FileWriteDouble(file_handle,ObjectGetDouble(m_chart_id,m_name,OBJPROP_LEVELVALUE,i))!=sizeof(double)) return(false);
      //--- level name
      str=ObjectGetString(m_chart_id,m_name,OBJPROP_LEVELTEXT,i);
      len=StringLen(str);
      if(FileWriteInteger(file_handle,len,INT_VALUE)!=INT_VALUE) return(false);
      if(len!=0) if(FileWriteString(file_handle,str,len)!=len) return(false);
     }
//---
   return(true);
  }
//+------------------------------------------------------------------+
//| Reading object parameters from file.                             |
//| INPUT:  file_handle -handle previously opened for reading file.  |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CChartObject::Load(int file_handle)
  {
   int    i,len,num;
   string str;
//--- checking
   if(file_handle<=0) return(false);
   if(m_chart_id==-1) return(false);
//--- reading
//--- reading and check begin marker - 0xFFFFFFFFFFFFFFFF
   if(FileReadLong(file_handle)!=-1) return(false);
//--- reading and check object type
   if(FileReadInteger(file_handle,INT_VALUE)!=Type()) return(false);
//--- reading object name
   len=FileReadInteger(file_handle,INT_VALUE);
   if(len!=0) str=FileReadString(file_handle,len);
   else       str="";
   if(!ObjectSetString(m_chart_id,m_name,OBJPROP_NAME,str)) return(false);
//--- reading object color
   if(!ObjectSetInteger(m_chart_id,m_name,OBJPROP_COLOR,FileReadLong(file_handle))) return(false);
//--- reading object line style
   if(!ObjectSetInteger(m_chart_id,m_name,OBJPROP_STYLE,FileReadInteger(file_handle))) return(false);
//--- reading object line style
   if(!ObjectSetInteger(m_chart_id,m_name,OBJPROP_WIDTH,FileReadInteger(file_handle))) return(false);
//--- reading the property value "Background"
   if(!ObjectSetInteger(m_chart_id,m_name,OBJPROP_BACK,FileReadInteger(file_handle,CHAR_VALUE))) return(false);
//--- reading the property value "Selectable"
   if(!ObjectSetInteger(m_chart_id,m_name,OBJPROP_SELECTABLE,FileReadInteger(file_handle,CHAR_VALUE))) return(false);
//--- reading the property value "Timeframes"
   if(!ObjectSetInteger(m_chart_id,m_name,OBJPROP_TIMEFRAMES,FileReadInteger(file_handle,INT_VALUE))) return(false);
//--- reading comment
   len=FileReadInteger(file_handle,INT_VALUE);
   if(len!=0) str=FileReadString(file_handle,len);
   else       str="";
   if(!ObjectSetString(m_chart_id,m_name,OBJPROP_TEXT,str)) return(false);
//--- reading number of point
   num=FileReadInteger(file_handle,INT_VALUE);
//--- reading points
   if(num!=0)
     {
      for(i=0;i<num;i++)
        {
         if(!ObjectSetInteger(m_chart_id,m_name,OBJPROP_TIME,i,FileReadLong(file_handle))) return(false);
         if(!ObjectSetDouble(m_chart_id,m_name,OBJPROP_PRICE,i,FileReadDouble(file_handle))) return(false);
        }
     }
//--- reading number of levels
   num=FileReadInteger(file_handle,INT_VALUE);
   if(!ObjectSetInteger(m_chart_id,m_name,OBJPROP_LEVELS,0,FileReadLong(file_handle))) return(false);
//--- reading levels
   if(num!=0)
     {
      for(i=0;i<num;i++)
        {
         //--- level color
         if(!ObjectSetInteger(m_chart_id,m_name,OBJPROP_LEVELCOLOR,i,FileReadLong(file_handle))) return(false);
         //--- levelline style
         if(!ObjectSetInteger(m_chart_id,m_name,OBJPROP_LEVELSTYLE,i,FileReadInteger(file_handle))) return(false);
         //--- level line width
         if(!ObjectSetInteger(m_chart_id,m_name,OBJPROP_LEVELWIDTH,i,FileReadInteger(file_handle))) return(false);
         //--- level value
         if(!ObjectSetDouble(m_chart_id,m_name,OBJPROP_LEVELVALUE,i,FileReadDouble(file_handle))) return(false);
         //--- level name
         len=FileReadInteger(file_handle,INT_VALUE);
         if(len!=0) str=FileReadString(file_handle,len);
         else       str="";
         if(!ObjectSetString(m_chart_id,m_name,OBJPROP_LEVELTEXT,i,str)) return(false);
        }
     }
//---
   return(true);
  }
//+------------------------------------------------------------------+
