//+------------------------------------------------------------------+
//|                                          ChartObjectSubChart.mqh |
//|                        Copyright 2010, MetaQuotes Software Corp. |
//|                                        http://www.metaquotes.net |
//|                                              Revision 2010.02.22 |
//+------------------------------------------------------------------+
#include "ChartObject.mqh"
//+------------------------------------------------------------------+
//| Class CChartObjectSubChart.                                      |
//| Appointment: Class chart object "SubChart".                      |
//|              Derives from class CChartObject.                    |
//+------------------------------------------------------------------+
class CChartObjectSubChart : public CChartObject
  {
public:
   //--- methods access to properties object
   int               X_Distance() const;
   bool              X_Distance(int X);
   int               Y_Distance() const;
   bool              Y_Distance(int Y);
   ENUM_BASE_CORNER  Corner() const;
   bool              Corner(ENUM_BASE_CORNER corner);
   int               X_Size() const;
   bool              X_Size(int size);
   int               Y_Size() const;
   bool              Y_Size(int size);
   string            Symbol() const;
   bool              Symbol(string symbol);
   int               Period() const;
   bool              Period(int period);
   int               Scale() const;
   bool              Scale(int scale);
   bool              DateScale() const;
   bool              DateScale(bool scale);
   bool              PriceScale() const;
   bool              PriceScale(bool scale);
   //--- change time/price coordinates blocked
   bool              Time(datetime time) { return(false); }
   bool              Price(double price) { return(false); }
   //--- method create object
   bool              Create(long chart_id,string name,int window,int X,int Y,int sizeX,int sizeY);
   //--- method of identifying the object
   virtual int       Type() const { return(OBJ_CHART); }
   //--- methods for working with files
   virtual bool      Save(int file_handle);
   virtual bool      Load(int file_handle);
  };
//+------------------------------------------------------------------+
//| Create object "SubChart".                                        |
//| INPUT:  chart_id-chart idintifier,                               |
//|         name    -object name,                                    |
//|         window  -subwindow number,                               |
//|         X       -distance X from anchor point to base corner,    |
//|         Y       -distance Y from anchor point to base corner,    |
//|         sizeX   -size X,                                         |
//|         sizeY   -size Y.                                         |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CChartObjectSubChart::Create(long chart_id,string name,int window,int X,int Y,int sizeX,int sizeY)
  {
   bool result=ObjectCreate(chart_id,name,(ENUM_OBJECT)Type(),window,0,0,0);
//---
   if(result) result&=Attach(chart_id,name,window,1);
   result&=X_Distance(X);
   result&=Y_Distance(Y);
   result&=X_Size(sizeX);
   result&=Y_Size(sizeY);
//---
   return(result);
  }
//+------------------------------------------------------------------+
//| Get distance X.                                                  |
//| INPUT:  no.                                                      |
//| OUTPUT: distance X.                                              |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
int CChartObjectSubChart::X_Distance() const
  {
//--- checking
   if(m_chart_id==-1) return(0);
//---
   return((int)ObjectGetInteger(m_chart_id,m_name,OBJPROP_XDISTANCE));
  }
//+------------------------------------------------------------------+
//| Set distance X.                                                  |
//| INPUT:  X -new distance.                                         |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CChartObjectSubChart::X_Distance(int X)
  {
//--- checking
   if(m_chart_id==-1) return(false);
//---
   return(ObjectSetInteger(m_chart_id,m_name,OBJPROP_XDISTANCE,X));
  }
//+------------------------------------------------------------------+
//| Get distance Y.                                                  |
//| INPUT:  no.                                                      |
//| OUTPUT: distance Y.                                              |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
int CChartObjectSubChart::Y_Distance() const
  {
//--- checking
   if(m_chart_id==-1) return(0);
//---
   return((int)ObjectGetInteger(m_chart_id,m_name,OBJPROP_YDISTANCE));
  }
//+------------------------------------------------------------------+
//| Set distance Y.                                                  |
//| INPUT:  Y -new distance.                                         |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CChartObjectSubChart::Y_Distance(int Y)
  {
//--- checking
   if(m_chart_id==-1) return(false);
//---
   return(ObjectSetInteger(m_chart_id,m_name,OBJPROP_YDISTANCE,Y));
  }
//+------------------------------------------------------------------+
//| Get base corner.                                                 |
//| INPUT:  no.                                                      |
//| OUTPUT: base corner.                                             |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
ENUM_BASE_CORNER CChartObjectSubChart::Corner() const
  {
//--- checking
   if(m_chart_id==-1) return(WRONG_VALUE);
//---
   return((ENUM_BASE_CORNER)ObjectGetInteger(m_chart_id,m_name,OBJPROP_CORNER));
  }
//+------------------------------------------------------------------+
//| Set base corner.                                                 |
//| INPUT:  corner -new base corner.                                 |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CChartObjectSubChart::Corner(ENUM_BASE_CORNER corner)
  {
//--- checking
   if(m_chart_id==-1) return(false);
//---
   return(ObjectSetInteger(m_chart_id,m_name,OBJPROP_CORNER,corner));
  }
//+------------------------------------------------------------------+
//| Get size X.                                                      |
//| INPUT:  no.                                                      |
//| OUTPUT: size X.                                                  |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
int CChartObjectSubChart::X_Size() const
  {
//--- checking
   if(m_chart_id==-1) return(0);
//---
   return((int)ObjectGetInteger(m_chart_id,m_name,OBJPROP_XSIZE));
  }
//+------------------------------------------------------------------+
//| Set size X.                                                      |
//| INPUT:  size -new size.                                          |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CChartObjectSubChart::X_Size(int size)
  {
//--- checking
   if(m_chart_id==-1) return(false);
//---
   return(ObjectSetInteger(m_chart_id,m_name,OBJPROP_XSIZE,size));
  }
//+------------------------------------------------------------------+
//| Get size Y.                                                      |
//| INPUT:  no.                                                      |
//| OUTPUT: size Y.                                                  |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
int CChartObjectSubChart::Y_Size() const
  {
//--- checking
   if(m_chart_id==-1) return(0);
//---
   return((int)ObjectGetInteger(m_chart_id,m_name,OBJPROP_YSIZE));
  }
//+------------------------------------------------------------------+
//| Set size Y.                                                      |
//| INPUT:  size -new size.                                          |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CChartObjectSubChart::Y_Size(int size)
  {
//--- checking
   if(m_chart_id==-1) return(false);
//---
   return(ObjectSetInteger(m_chart_id,m_name,OBJPROP_YSIZE,size));
  }
//+------------------------------------------------------------------+
//| Get chart symbol.                                                |
//| INPUT:  no.                                                      |
//| OUTPUT: chart symbol.                                            |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
string CChartObjectSubChart::Symbol() const
  {
//--- checking
   if(m_chart_id==-1) return("");
//---
   return(ObjectGetString(m_chart_id,m_name,OBJPROP_SYMBOL));
  }
//+------------------------------------------------------------------+
//| Set chart symbol.                                                |
//| INPUT:  symbol -new chart symbol.                                |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CChartObjectSubChart::Symbol(string symbol)
  {
//--- checking
   if(m_chart_id==-1) return(false);
//---
   return(ObjectSetString(m_chart_id,m_name,OBJPROP_SYMBOL,symbol));
  }
//+------------------------------------------------------------------+
//| Get chart period.                                                |
//| INPUT:  no.                                                      |
//| OUTPUT: chart period.                                            |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
int CChartObjectSubChart::Period() const
  {
//--- checking
   if(m_chart_id==-1) return(0);
//---
   return((int)ObjectGetInteger(m_chart_id,m_name,OBJPROP_PERIOD));
  }
//+------------------------------------------------------------------+
//| Set chart period.                                                |
//| INPUT:  period -new chart period.                                |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CChartObjectSubChart::Period(int period)
  {
//--- checking
   if(m_chart_id==-1) return(false);
//---
   return(ObjectSetInteger(m_chart_id,m_name,OBJPROP_PERIOD,period));
  }
//+------------------------------------------------------------------+
//| Get chart scale.                                                 |
//| INPUT:  no.                                                      |
//| OUTPUT: chart scale.                                             |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
int CChartObjectSubChart::Scale() const
  {
//--- checking
   if(m_chart_id==-1) return(-1);
//---
   return((int)ObjectGetInteger(m_chart_id,m_name,OBJPROP_CHART_SCALE));
  }
//+------------------------------------------------------------------+
//| Set chart scale.                                                 |
//| INPUT:  scale -new chart scale.                                  |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CChartObjectSubChart::Scale(int scale)
  {
//--- checking
   if(m_chart_id==-1) return(false);
//---
   return(ObjectSetInteger(m_chart_id,m_name,OBJPROP_CHART_SCALE,scale));
  }
//+------------------------------------------------------------------+
//| Get flag "time scale".                                           |
//| INPUT:  no.                                                      |
//| OUTPUT: flag "time scale".                                       |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CChartObjectSubChart::DateScale() const
  {
//--- checking
   if(m_chart_id==-1) return(false);
//---
   return(ObjectGetInteger(m_chart_id,m_name,OBJPROP_DATE_SCALE));
  }
//+------------------------------------------------------------------+
//| Set flag "time scale".                                           |
//| INPUT:  new_ray -new flag "time scale".                          |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CChartObjectSubChart::DateScale(bool scale)
  {
//--- checking
   if(m_chart_id==-1) return(false);
//---
   return(ObjectSetInteger(m_chart_id,m_name,OBJPROP_DATE_SCALE,scale));
  }
//+------------------------------------------------------------------+
//| Get flag "price scale".                                          |
//| INPUT:  no.                                                      |
//| OUTPUT: flag "price scale".                                      |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CChartObjectSubChart::PriceScale() const
  {
//--- checking
   if(m_chart_id==-1) return(false);
//---
   return(ObjectGetInteger(m_chart_id,m_name,OBJPROP_PRICE_SCALE));
  }
//+------------------------------------------------------------------+
//| Set flag "price scale".                                          |
//| INPUT:  new_ray -new flag "price scale".                         |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CChartObjectSubChart::PriceScale(bool scale)
  {
//--- checking
   if(m_chart_id==-1) return(false);
//---
   return(ObjectSetInteger(m_chart_id,m_name,OBJPROP_PRICE_SCALE,scale));
  }
//+------------------------------------------------------------------+
//| Writing object parameters to file.                               |
//| INPUT:  file_handle -handle previously opened for writing file.  |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CChartObjectSubChart::Save(int file_handle)
  {
   bool   resutl;
   int    len;
   string str;
//--- checking
   if(file_handle<=0) return(false);
   if(m_chart_id==-1) return(false);
//--- writing
   resutl=CChartObject::Save(file_handle);
   if(resutl)
     {
      //--- writing the property value "distance X"
      if(FileWriteInteger(file_handle,(int)ObjectGetInteger(m_chart_id,m_name,OBJPROP_XDISTANCE),INT_VALUE)!=sizeof(int)) return(false);
      //--- writing the property value "distance Y"
      if(FileWriteInteger(file_handle,(int)ObjectGetInteger(m_chart_id,m_name,OBJPROP_YDISTANCE),INT_VALUE)!=sizeof(int)) return(false);
      //--- writing the property value "corner"
      if(FileWriteInteger(file_handle,(int)ObjectGetInteger(m_chart_id,m_name,OBJPROP_CORNER),INT_VALUE)!=sizeof(int)) return(false);
      //--- writing the property value "size X"
      if(FileWriteInteger(file_handle,(int)ObjectGetInteger(m_chart_id,m_name,OBJPROP_XSIZE),INT_VALUE)!=sizeof(int)) return(false);
      //--- writing the property value "size Y"
      if(FileWriteInteger(file_handle,(int)ObjectGetInteger(m_chart_id,m_name,OBJPROP_YSIZE),INT_VALUE)!=sizeof(int)) return(false);
      //--- writing the property value "symbol"
      str=ObjectGetString(m_chart_id,m_name,OBJPROP_SYMBOL);
      len=StringLen(str);
      if(FileWriteInteger(file_handle,len,INT_VALUE)!=INT_VALUE) return(false);
      if(len!=0) if(FileWriteString(file_handle,str,len)!=len) return(false);
      //--- writing the property value "period"
      if(FileWriteInteger(file_handle,(int)ObjectGetInteger(m_chart_id,m_name,OBJPROP_PERIOD),INT_VALUE)!=sizeof(int)) return(false);
      //--- writing the property value "scale"
      if(FileWriteDouble(file_handle,ObjectGetDouble(m_chart_id,m_name,OBJPROP_SCALE))!=sizeof(double)) return(false);
      //--- writing the property value "time scale"
      if(FileWriteInteger(file_handle,(int)ObjectGetInteger(m_chart_id,m_name,OBJPROP_DATE_SCALE),CHAR_VALUE)!=sizeof(char)) return(false);
      //--- writing the property value "price scale"
      if(FileWriteInteger(file_handle,(int)ObjectGetInteger(m_chart_id,m_name,OBJPROP_PRICE_SCALE),CHAR_VALUE)!=sizeof(char)) return(false);
     }
//---
   return(resutl);
  }
//+------------------------------------------------------------------+
//| Reading object parameters from file.                             |
//| INPUT:  file_handle -handle previously opened for reading file.  |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CChartObjectSubChart::Load(int file_handle)
  {
   bool   resutl;
   int    len;
   string str;
//--- checking
   if(file_handle<=0) return(false);
   if(m_chart_id==-1) return(false);
//--- reading
   resutl=CChartObject::Load(file_handle);
   if(resutl)
     {
      //--- reading the property value "distance X"
      if(!ObjectSetInteger(m_chart_id,m_name,OBJPROP_XDISTANCE,FileReadInteger(file_handle,INT_VALUE))) return(false);
      //--- reading the property value "distance Y"
      if(!ObjectSetInteger(m_chart_id,m_name,OBJPROP_YDISTANCE,FileReadInteger(file_handle,INT_VALUE))) return(false);
      //--- reading the property value "corner"
      if(!ObjectSetInteger(m_chart_id,m_name,OBJPROP_CORNER,FileReadInteger(file_handle,INT_VALUE))) return(false);
      //--- reading the property value "size X"
      if(!ObjectSetInteger(m_chart_id,m_name,OBJPROP_XSIZE,FileReadInteger(file_handle,INT_VALUE))) return(false);
      //--- reading the property value "size Y"
      if(!ObjectSetInteger(m_chart_id,m_name,OBJPROP_YSIZE,FileReadInteger(file_handle,INT_VALUE))) return(false);
      //--- reading the property value "symbol"
      len=FileReadInteger(file_handle,INT_VALUE);
      if(len!=0) str=FileReadString(file_handle,len);
      else       str="";
      if(!ObjectSetString(m_chart_id,m_name,OBJPROP_SYMBOL,str)) return(false);
      //--- reading the property value "period"
      if(!ObjectSetInteger(m_chart_id,m_name,OBJPROP_PERIOD,FileReadInteger(file_handle,INT_VALUE))) return(false);
      //--- reading the property value "scale"
      if(!ObjectSetDouble(m_chart_id,m_name,OBJPROP_SCALE,FileReadDatetime(file_handle))) return(false);
      //--- reading the property value "time scale"
      if(!ObjectSetInteger(m_chart_id,m_name,OBJPROP_DATE_SCALE,FileReadInteger(file_handle,CHAR_VALUE))) return(false);
      //--- reading the property value "price scale"
      if(!ObjectSetInteger(m_chart_id,m_name,OBJPROP_PRICE_SCALE,FileReadInteger(file_handle,CHAR_VALUE))) return(false);
     }
//---
   return(resutl);
  }
//+------------------------------------------------------------------+
