//+------------------------------------------------------------------+
//|                                         ChartObjectsChannels.mqh |
//|                        Copyright 2010, MetaQuotes Software Corp. |
//|                                        http://www.metaquotes.net |
//|                                              Revision 2010.02.22 |
//+------------------------------------------------------------------+
//| All channels.                                                    |
//+------------------------------------------------------------------+
#include "ChartObjectsLines.mqh"
//+------------------------------------------------------------------+
//| Class CChartObjectChannel.                                       |
//| Appointment: Class chart object "Equidistant channel".           |
//|              Derives from class CChartObjectTrend.               |
//+------------------------------------------------------------------+
class CChartObjectChannel : public CChartObjectTrend
  {
public:
   //--- method create object
   bool              Create(long chart_id,string name,int window,datetime time1,double price1,datetime time2,double price2,datetime time3,double price3);
   //--- method of identifying the object
   virtual int       Type() const { return(OBJ_CHANNEL); }
  };
//+------------------------------------------------------------------+
//| Create object "Equidistant channel".                             |
//| INPUT:  chart_id-chart idintifier,                               |
//|         name    -object name,                                    |
//|         window  -subwindow number,                               |
//|         time1   -first coordinate time,                          |
//|         price1  -first coordinate price,                         |
//|         time2   -second coordinate time,                         |
//|         price2  -second coordinate price,                        |
//|         time3   -third coordinate time,                          |
//|         price3  -third coordinate price.                         |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CChartObjectChannel::Create(long chart_id,string name,int window,datetime time1,double price1,datetime time2,double price2,datetime time3,double price3)
  {
   bool result=ObjectCreate(chart_id,name,OBJ_CHANNEL,window,time1,price1,time2,price2,time3,price3);
   if(result) result&=Attach(chart_id,name,window,3);
//---
   return(result);
  }
//+------------------------------------------------------------------+
//| Class CChartObjectStdDevChannel.                                 |
//| Appointment: Class chart object "Channel standard deviations".   |
//|              Derives from class CChartObjectTrend.               |
//+------------------------------------------------------------------+
class CChartObjectStdDevChannel : public CChartObjectTrend
  {
public:
   //--- methods access to properties object
   double            Deviations() const;
   bool              Deviations(double deviation);
   //--- method create object
   bool              Create(long chart_id,string name,int window,datetime time1,datetime time2,double deviation);
   //--- method of identifying the object
   virtual int       Type() const { return(OBJ_STDDEVCHANNEL); }
   //--- methods for working with files
   virtual bool      Save(int file_handle);
   virtual bool      Load(int file_handle);
  };
//+------------------------------------------------------------------+
//| Create object "Channel standard deviations".                     |
//| INPUT:  chart_id-chart idintifier,                               |
//|         name    -object name,                                    |
//|         window  -subwindow number,                               |
//|         time1   -first coordinate time,                          |
//|         time2   -second coordinate time,                         |
//|         deviation-deviation.                                     |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CChartObjectStdDevChannel::Create(long chart_id,string name,int window,datetime time1,datetime time2,double deviation)
  {
   bool result=ObjectCreate(chart_id,name,OBJ_STDDEVCHANNEL,window,time1,0.0,time2,0.0);
   if(result)
     {
      result&=Attach(chart_id,name,window,2);
      result&=Deviations(deviation);
     }
//---
   return(result);
  }
//+------------------------------------------------------------------+
//| Get the property value "Deviations".                             |
//| INPUT:  no.                                                      |
//| OUTPUT: the property value "Deviations".                         |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
double CChartObjectStdDevChannel::Deviations() const
  {
//--- checking
   if(m_chart_id==-1) return(EMPTY_VALUE);
//---
   return(ObjectGetDouble(m_chart_id,m_name,OBJPROP_DEVIATION));
  }
//+------------------------------------------------------------------+
//| Set the property value "Deviations".                             |
//| INPUT:  deviations -new the property value "Deviations".         |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CChartObjectStdDevChannel::Deviations(double deviation)
  {
//--- checking
   if(m_chart_id==-1) return(false);
//---
   return(ObjectSetDouble(m_chart_id,m_name,OBJPROP_DEVIATION,deviation));
  }
//+------------------------------------------------------------------+
//| Writing object parameters to file.                               |
//| INPUT:  file_handle -handle previously opened for writing file.  |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CChartObjectStdDevChannel::Save(int file_handle)
  {
   bool resutl;
//--- checking
   if(file_handle<=0) return(false);
   if(m_chart_id==-1) return(false);
//--- writing
   resutl=CChartObjectTrend::Save(file_handle);
   if(resutl)
     {
      //--- writing the property value "Deviations"
      if(FileWriteDouble(file_handle,ObjectGetDouble(m_chart_id,m_name,OBJPROP_DEVIATION))!=sizeof(double)) return(false);
     }
//---
   return(resutl);
  }
//+------------------------------------------------------------------+
//| Reading object parameters from file.                             |
//| INPUT:  file_handle -handle previously opened for reading file.  |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CChartObjectStdDevChannel::Load(int file_handle)
  {
   bool resutl;
//--- checking
   if(file_handle<=0) return(false);
   if(m_chart_id==-1) return(false);
//--- reading
   resutl=CChartObjectTrend::Load(file_handle);
   if(resutl)
     {
      //--- reading the property value "Deviations"
      if(!ObjectSetDouble(m_chart_id,m_name,OBJPROP_DEVIATION,FileReadDouble(file_handle))) return(false);
     }
//---
   return(resutl);
  }
//+------------------------------------------------------------------+
//| Class CChartObjectRegression.                                    |
//| Appointment: Class chart object "Channel linear regression"      |
//|              Derives from class CChartObjectTrend.               |
//+------------------------------------------------------------------+
class CChartObjectRegression : public CChartObjectTrend
  {
public:
   //--- method create object
   bool              Create(long chart_id,string name,int window,datetime time1,datetime time2);
   //--- method of identifying the object
   virtual int       Type() const { return(OBJ_REGRESSION); }
  };
//+------------------------------------------------------------------+
//| Create object "Channel linear regression".                       |
//| INPUT:  chart_id-chart idintifier,                               |
//|         name    -object name,                                    |
//|         window  -subwindow number,                               |
//|         time1   -first coordinate time,                          |
//|         time2   -second coordinate time.                         |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CChartObjectRegression::Create(long chart_id,string name,int window,datetime time1,datetime time2)
  {
   bool result=ObjectCreate(chart_id,name,OBJ_REGRESSION,window,time1,0.0,time2,0.0);
   if(result) result&=Attach(chart_id,name,window,2);
//---
   return(result);
  }
//+------------------------------------------------------------------+
//| Class CChartObjectPitchfork.                                     |
//| Appointment: Class chart object "Andrews pitchfork".             |
//|              Derives from class CChartObjectTrend.               |
//+------------------------------------------------------------------+
class CChartObjectPitchfork : public CChartObjectTrend
  {
public:
   //--- method create object
   bool              Create(long chart_id,string name,int window,datetime time1,double price1,datetime time2,double price2,datetime time3,double price3);
   //--- method of identifying the object
   virtual int       Type() const { return(OBJ_CHANNEL); }
  };
//+------------------------------------------------------------------+
//| Create object "Andrews pitchfork".                               |
//| INPUT:  chart_id-chart idintifier,                               |
//|         name    -object name,                                    |
//|         window  -subwindow number,                               |
//|         time1   -first coordinate time,                          |
//|         price1  -first coordinate prise,                         |
//|         time2   -second coordinate time,                         |
//|         price2  -second coordinate price,                        |
//|         time3   -third coordinate time,                          |
//|         price3  -third coordinate price.                         |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CChartObjectPitchfork::Create(long chart_id,string name,int window,datetime time1,double price1,datetime time2,double price2,datetime time3,double price3)
  {
   bool result=ObjectCreate(chart_id,name,OBJ_PITCHFORK,window,time1,price1,time2,price2,time3,price3);
   if(result) result&=Attach(chart_id,name,window,3);
//---
   return(result);
  }
//+------------------------------------------------------------------+
