//+------------------------------------------------------------------+
//|                                          ChartObjectsElliott.mqh |
//|                        Copyright 2010, MetaQuotes Software Corp. |
//|                                        http://www.metaquotes.net |
//|                                              Revision 2010.02.22 |
//+------------------------------------------------------------------+
//| All instruments Elliott.                                         |
//+------------------------------------------------------------------+
#include "ChartObject.mqh"
//+------------------------------------------------------------------+
//| Class CChartObjectElliottWave3.                                  |
//| Appointment: Class chart object "ElliottCorrectiveWave".         |
//|              Derives from class CChartObject.                    |
//+------------------------------------------------------------------+
class CChartObjectElliottWave3 : public CChartObject
  {
public:
   //--- methods access to properties object
   ENUM_ELLIOT_WAVE_DEGREE Degree() const;
   bool              Degree(ENUM_ELLIOT_WAVE_DEGREE degree);
   bool              Lines() const;
   bool              Lines(bool lines);
   //--- method create object
   bool              Create(long chart_id,string name,int window,datetime time1,double price1,datetime time2,double price2,datetime time3,double price3);
   //--- method of identifying the object
   virtual int       Type() const { return(OBJ_ELLIOTWAVE3); }
   //--- methods for working with files
   virtual bool      Save(int file_handle);
   virtual bool      Load(int file_handle);
  };
//+------------------------------------------------------------------+
//| Create object "ElliottCorrectiveWave".                           |
//| INPUT:  chart_id-chart idintifier,                               |
//|         name    -object name,                                    |
//|         window  -subwindow number,                               |
//|         time1   -first coordinate time,                          |
//|         price1  -first coordinate price,                         |
//|         time2   -second coordinate time,                         |
//|         price2  -second coordinate price,                        |
//|         time3   -third coordinate time,                          |
//|         price3  -third coordinate price.                         |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CChartObjectElliottWave3::Create(long chart_id,string name,int window,datetime time1,double price1,datetime time2,double price2,datetime time3,double price3)
  {
   bool result=ObjectCreate(chart_id,name,OBJ_ELLIOTWAVE3,window,time1,price1,time2,price2,time3,price3);
//---
   if(result) result&=Attach(chart_id,name,window,3);
//---
   return(result);
  }
//+------------------------------------------------------------------+
//| Get the property value "Degree".                                 |
//| INPUT:  no.                                                      |
//| OUTPUT: the property value "Degree".                             |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
ENUM_ELLIOT_WAVE_DEGREE CChartObjectElliottWave3::Degree() const
  {
//--- checking
   if(m_chart_id==-1) return(WRONG_VALUE);
//---
   return((ENUM_ELLIOT_WAVE_DEGREE)ObjectGetInteger(m_chart_id,m_name,OBJPROP_DEGREE));
  }
//+------------------------------------------------------------------+
//| Set the property value "Degree".                                 |
//| INPUT:  degree -new the property value "Degree".                 |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CChartObjectElliottWave3::Degree(ENUM_ELLIOT_WAVE_DEGREE degree)
  {
//--- checking
   if(m_chart_id==-1) return(false);
//---
   return(ObjectSetInteger(m_chart_id,m_name,OBJPROP_DEGREE,degree));
  }
//+------------------------------------------------------------------+
//| Get the property value "Lines".                                  |
//| INPUT:  no.                                                      |
//| OUTPUT: the property value "Lines".                              |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CChartObjectElliottWave3::Lines() const
  {
//--- checking
   if(m_chart_id==-1) return(false);
//---
   return(ObjectGetInteger(m_chart_id,m_name,OBJPROP_DRAWLINES));
  }
//+------------------------------------------------------------------+
//| Set the property value "Lines".                                  |
//| INPUT:  lines -new the property value "Lines".                   |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CChartObjectElliottWave3::Lines(bool lines)
  {
//--- checking
   if(m_chart_id==-1) return(false);
//---
   return(ObjectSetInteger(m_chart_id,m_name,OBJPROP_DRAWLINES,lines));
  }
//+------------------------------------------------------------------+
//| Writing object parameters to file.                               |
//| INPUT:  file_handle -handle previously opened for writing file.  |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CChartObjectElliottWave3::Save(int file_handle)
  {
   bool resutl;
//--- checking
   if(file_handle<=0) return(false);
   if(m_chart_id==-1) return(false);
//--- writing
   resutl=CChartObject::Save(file_handle);
   if(resutl)
     {
      //--- writing the property value "Degree"
      if(FileWriteInteger(file_handle,(int)ObjectGetInteger(m_chart_id,m_name,OBJPROP_DEGREE),INT_VALUE)!=sizeof(int)) return(false);
      //--- writing the property value "Lines"
      if(FileWriteInteger(file_handle,(int)ObjectGetInteger(m_chart_id,m_name,OBJPROP_DRAWLINES),INT_VALUE)!=sizeof(int)) return(false);
     }
//---
   return(resutl);
  }
//+------------------------------------------------------------------+
//| Reading object parameters from file.                             |
//| INPUT:  file_handle -handle previously opened for reading file.  |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CChartObjectElliottWave3::Load(int file_handle)
  {
   bool resutl;
//--- checking
   if(file_handle<=0) return(false);
   if(m_chart_id==-1) return(false);
//--- reading
   resutl=CChartObject::Load(file_handle);
   if(resutl)
     {
      //--- reading the property value "Degree"
      if(!ObjectSetInteger(m_chart_id,m_name,OBJPROP_DEGREE,FileReadInteger(file_handle,INT_VALUE))) return(false);
      //--- reading the property value "Lines"
      if(!ObjectSetInteger(m_chart_id,m_name,OBJPROP_DRAWLINES,FileReadInteger(file_handle,INT_VALUE))) return(false);
     }
//---
   return(resutl);
  }
//+------------------------------------------------------------------+
//| Class CChartObjectElliottWave5.                                  |
//| Appointment: Class chart object "ElliottMotiveWave".             |
//|              Derives from class CChartObjectElliottWave3.        |
//+------------------------------------------------------------------+
class CChartObjectElliottWave5 : public CChartObjectElliottWave3
  {
public:
   //--- method create object
   bool              Create(long chart_id,string name,int window,datetime time1,double price1,datetime time2,double price2,datetime time3,double price3,datetime time4,double price4,datetime time5,double price5);
   //--- method of identifying the object
   virtual int       Type() const { return(OBJ_ELLIOTWAVE5); }
  };
//+------------------------------------------------------------------+
//| Create object "ElliottMotiveWave".                               |
//| INPUT:  chart_id-chart idintifier,                               |
//|         name    -object name,                                    |
//|         window  -subwindow number,                               |
//|         time1   -first coordinate time,                          |
//|         price1  -first coordinate price,                         |
//|         time2   -second coordinate time,                         |
//|         price2  -second coordinate price,                        |
//|         time3   -third coordinate time,                          |
//|         price3  -third coordinate price,                         |
//|         time4   -fourth coordinate time,                         |
//|         price4  -fourth coordinate price,                        |
//|         time5   -fifth coordinate time,                          |
//|         price5  -fifth coordinate price.                         |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CChartObjectElliottWave5::Create(long chart_id,string name,int window,datetime time1,double price1,datetime time2,double price2,datetime time3,double price3,datetime time4,double price4,datetime time5,double price5)
  {
   bool result=ObjectCreate(chart_id,name,OBJ_ELLIOTWAVE5,window,time1,price1,time2,price2,time3,price3,time4,price4,time5,price5);
//---
   if(result) result&=Attach(chart_id,name,window,5);
//---
   return(result);
  }
//+------------------------------------------------------------------+
