//+------------------------------------------------------------------+
//|                                             ChartObjectsFibo.mqh |
//|                        Copyright 2010, MetaQuotes Software Corp. |
//|                                        http://www.metaquotes.net |
//|                                              Revision 2010.02.22 |
//+------------------------------------------------------------------+
//| All instruments Fibonacci.                                       |
//+------------------------------------------------------------------+
#include "ChartObjectsLines.mqh"
//+------------------------------------------------------------------+
//| Class CChartObjectFibo.                                          |
//| Appointment: Class chart object "Fibonacci Lines".               |
//|              Derives from class CChartObjectTrend.               |
//+------------------------------------------------------------------+
class CChartObjectFibo : public CChartObjectTrend
  {
public:
   //--- method create object
   bool              Create(long chart_id,string name,int window,datetime time1,double price1,datetime time2,double price2);
   //--- method of identifying the object
   virtual int       Type() const { return(OBJ_FIBO); }
  };
//+------------------------------------------------------------------+
//| Create object "Fibonacci Lines".                                 |
//| INPUT:  chart_id-chart idintifier,                               |
//|         name    -object name,                                    |
//|         window  -subwindow number,                               |
//|         time1   -first coordinate time,                          |
//|         price1  -first coordinate price,                         |
//|         time2   -second coordinate time,                         |
//|         price2  -second coordinate price.                        |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CChartObjectFibo::Create(long chart_id,string name,int window,datetime time1,double price1,datetime time2,double price2)
  {
   bool result=ObjectCreate(chart_id,name,OBJ_FIBO,window,time1,price1,time2,price2);
//---
   if(result) result&=Attach(chart_id,name,window,2);
//---
   return(result);
  }
//+------------------------------------------------------------------+
//| Class CChartObjectFiboTimes.                                     |
//| Appointment: Class chart object "Fibonacci Time Zones"           |
//|              Derives from class CChartObject.                    |
//+------------------------------------------------------------------+
class CChartObjectFiboTimes : public CChartObject
  {
public:
   //--- method create object
   bool              Create(long chart_id,string name,int window,datetime time1,double price1,datetime time2,double price2);
   //--- method of identifying the object
   virtual int       Type() const { return(OBJ_FIBOTIMES); }
  };
//+------------------------------------------------------------------+
//| Create object "Fibonacci Time Zones".                            |
//| INPUT:  chart_id-chart idintifier,                               |
//|         name    -object name,                                    |
//|         window  -subwindow number,                               |
//|         time1   -first coordinate time,                          |
//|         price1  -first coordinate price,                         |
//|         time2   -second coordinate time,                         |
//|         price2  -second coordinate price.                        |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CChartObjectFiboTimes::Create(long chart_id,string name,int window,datetime time1,double price1,datetime time2,double price2)
  {
   bool result=ObjectCreate(chart_id,name,OBJ_FIBOTIMES,window,time1,price1,time2,price2);
//---
   if(result) result&=Attach(chart_id,name,window,2);
//---
   return(result);
  }
//+------------------------------------------------------------------+
//| Class CChartObjectFiboFan.                                       |
//| Appointment: Class chart object "Fibonacci Fan".                 |
//|              Derives from class CChartObject.                    |
//+------------------------------------------------------------------+
class CChartObjectFiboFan : public CChartObject
  {
public:
   //--- method create object
   bool              Create(long chart_id,string name,int window,datetime time1,double price1,datetime time2,double price2);
   //--- method of identifying the object
   virtual int       Type() const { return(OBJ_FIBOFAN); }
  };
//+------------------------------------------------------------------+
//| Create object "Fibonacci Fan".                                   |
//| INPUT:  chart_id-chart idintifier,                               |
//|         name    -object name,                                    |
//|         window  -subwindow number,                               |
//|         time1   -first coordinate time,                          |
//|         price1  -first coordinate price,                         |
//|         time2   -second coordinate time,                         |
//|         price2  -second coordinate price.                        |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CChartObjectFiboFan::Create(long chart_id,string name,int window,datetime time1,double price1,datetime time2,double price2)
  {
   bool result=ObjectCreate(chart_id,name,OBJ_FIBOFAN,window,time1,price1,time2,price2);
//---
   if(result) result&=Attach(chart_id,name,window,2);
//---
   return(result);
  }
//+------------------------------------------------------------------+
//| Class CChartObjectFiboArc.                                       |
//| Appointment: Class chart object "Fibonacci Arcs".                |
//|              Derives from class CChartObject.                    |
//+------------------------------------------------------------------+
class CChartObjectFiboArc : public CChartObject
  {
public:
   //--- methods access to properties object
   double            Scale() const;
   bool              Scale(double scale);
   bool              Ellipse() const;
   bool              Ellipse(bool ellipse);
   //--- method create object
   bool              Create(long chart_id,string name,int window,datetime time1,double price1,datetime time2,double price2,double scale);
   //--- method of identifying the object
   virtual int       Type() const { return(OBJ_FIBOARC); }
   //--- methods for working with files
   virtual bool      Save(int file_handle);
   virtual bool      Load(int file_handle);
  };
//+------------------------------------------------------------------+
//| Create object "Fibonacci Arcs".                                  |
//| INPUT:  chart_id-chart idintifier,                               |
//|         name    -object name,                                    |
//|         window  -subwindow number,                               |
//|         time1   -first coordinate time,                          |
//|         price1  -first coordinate price,                         |
//|         time2   -second coordinate time,                         |
//|         price2  -second coordinate price,                        |
//|         scale   -scale.                                          |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CChartObjectFiboArc::Create(long chart_id,string name,int window,datetime time1,double price1,datetime time2,double price2,double scale)
  {
   bool result=ObjectCreate(chart_id,name,OBJ_FIBOARC,window,time1,price1,time2,price2);
//---
   if(result)
     {
      result&=Attach(chart_id,name,window,2);
      result&=Scale(scale);
     }
//---
   return(result);
  }
//+------------------------------------------------------------------+
//| Get the property value "Scale".                                  |
//| INPUT:  no.                                                      |
//| OUTPUT: the property value "Scale".                              |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
double CChartObjectFiboArc::Scale() const
  {
//--- checking
   if(m_chart_id==-1) return(EMPTY_VALUE);
//---
   return(ObjectGetDouble(m_chart_id,m_name,OBJPROP_SCALE));
  }
//+------------------------------------------------------------------+
//| Set the property value "Scale".                                  |
//| INPUT:  scale -new the property value "Scale".                   |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CChartObjectFiboArc::Scale(double scale)
  {
//--- checking
   if(m_chart_id==-1) return(false);
//---
   return(ObjectSetDouble(m_chart_id,m_name,OBJPROP_SCALE,scale));
  }
//+------------------------------------------------------------------+
//| Get the property value "Ellipse".                                |
//| INPUT:  no.                                                      |
//| OUTPUT: the property value "Ellipse".                            |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CChartObjectFiboArc::Ellipse() const
  {
//--- checking
   if(m_chart_id==-1) return(false);
//---
   return(ObjectGetInteger(m_chart_id,m_name,OBJPROP_ELLIPSE));
  }
//+------------------------------------------------------------------+
//| Set the property value "Ellipse".                                |
//| INPUT:  new_ray -new the property value "Ellipse".               |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CChartObjectFiboArc::Ellipse(bool ellipse)
  {
//--- checking
   if(m_chart_id==-1) return(false);
//---
   return(ObjectSetInteger(m_chart_id,m_name,OBJPROP_ELLIPSE,ellipse));
  }
//+------------------------------------------------------------------+
//| Writing object parameters to file.                               |
//| INPUT:  file_handle -handle previously opened for writing file.  |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CChartObjectFiboArc::Save(int file_handle)
  {
   bool resutl;
//--- checking
   if(file_handle<=0) return(false);
   if(m_chart_id==-1) return(false);
//--- writing
   resutl=CChartObject::Save(file_handle);
   if(resutl)
     {
      //--- writing the property value "Scale"
      if(FileWriteDouble(file_handle,ObjectGetDouble(m_chart_id,m_name,OBJPROP_SCALE))!=sizeof(double)) return(false);
      //--- writing the property value "Ellipse"
      if(FileWriteInteger(file_handle,(int)ObjectGetInteger(m_chart_id,m_name,OBJPROP_ELLIPSE),CHAR_VALUE)!=sizeof(char)) return(false);
     }
//---
   return(resutl);
  }
//+------------------------------------------------------------------+
//| Reading object parameters from file.                             |
//| INPUT:  file_handle -handle previously opened for reading file.  |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CChartObjectFiboArc::Load(int file_handle)
  {
   bool resutl;
//--- checking
   if(file_handle<=0) return(false);
   if(m_chart_id==-1) return(false);
//--- reading
   resutl=CChartObject::Load(file_handle);
   if(resutl)
     {
      //--- reading the property value "Scale"
      if(!ObjectSetDouble(m_chart_id,m_name,OBJPROP_SCALE,FileReadDouble(file_handle))) return(false);
      //--- reading the property value "Ellipse"
      if(!ObjectSetInteger(m_chart_id,m_name,OBJPROP_ELLIPSE,FileReadInteger(file_handle,CHAR_VALUE))) return(false);
     }
//---
   return(resutl);
  }
//+------------------------------------------------------------------+
//| Class CChartObjectFiboChannel.                                   |
//| Appointment: Class chart object "Fibonacci Channel".             |
//|              Derives from class CChartObjectTrend.               |
//+------------------------------------------------------------------+
class CChartObjectFiboChannel : public CChartObjectTrend
  {
public:
   //--- method create object
   bool              Create(long chart_id,string name,int window,datetime time1,double price1,datetime time2,double price2,datetime time3,double price3);
   //--- method of identifying the object
   virtual int       Type() const { return(OBJ_FIBOCHANNEL); }
  };
//+------------------------------------------------------------------+
//| Create object "Fibonacci Channel".                               |
//| INPUT:  chart_id-chart idintifier,                               |
//|         name    -object name,                                    |
//|         window  -subwindow number,                               |
//|         time1   -first coordinate time,                          |
//|         price1  -first coordinate price,                         |
//|         time2   -second coordinate time,                         |
//|         price2  -second coordinate price,                        |
//|         time3   -third coordinate time,                          |
//|         price3  -third coordinate price.                         |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CChartObjectFiboChannel::Create(long chart_id,string name,int window,datetime time1,double price1,datetime time2,double price2,datetime time3,double price3)
  {
   bool result=ObjectCreate(chart_id,name,OBJ_FIBOCHANNEL,window,time1,price1,time2,price2,time3,price3);
//---
   if(result) result&=Attach(chart_id,name,window,3);
//---
   return(result);
  }
//+------------------------------------------------------------------+
//| Class CChartObjectFiboExpansion.                                 |
//| Appointment: Class chart object "Fibonacci Expansion".           |
//|              Derives from class CChartObjectTrend.               |
//+------------------------------------------------------------------+
class CChartObjectFiboExpansion : public CChartObjectTrend
  {
public:
   //--- method create object
   bool              Create(long chart_id,string name,int window,datetime time1,double price1,datetime time2,double price2,datetime time3,double price3);
   //--- method of identifying the object
   virtual int       Type() const { return(OBJ_EXPANSION); }
  };
//+------------------------------------------------------------------+
//| Create object "Fibonacci Expansion".                             |
//| INPUT:  chart_id-chart idintifier,                               |
//|         name    -object name,                                    |
//|         window  -subwindow number,                               |
//|         time1   -first coordinate time,                          |
//|         price1  -first coordinate price,                         |
//|         time2   -second coordinate time,                         |
//|         price2  -second coordinate price,                        |
//|         time3   -third coordinate time,                          |
//|         price3  -third coordinate price.                         |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CChartObjectFiboExpansion::Create(long chart_id,string name,int window,datetime time1,double price1,datetime time2,double price2,datetime time3,double price3)
  {
   bool result=ObjectCreate(chart_id,name,OBJ_EXPANSION,window,time1,price1,time2,price2,time3,price3);
//---
   if(result) result&=Attach(chart_id,name,window,3);
//---
   return(result);
  }
//+------------------------------------------------------------------+
