//+------------------------------------------------------------------+
//|                                             ChartObjectsGann.mqh |
//|                        Copyright 2010, MetaQuotes Software Corp. |
//|                                        http://www.metaquotes.net |
//|                                              Revision 2010.02.22 |
//+------------------------------------------------------------------+
//| All instruments Gunn.                                            |
//+------------------------------------------------------------------+
#include "ChartObjectsLines.mqh"
//+------------------------------------------------------------------+
//| Class CChartObjectGannLine.                                      |
//| Appointment: Class chart object "Gann Line".                     |
//|              Derives from class CChartObjectTrendByAngle.        |
//+------------------------------------------------------------------+
class CChartObjectGannLine : public CChartObjectTrendByAngle
  {
public:
   //--- methods access to properties object
   double            PipsPerBar() const;
   bool              PipsPerBar(double ppb);
   //--- method create object
   bool              Create(long chart_id,string name,int window,datetime time1,double price1,datetime time2,double ppb);
   //--- method of identifying the object
   virtual int       Type() const { return(OBJ_GANNLINE); }
   //--- methods for working with files
   virtual bool      Save(int file_handle);
   virtual bool      Load(int file_handle);
  };
//+------------------------------------------------------------------+
//| Create object "Gann Line".                                       |
//| INPUT:  chart_id-chart idintifier,                               |
//|         name    -object name,                                    |
//|         window  -subwindow number,                               |
//|         time1   -first coordinate time,                          |
//|         price1  -first coordinate price,                         |
//|         time2   -second coordinate time,                         |
//|         ppb     -scale.                                          |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CChartObjectGannLine::Create(long chart_id,string name,int window,datetime time1,double price1,datetime time2,double ppb)
  {
   bool result=ObjectCreate(chart_id,name,OBJ_GANNLINE,window,time1,price1,time2,0.0);
//---
   if(result)
     {
      result&=Attach(chart_id,name,window,2);
      result&=PipsPerBar(ppb);
     }
//---
   return(result);
  }
//+------------------------------------------------------------------+
//| Get the property value "PipsPerBar".                             |
//| INPUT:  no.                                                      |
//| OUTPUT: the property value "PipsPerBar".                         |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
double CChartObjectGannLine::PipsPerBar() const
  {
//--- checking
   if(m_chart_id==-1) return(EMPTY_VALUE);
//---
   return(ObjectGetDouble(m_chart_id,m_name,OBJPROP_SCALE));
  }
//+------------------------------------------------------------------+
//| Set the property value "PipsPerBar".                             |
//| INPUT:  deviations -new the property value "PipsPerBar".         |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CChartObjectGannLine::PipsPerBar(double ppb)
  {
//--- checking
   if(m_chart_id==-1) return(false);
//---
   return(ObjectSetDouble(m_chart_id,m_name,OBJPROP_SCALE,ppb));
  }
//+------------------------------------------------------------------+
//| Writing object parameters to file.                               |
//| INPUT:  file_handle -handle previously opened for writing file.  |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CChartObjectGannLine::Save(int file_handle)
  {
   bool resutl;
//--- checking
   if(file_handle<=0) return(false);
   if(m_chart_id==-1) return(false);
//--- writing
   resutl=CChartObjectTrend::Save(file_handle);
   if(resutl)
     {
      //--- writing the property value "PipsPerBar"
      if(FileWriteDouble(file_handle,ObjectGetDouble(m_chart_id,m_name,OBJPROP_SCALE))!=sizeof(double)) return(false);
     }
//---
   return(resutl);
  }
//+------------------------------------------------------------------+
//| Reading object parameters from file.                             |
//| INPUT:  file_handle -handle previously opened for reading file.  |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CChartObjectGannLine::Load(int file_handle)
  {
   bool resutl;
//--- checking
   if(file_handle<=0) return(false);
   if(m_chart_id==-1) return(false);
//--- reading
   resutl=CChartObjectTrend::Load(file_handle);
   if(resutl)
     {
      //--- reading the property value "PipsPerBar"
      if(!ObjectSetDouble(m_chart_id,m_name,OBJPROP_SCALE,FileReadDouble(file_handle))) return(false);
     }
//---
   return(resutl);
  }
//+------------------------------------------------------------------+
//| Class CChartObjectGannFan.                                       |
//| Appointment: Class chart object "Gann Fan".                      |
//|              Derives from class CChartObjectTrend.               |
//+------------------------------------------------------------------+
class CChartObjectGannFan : public CChartObjectTrend
  {
public:
   //--- methods access to properties object
   double            PipsPerBar() const;
   bool              PipsPerBar(double ppb);
   bool              Downtrend() const;
   bool              Downtrend(bool downtrend);
   //--- method create object
   bool              Create(long chart_id,string name,int window,datetime time1,double price1,datetime time2,double ppb);
   //--- method of identifying the object
   virtual int       Type() const { return(OBJ_GANNFAN); }
   //--- methods for working with files
   virtual bool      Save(int file_handle);
   virtual bool      Load(int file_handle);
  };
//+------------------------------------------------------------------+
//| Create object "Gann Fan".                                        |
//| INPUT:  chart_id-chart idintifier,                               |
//|         name    -object name,                                    |
//|         window  -subwindow number,                               |
//|         time1   -first coordinate time,                          |
//|         price1  -first coordinate price,                         |
//|         time2   -second coordinate time,                         |
//|         ppb     -scale.                                          |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CChartObjectGannFan::Create(long chart_id,string name,int window,datetime time1,double price1,datetime time2,double ppb)
  {
   bool result=ObjectCreate(chart_id,name,OBJ_GANNFAN,window,time1,price1,time2,0.0);
//---
   if(result)
     {
      result&=Attach(chart_id,name,window,2);
      result&=PipsPerBar(ppb);
     }
//---
   return(result);
  }
//+------------------------------------------------------------------+
//| Get the property value "PipsPerBar".                             |
//| INPUT:  no.                                                      |
//| OUTPUT: the property value "PipsPerBar".                         |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
double CChartObjectGannFan::PipsPerBar() const
  {
//--- checking
   if(m_chart_id==-1) return(EMPTY_VALUE);
//---
   return(ObjectGetDouble(m_chart_id,m_name,OBJPROP_SCALE));
  }
//+------------------------------------------------------------------+
//| Set the property value "PipsPerBar".                             |
//| INPUT:  deviations -new the property value "PipsPerBar".         |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CChartObjectGannFan::PipsPerBar(double ppb)
  {
//--- checking
   if(m_chart_id==-1) return(false);
//---
   return(ObjectSetDouble(m_chart_id,m_name,OBJPROP_SCALE,ppb));
  }
//+------------------------------------------------------------------+
//| Get the property value "Downtrend".                              |
//| INPUT:  no.                                                      |
//| OUTPUT: the property value "Downtrend".                          |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CChartObjectGannFan::Downtrend() const
  {
//--- checking
   if(m_chart_id==-1) return(false);
//---
   return(ObjectGetInteger(m_chart_id,m_name,OBJPROP_DIRECTION));
  }
//+------------------------------------------------------------------+
//| Set the property value "Downtrend".                              |
//| INPUT:  new_ray -new the property value "Downtrend".             |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CChartObjectGannFan::Downtrend(bool downtrend)
  {
//--- checking
   if(m_chart_id==-1) return(false);
//---
   return(ObjectSetInteger(m_chart_id,m_name,OBJPROP_DIRECTION,downtrend));
  }
//+------------------------------------------------------------------+
//| Writing object parameters to file.                               |
//| INPUT:  file_handle -handle previously opened for writing file.  |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CChartObjectGannFan::Save(int file_handle)
  {
   bool resutl;
//--- checking
   if(file_handle<=0) return(false);
   if(m_chart_id==-1) return(false);
//--- writing
   resutl=CChartObjectTrend::Save(file_handle);
   if(resutl)
     {
      //--- writing the property value "PipsPerBar"
      if(FileWriteDouble(file_handle,ObjectGetDouble(m_chart_id,m_name,OBJPROP_SCALE))!=sizeof(double)) return(false);
      //--- writing the property value "Downtrend"
      if(FileWriteInteger(file_handle,(int)ObjectGetInteger(m_chart_id,m_name,OBJPROP_DIRECTION),CHAR_VALUE)!=sizeof(char)) return(false);
     }
//---
   return(resutl);
  }
//+------------------------------------------------------------------+
//| Reading object parameters from file.                             |
//| INPUT:  file_handle -handle previously opened for reading file.  |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CChartObjectGannFan::Load(int file_handle)
  {
   bool resutl;
//--- checking
   if(file_handle<=0) return(false);
   if(m_chart_id==-1) return(false);
//--- reading
   resutl=CChartObjectTrend::Load(file_handle);
   if(resutl)
     {
      //--- reading the property value "PipsPerBar"
      if(!ObjectSetDouble(m_chart_id,m_name,OBJPROP_SCALE,FileReadDouble(file_handle))) return(false);
      //--- reading the property value "Downtrend"
      if(!ObjectSetInteger(m_chart_id,m_name,OBJPROP_DIRECTION,FileReadInteger(file_handle,CHAR_VALUE))) return(false);
     }
//---
   return(resutl);
  }
//+------------------------------------------------------------------+
//| Class CChartObjectGannGrid.                                      |
//| Appointment: Class chart object "Gann Grid".                     |
//|              Derives from class CChartObjectTrend.               |
//+------------------------------------------------------------------+
class CChartObjectGannGrid : public CChartObjectTrend
  {
public:
   //--- methods access to properties object
   double            PipsPerBar() const;
   bool              PipsPerBar(double ppb);
   bool              Downtrend() const;
   bool              Downtrend(bool downtrend);
   //--- method create object
   bool              Create(long chart_id,string name,int window,datetime time1,double price1,datetime time2,double ppb);
   //--- method of identifying the object
   virtual int       Type() const { return(OBJ_GANNGRID); }
   //--- methods for working with files
   virtual bool      Save(int file_handle);
   virtual bool      Load(int file_handle);
  };
//+------------------------------------------------------------------+
//| Create object "Gann Grid".                                       |
//| INPUT:  chart_id-chart idintifier,                               |
//|         name    -object name,                                    |
//|         window  -subwindow number,                               |
//|         time1   -first coordinate time,                          |
//|         price1  -first coordinate price,                         |
//|         time2   -second coordinate time,                         |
//|         ppb     -scale.                                          |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CChartObjectGannGrid::Create(long chart_id,string name,int window,datetime time1,double price1,datetime time2,double ppb)
  {
   bool result=ObjectCreate(chart_id,name,OBJ_GANNGRID,window,time1,price1,time2,0.0);
//---
   if(result)
     {
      result&=Attach(chart_id,name,window,2);
      result&=PipsPerBar(ppb);
     }
//---
   return(result);
  }
//+------------------------------------------------------------------+
//| Get the property value "PipsPerBar".                             |
//| INPUT:  no.                                                      |
//| OUTPUT: the property value "PipsPerBar".                         |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
double CChartObjectGannGrid::PipsPerBar() const
  {
//--- checking
   if(m_chart_id==-1) return(EMPTY_VALUE);
//---
   return(ObjectGetDouble(m_chart_id,m_name,OBJPROP_SCALE));
  }
//+------------------------------------------------------------------+
//| Set the property value "PipsPerBar".                             |
//| INPUT:  deviations -new the property value "PipsPerBar".         |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CChartObjectGannGrid::PipsPerBar(double ppb)
  {
//--- checking
   if(m_chart_id==-1) return(false);
//---
   return(ObjectSetDouble(m_chart_id,m_name,OBJPROP_SCALE,ppb));
  }
//+------------------------------------------------------------------+
//| Get the property value "Downtrend".                              |
//| INPUT:  no.                                                      |
//| OUTPUT: the property value "Downtrend".                          |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CChartObjectGannGrid::Downtrend() const
  {
//--- checking
   if(m_chart_id==-1) return(false);
//---
   return(ObjectGetInteger(m_chart_id,m_name,OBJPROP_DIRECTION));
  }
//+------------------------------------------------------------------+
//| Set the property value "Downtrend".                              |
//| INPUT:  new_ray -new the property value "Downtrend".             |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CChartObjectGannGrid::Downtrend(bool downtrend)
  {
//--- checking
   if(m_chart_id==-1) return(false);
//---
   return(ObjectSetInteger(m_chart_id,m_name,OBJPROP_DIRECTION,downtrend));
  }
//+------------------------------------------------------------------+
//| Writing object parameters to file.                               |
//| INPUT:  file_handle -handle previously opened for writing file.  |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CChartObjectGannGrid::Save(int file_handle)
  {
   bool resutl;
//--- checking
   if(file_handle<=0) return(false);
   if(m_chart_id==-1) return(false);
//--- writing
   resutl=CChartObjectTrend::Save(file_handle);
   if(resutl)
     {
      //--- writing the property value "PipsPerBar"
      if(FileWriteDouble(file_handle,ObjectGetDouble(m_chart_id,m_name,OBJPROP_SCALE))!=sizeof(double)) return(false);
      //--- writing the property value "Downtrend"
      if(FileWriteInteger(file_handle,(int)ObjectGetInteger(m_chart_id,m_name,OBJPROP_DIRECTION),CHAR_VALUE)!=sizeof(char)) return(false);
     }
//---
   return(resutl);
  }
//+------------------------------------------------------------------+
//| Reading object parameters from file.                             |
//| INPUT:  file_handle -handle previously opened for reading file.  |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CChartObjectGannGrid::Load(int file_handle)
  {
   bool resutl;
//--- checking
   if(file_handle<=0) return(false);
   if(m_chart_id==-1) return(false);
//--- reading
   resutl=CChartObjectTrend::Load(file_handle);
   if(resutl)
     {
      //--- reading the property value "PipsPerBar"
      if(!ObjectSetDouble(m_chart_id,m_name,OBJPROP_SCALE,FileReadDouble(file_handle))) return(false);
      //--- reading the property value "Downtrend"
      if(!ObjectSetInteger(m_chart_id,m_name,OBJPROP_DIRECTION,FileReadInteger(file_handle,CHAR_VALUE))) return(false);
     }
//---
   return(resutl);
  }
//+------------------------------------------------------------------+
