//+------------------------------------------------------------------+
//|                                            ChartObjectsLines.mqh |
//|                        Copyright 2010, MetaQuotes Software Corp. |
//|                                        http://www.metaquotes.net |
//|                                              Revision 2010.02.22 |
//+------------------------------------------------------------------+
//| All lines.                                                       |
//+------------------------------------------------------------------+
#include "ChartObject.mqh"
//+------------------------------------------------------------------+
//| Class CChartObjectVLine.                                         |
//| Appointment: Class chart object "Vertical line".                 |
//|              Derives from class CChartObject.                    |
//+------------------------------------------------------------------+
class CChartObjectVLine : public CChartObject
  {
public:
   //--- method create object
   bool              Create(long chart_id,string name,int window,datetime time);
   //--- method of identifying the object
   virtual int       Type() const { return(OBJ_VLINE); }
  };
//+------------------------------------------------------------------+
//| Create object "Vertical line".                                   |
//| INPUT:  chart_id-chart idintifier,                               |
//|         name    -object name,                                    |
//|         window  -subwindow number,                               |
//|         time    -coordinate time.                                |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CChartObjectVLine::Create(long chart_id,string name,int window,datetime time)
  {
   bool result=ObjectCreate(chart_id,name,OBJ_VLINE,window,time,0.0);
   if(result) result&=Attach(chart_id,name,window,1);
//---
   return(result);
  }
//+------------------------------------------------------------------+
//| Class CChartObjectHLine.                                         |
//| Appointment: Class chart object "Horizontal line".               |
//|              Derives from class CChartObject.                    |
//+------------------------------------------------------------------+
class CChartObjectHLine : public CChartObject
  {
public:
   //--- method create object
   bool              Create(long chart_id,string name,int window,double price);
   //--- method of identifying the object
   virtual int       Type() const { return(OBJ_HLINE); }
  };
//+------------------------------------------------------------------+
//| Create object "Horizontal line".                                 |
//| INPUT:  chart_id-chart idintifier,                               |
//|         name    -object name,                                    |
//|         window  -subwindow number,                               |
//|         price   -coordinate price.                               |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CChartObjectHLine::Create(long chart_id,string name,int window,double price)
  {
   bool result=ObjectCreate(chart_id,name,OBJ_HLINE,window,0,price);
   if(result) result&=Attach(chart_id,name,window,1);
//---
   return(result);
  }
//+------------------------------------------------------------------+
//|  CChartObjectTrend.                                         |
//| Appointment: Class chart object "Trendline".                     |
//|              Derives from class CChartObject.                    |
//| Is the parent class for all objects having properties            |
//| RAY_LEFT  RAY_RIGHT.                                            |
//+------------------------------------------------------------------+
class CChartObjectTrend : public CChartObject
  {
public:
   //--- methods access to properties object
   bool              RayLeft() const;
   bool              RayLeft(bool new_sel);
   bool              RayRight() const;
   bool              RayRight(bool new_sel);
   //--- method create object
   bool              Create(long chart_id,string name,int window,datetime time1,double price1,datetime time2,double price2);
   //--- method of identifying the object
   virtual int       Type() const { return(OBJ_TREND); }
   //--- methods for working with files
   virtual bool      Save(int file_handle);
   virtual bool      Load(int file_handle);
  };
//+------------------------------------------------------------------+
//| Create object "Trendline".                                       |
//| INPUT:  chart_id-chart idintifier,                               |
//|         name    -object name,                                    |
//|         window  -subwindow number,                               |
//|         time1   -first coordinate time,                          |
//|         price1  -first coordinate price,                         |
//|         time2   -second coordinate time,                         |
//|         price2  -second coordinate price.                        |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CChartObjectTrend::Create(long chart_id,string name,int window,datetime time1,double price1,datetime time2,double price2)
  {
   bool result=ObjectCreate(chart_id,name,OBJ_TREND,window,time1,price1,time2,price2);
   if(result) result&=Attach(chart_id,name,window,2);
//---
   return(result);
  }
//+------------------------------------------------------------------+
//| Get flag "ray left".                                             |
//| INPUT:  no.                                                      |
//| OUTPUT: flag "ray left".                                         |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CChartObjectTrend::RayLeft() const
  {
//--- checking
   if(m_chart_id==-1) return(false);
//---
   return(ObjectGetInteger(m_chart_id,m_name,OBJPROP_RAY_LEFT));
  }
//+------------------------------------------------------------------+
//| Set flag "ray left".                                             |
//| INPUT:  new_ray -new flag "ray left".                            |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CChartObjectTrend::RayLeft(bool new_ray)
  {
//--- checking
   if(m_chart_id==-1) return(false);
//---
   return(ObjectSetInteger(m_chart_id,m_name,OBJPROP_RAY_LEFT,new_ray));
  }
//+------------------------------------------------------------------+
//| Get flag "ray right".                                            |
//| INPUT:  no.                                                      |
//| OUTPUT: flag "ray right".                                        |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CChartObjectTrend::RayRight() const
  {
//--- checking
   if(m_chart_id==-1) return(false);
//---
   return(ObjectGetInteger(m_chart_id,m_name,OBJPROP_RAY_RIGHT));
  }
//+------------------------------------------------------------------+
//| Set flag "ray right".                                            |
//| INPUT:  new_ray -new flag "ray right".                           |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CChartObjectTrend::RayRight(bool new_ray)
  {
//--- checking
   if(m_chart_id==-1) return(false);
//---
   return(ObjectSetInteger(m_chart_id,m_name,OBJPROP_RAY_RIGHT,new_ray));
  }
//+------------------------------------------------------------------+
//| Writing object parameters to file.                               |
//| INPUT:  file_handle -handle previously opened for writing file.  |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CChartObjectTrend::Save(int file_handle)
  {
   bool resutl;
//--- checking
   if(file_handle<=0) return(false);
   if(m_chart_id==-1) return(false);
//--- writing
   resutl=CChartObject::Save(file_handle);
   if(resutl)
     {
      //--- writing the property value "ray left"
      if(FileWriteInteger(file_handle,(int)ObjectGetInteger(m_chart_id,m_name,OBJPROP_RAY_LEFT),CHAR_VALUE)!=sizeof(char)) return(false);
      //--- writing the property value "ray right"
      if(FileWriteInteger(file_handle,(int)ObjectGetInteger(m_chart_id,m_name,OBJPROP_RAY_RIGHT),CHAR_VALUE)!=sizeof(char)) return(false);
     }
//---
   return(resutl);
  }
//+------------------------------------------------------------------+
//| Reading object parameters from file.                             |
//| INPUT:  file_handle -handle previously opened for reading file.  |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CChartObjectTrend::Load(int file_handle)
  {
   bool resutl;
//--- checking
   if(file_handle<=0) return(false);
   if(m_chart_id==-1) return(false);
//--- reading
   resutl=CChartObject::Load(file_handle);
   if(resutl)
     {
      //--- reading the property value "ray left"
      if(!ObjectSetInteger(m_chart_id,m_name,OBJPROP_RAY_LEFT,FileReadInteger(file_handle,CHAR_VALUE))) return(false);
      //--- reading the property value "ray right"
      if(!ObjectSetInteger(m_chart_id,m_name,OBJPROP_RAY_RIGHT,FileReadInteger(file_handle,CHAR_VALUE))) return(false);
     }
//---
   return(resutl);
  }
//+------------------------------------------------------------------+
//| Class CChartObjectTrendByAngle.                                  |
//| Appointment: Class chart object "Trendline by angle".            |
//|              Derives from class CChartObjectTrend.               |
//+------------------------------------------------------------------+
class CChartObjectTrendByAngle : public CChartObjectTrend
  {
public:
   //--- methods access to properties object
   double            Angle() const;
   bool              Angle(double angle);
   //--- method create object
   bool              Create(long chart_id,string name,int window,datetime time1,double price1,datetime time2,double price2);
   //--- method of identifying the object
   virtual int       Type() { return(OBJ_TRENDBYANGLE); }
  };
//+------------------------------------------------------------------+
//| Create object "Trendline by angle".                              |
//| INPUT:  chart_id-chart idintifier,                               |
//|         name    -object name,                                    |
//|         window  -subwindow number,                               |
//|         time1   -first coordinate time,                          |
//|         price1  -first coordinate price,                         |
//|         time2   -second coordinate time,                         |
//|         price2  -second coordinate price.                        |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CChartObjectTrendByAngle::Create(long chart_id,string name,int window,datetime time1,double price1,datetime time2,double price2)
  {
   bool result=ObjectCreate(chart_id,name,OBJ_TRENDBYANGLE,window,time1,price1,time2,price2);
//---
   if(result) result&=Attach(chart_id,name,window,2);
//---
   return(result);
  }
//+------------------------------------------------------------------+
//| Get angle.                                                       |
//| INPUT:  no.                                                      |
//| OUTPUT: angle.                                                   |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
double CChartObjectTrendByAngle::Angle() const
  {
//--- checking
   if(m_chart_id==-1) return(EMPTY_VALUE);
//---
   return(ObjectGetDouble(m_chart_id,m_name,OBJPROP_ANGLE));
  }
//+------------------------------------------------------------------+
//| Set angle.                                                       |
//| INPUT:  angle -new angle.                                        |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CChartObjectTrendByAngle::Angle(double angle)
  {
//--- checking
   if(m_chart_id==-1) return(false);
//---
   return(ObjectSetDouble(m_chart_id,m_name,OBJPROP_ANGLE,angle));
  }
//+------------------------------------------------------------------+
//| Class CChartObjectCycles.                                        |
//| Appointment: Class chart object "Cycles".                        |
//|              Derives from class CChartObject.                    |
//+------------------------------------------------------------------+
class CChartObjectCycles : public CChartObject
  {
public:
   //--- method create object
   bool              Create(long chart_id,string name,int window,datetime time1,double price1,datetime time2,double price2);
   //--- method of identifying the object
   virtual int       Type() const { return(OBJ_TREND); }
  };
//+------------------------------------------------------------------+
//| Create object "Cycles".                                          |
//| INPUT:  chart_id-chart idintifier,                               |
//|         name    -object name,                                    |
//|         window  -subwindow number,                               |
//|         time1   -first coordinate time,                          |
//|         price1  -first coordinate price,                         |
//|         time2   -second coordinate time,                         |
//|         price2  -second coordinate price.                        |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CChartObjectCycles::Create(long chart_id,string name,int window,datetime time1,double price1,datetime time2,double price2)
  {
   bool result=ObjectCreate(chart_id,name,OBJ_CYCLES,window,time1,price1,time2,price2);
//---
   if(result) result&=Attach(chart_id,name,window,2);
//---
   return(result);
  }
//+------------------------------------------------------------------+
