//+------------------------------------------------------------------+
//|                                                     Chart.mqh    |
//|                        Copyright 2010, MetaQuotes Software Corp. |
//|                                        http://www.metaquotes.net |
//|                                              Revision 2010.02.22 |
//+------------------------------------------------------------------+
#include <Object.mqh>
//+------------------------------------------------------------------+
//| Class CChart.                                                    |
//| Appointment: Class of object "Chart".                            |
//|              Derives from class CObject.                         |
//+------------------------------------------------------------------+
class CChart : public CObject
  {
protected:
   long              m_chart_id;           // chart idintifier
public:
                     CChart();
                    ~CChart();
   //--- methods of access to protected data
   long              ChartId() const       { return(m_chart_id); }
   //--- method of identifying the object
   virtual int       Type() const          { return(0x1111);     }
   //--- methods access to properties chart
   //--- common propertyes
   ENUM_CHART_MODE   Mode() const;
   bool              Mode(ENUM_CHART_MODE mode);
   bool              Foreground() const;
   bool              Foreground(bool foreground);
   bool              Shift() const;
   bool              Shift(bool shift);
   double            ShiftSize() const;
   bool              ShiftSize(double shift);
   bool              AutoScroll() const;
   bool              AutoScroll(bool auto_scroll);
   int               Scale() const;
   bool              Scale(int scale);
   bool              ScaleFix() const;
   bool              ScaleFix(bool scale_fix);
   bool              ScaleFix_11() const;
   bool              ScaleFix_11(bool scale_fix_11);
   double            FixedMax() const;
   bool              FixedMax(double fixed_max);
   double            FixedMin() const;
   bool              FixedMin(double fixed_min);
   bool              ScalePPB() const;
   bool              ScalePPB(bool scale_ppb);
   double            PointsPerBar() const;
   bool              PointsPerBar(double points_per_bar);
   //--- show propertyes
   bool              ShowOHLC() const;
   bool              ShowOHLC(bool show);
   bool              ShowLineBid() const;
   bool              ShowLineBid(bool show);
   bool              ShowLineAsk() const;
   bool              ShowLineAsk(bool show);
   bool              ShowLastLine() const;
   bool              ShowLastLine(bool show);
   bool              ShowPeriodSep() const;
   bool              ShowPeriodSep(bool show);
   bool              ShowGrid() const;
   bool              ShowGrid(bool show);
   ENUM_CHART_VOLUME_MODE ShowVolumes() const;
   bool                   ShowVolumes(ENUM_CHART_VOLUME_MODE show);
   bool              ShowObjectDescr() const;
   bool              ShowObjectDescr(bool show);
   //--- colors propertyes
   color             ColorBackground() const;
   bool              ColorBackground(color new_color);
   color             ColorForeground() const;
   bool              ColorForeground(color new_color);
   color             ColorGrid() const;
   bool              ColorGrid(color new_color);
   color             ColorBarUp() const;
   bool              ColorBarUp(color new_color);
   color             ColorBarDown() const;
   bool              ColorBarDown(color new_color);
   color             ColorCandleBull() const;
   bool              ColorCandleBull(color new_color);
   color             ColorCandleBear() const;
   bool              ColorCandleBear(color new_color);
   color             ColorChartLine() const;
   bool              ColorChartLine(color new_color);
   color             ColorVolumes() const;
   bool              ColorVolumes(color new_color);
   color             ColorLineBid() const;
   bool              ColorLineBid(color new_color);
   color             ColorLineAsk() const;
   bool              ColorLineAsk(color new_color);
   color             ColorLineLast() const;
   bool              ColorLineLast(color new_color);
   color             ColorStopLevels() const;
   bool              ColorStopLevels(color new_color);
   //--- methods access to READ ONLY properties chart
   int               VisibleBars() const;
   int               WindowsTotal() const;
   bool              WindowIsVisible(int num) const;
   int               WindowHandle() const;
   int               FirstVisibleBar() const;
   int               WidthInBars() const;
   int               WidthInPixels() const;
   int               HeightInPixels(int num) const;
   double            PriceMin(int num) const;
   double            PriceMax(int num) const;
   //--- methods binding chart
   void              Attach()             { m_chart_id=ChartID();             }
   void              Attach(long chart)   { m_chart_id=chart;                 }
   void              FirstChart()         { m_chart_id=ChartFirst();          }
   void              NextChart()          { m_chart_id=ChartNext(m_chart_id); }
   long              Open(const string symbol_name,ENUM_TIMEFRAMES timeframe);
   void              Detach()             { m_chart_id=-1;                    }
   void              Close();
   //--- navigation method
   bool              Navigate(ENUM_CHART_POSITION position,int shift=0);
   //--- access methods to the API MQL5 functions
   string            Symbol() const       { return(ChartSymbol(m_chart_id));  }
   ENUM_TIMEFRAMES   Period() const       { return(ChartPeriod(m_chart_id));  }
   void              Redraw()             { ChartRedraw(m_chart_id);          }
   long              GetInteger(ENUM_CHART_PROPERTY_INTEGER prop_id,int sub_window=0) const;
   bool              GetInteger(ENUM_CHART_PROPERTY_INTEGER prop_id,int sub_window,long& value) const;
   bool              SetInteger(ENUM_CHART_PROPERTY_INTEGER prop_id,long value);
   double            GetDouble(ENUM_CHART_PROPERTY_DOUBLE prop_id,int sub_window=0) const;
   bool              GetDouble(ENUM_CHART_PROPERTY_DOUBLE prop_id,int sub_window,double& value) const;
   bool              SetDouble(ENUM_CHART_PROPERTY_DOUBLE prop_id,double value);
   string            GetString(ENUM_CHART_PROPERTY_STRING prop_id) const;
   bool              GetString(ENUM_CHART_PROPERTY_STRING prop_id,string& value) const;
   bool              SetString(ENUM_CHART_PROPERTY_STRING prop_id,const string value);
   bool              SetSymbolPeriod(const string symbol,ENUM_TIMEFRAMES period);
   bool              ApplyTemplate(const string filename);
   bool              ScreenShot(const string filename, int width, int height, ENUM_ALIGN_MODE align_mode=ALIGN_RIGHT) const;
   int               WindowOnDropped() const;
   double            PriceOnDropped() const;
   datetime          TimeOnDropped() const;
   int               XOnDropped() const;
   int               YOnDropped() const;
   //--- methods for working with files
   virtual bool      Save(int file_handle);
   virtual bool      Load(int file_handle);
  };
//+------------------------------------------------------------------+
//| Constructor CChart.                                              |
//| INPUT:  no.                                                      |
//| OUTPUT: no.                                                      |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
CChart::CChart()
  {
//--- initialize protected data
   m_chart_id=-1;
  }
//+------------------------------------------------------------------+
//| Destructor CChart.                                               |
//| INPUT:  no.                                                      |
//| OUTPUT: no.                                                      |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
CChart::~CChart()
  {
   if(m_chart_id!=-1) Close();
  }
//+------------------------------------------------------------------+
//| Opening chart.                                                   |
//| INPUT:  symbol_name  -chart symbol,                              |
//|         timeframe    -chart period.                              |
//| OUTPUT: ID chart for a successful opening, otherwise - 0.        |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
long CChart::Open(const string symbol_name,ENUM_TIMEFRAMES timeframe)
  {
   return(m_chart_id=ChartOpen(symbol_name,timeframe));
  }
//+------------------------------------------------------------------+
//| Get the type of presentation chart.                              |
//| INPUT:  no.                                                      |
//| OUTPUT: type of presentation chart (candles,bars, etc.).         |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
ENUM_CHART_MODE CChart::Mode() const
  {
//--- checking
   if(m_chart_id<=0) return(WRONG_VALUE);
//---
   return((ENUM_CHART_MODE)ChartGetInteger(m_chart_id,CHART_MODE));
  }
//+------------------------------------------------------------------+
//| Set the type of presentation chart.                              |
//| INPUT:  mode -new type of presentation.                          |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CChart::Mode(ENUM_CHART_MODE mode)
  {
//--- checking
   if(m_chart_id<=0) return(false);
//---
   return(ChartSetInteger(m_chart_id,CHART_MODE,mode));
  }
//+------------------------------------------------------------------+
//| Get the property value "Foreground".                             |
//| INPUT:  no.                                                      |
//| OUTPUT: the property value "Foreground".                         |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CChart::Foreground() const
  {
//--- checking
   if(m_chart_id<=0) return(false);
//---
   return(ChartGetInteger(m_chart_id,CHART_FOREGROUND));
  }
//+------------------------------------------------------------------+
//| Set the property value "Foreground".                             |
//| INPUT:  foreground - new the property value "Foreground".        |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CChart::Foreground(bool foreground)
  {
//--- checking
   if(m_chart_id<=0) return(false);
//---
   return(ChartSetInteger(m_chart_id,CHART_FOREGROUND,foreground));
  }
//+------------------------------------------------------------------+
//| Get the property value "Shift".                                  |
//| INPUT:  no.                                                      |
//| OUTPUT: the property value "Shift".                              |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CChart::Shift() const
  {
//--- checking
   if(m_chart_id<=0) return(false);
//---
   return(ChartGetInteger(m_chart_id,CHART_SHIFT));
  }
//+------------------------------------------------------------------+
//| Set the property value "Shift".                                  |
//| INPUT:  shift - new the property value "Shift".                  |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CChart::Shift(bool shift)
  {
//--- checking
   if(m_chart_id<=0) return(false);
//---
   return(ChartSetInteger(m_chart_id,CHART_SHIFT,shift));
  }
//+------------------------------------------------------------------+
//| Get the property value "ShiftSize".                              |
//| INPUT:  no.                                                      |
//| OUTPUT: the property value "ShiftSize".                          |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
double CChart::ShiftSize() const
  {
//--- checking
   if(m_chart_id<=0) return(DBL_MAX);
//---
   return(ChartGetDouble(m_chart_id,CHART_SHIFT_SIZE));
  }
//+------------------------------------------------------------------+
//| Set the property value "ShiftSize".                              |
//| INPUT:  shift - new the property value "ShiftSize".              |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CChart::ShiftSize(double shift)
  {
//--- checking
   if(m_chart_id<=0) return(false);
   if(shift<10) shift=10;
   if(shift>50) shift=50;
//---
   return(ChartSetDouble(m_chart_id,CHART_SHIFT_SIZE,shift));
  }
//+------------------------------------------------------------------+
//| Get the property value "AutoScroll".                             |
//| INPUT:  no.                                                      |
//| OUTPUT: the property value "AutoScroll".                         |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CChart::AutoScroll() const
  {
//--- checking
   if(m_chart_id<=0) return(false);
//---
   return(ChartGetInteger(m_chart_id,CHART_AUTOSCROLL));
  }
//+------------------------------------------------------------------+
//| Set the property value "AutoScroll".                             |
//| INPUT:  auto_scroll - new the property value "AutoScroll".       |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CChart::AutoScroll(bool auto_scroll)
  {
//--- checking
   if(m_chart_id<=0) return(false);
//---
   return(ChartSetInteger(m_chart_id,CHART_AUTOSCROLL,auto_scroll));
  }
//+------------------------------------------------------------------+
//| Get the property value "Scale".                                  |
//| INPUT:  no.                                                      |
//| OUTPUT: the property value "Scale".                              |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
int CChart::Scale() const
  {
//--- checking
   if(m_chart_id<=0) return(0);
//---
   return((int)ChartGetInteger(m_chart_id,CHART_SCALE));
  }
//+------------------------------------------------------------------+
//| Set the property value "Scale".                                  |
//| INPUT:  shift - new the property value "Scale".                  |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CChart::Scale(int shift)
  {
//--- checking
   if(m_chart_id<=0) return(false);
   if(shift<0)  shift=0;
   if(shift>32) shift=32;
//---
   return(ChartSetInteger(m_chart_id,CHART_SCALE,shift));
  }
//+------------------------------------------------------------------+
//| Get the property value "ScaleFix".                               |
//| INPUT:  no.                                                      |
//| OUTPUT: the property value "ScaleFix".                           |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CChart::ScaleFix() const
  {
//--- checking
   if(m_chart_id<=0) return(false);
//---
   return(ChartGetInteger(m_chart_id,CHART_SCALEFIX));
  }
//+------------------------------------------------------------------+
//| Set the property value "ScaleFix".                               |
//| INPUT:  scale_fix - new the property value "ScaleFix".           |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CChart::ScaleFix(bool scale_fix)
  {
//--- checking
   if(m_chart_id<=0) return(false);
//---
   return(ChartSetInteger(m_chart_id,CHART_SCALEFIX,scale_fix));
  }
//+------------------------------------------------------------------+
//| Get the property value "ScaleFix_11".                            |
//| INPUT:  no.                                                      |
//| OUTPUT: the property value "ScaleFix_11".                        |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CChart::ScaleFix_11() const
  {
//--- checking
   if(m_chart_id<=0) return(false);
//---
   return(ChartGetInteger(m_chart_id,CHART_SCALEFIX_11));
  }
//+------------------------------------------------------------------+
//| Set the property value "ScaleFix_11".                            |
//| INPUT:  scale_fix_11 - new the property value "ScaleFix_11".     |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CChart::ScaleFix_11(bool scale_fix_11)
  {
//--- checking
   if(m_chart_id<=0) return(false);
//---
   return(ChartSetInteger(m_chart_id,CHART_SCALEFIX_11,scale_fix_11));
  }
//+------------------------------------------------------------------+
//| Get the property value "FixedMax".                               |
//| INPUT:  no.                                                      |
//| OUTPUT: the property value "FixedMax".                           |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
double CChart::FixedMax() const
  {
//--- checking
   if(m_chart_id<=0) return(EMPTY_VALUE);
//---
   return(ChartGetDouble(m_chart_id,CHART_FIXED_MAX));
  }
//+------------------------------------------------------------------+
//| Set the property value "FixedMax".                               |
//| INPUT:  fixed_max - new the property value "FixedMax".           |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CChart::FixedMax(double fixed_max)
  {
//--- checking
   if(m_chart_id<=0) return(false);
//---
   return(ChartSetDouble(m_chart_id,CHART_FIXED_MAX,fixed_max));
  }
//+------------------------------------------------------------------+
//| Get the property value "FixedMin".                               |
//| INPUT:  no.                                                      |
//| OUTPUT: the property value "FixedMin".                           |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
double CChart::FixedMin() const
  {
//--- checking
   if(m_chart_id<=0) return(EMPTY_VALUE);
//---
   return(ChartGetDouble(m_chart_id,CHART_FIXED_MIN));
  }
//+------------------------------------------------------------------+
//| Set the property value "FixedMin".                               |
//| INPUT:  fixed_min - new the property value "FixedMin".           |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CChart::FixedMin(double fixed_min)
  {
//--- checking
   if(m_chart_id<=0) return(false);
//---
   return(ChartSetDouble(m_chart_id,CHART_FIXED_MIN,fixed_min));
  }
//+------------------------------------------------------------------+
//| Get the property value "ScalePointsPerBar".                      |
//| INPUT:  no.                                                      |
//| OUTPUT: the property value "ScalePointsPerBar".                  |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CChart::ScalePPB() const
  {
//--- checking
   if(m_chart_id<=0) return(false);
//---
   return(ChartGetInteger(m_chart_id,CHART_SCALE_PT_PER_BAR));
  }
//+------------------------------------------------------------------+
//| Set the property value "ScalePointsPerBar".                      |
//| INPUT:  scale_ppb - new the property value "ScalePointsPerBar".  |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CChart::ScalePPB(bool scale_ppb)
  {
//--- checking
   if(m_chart_id<=0) return(false);
//---
   return(ChartSetInteger(m_chart_id,CHART_SCALE_PT_PER_BAR,scale_ppb));
  }
//+------------------------------------------------------------------+
//| Get the property value "PointsPerBar".                           |
//| INPUT:  no.                                                      |
//| OUTPUT: the property value "PointsPerBar".                       |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
double CChart::PointsPerBar() const
  {
//--- checking
   if(m_chart_id<=0) return(EMPTY_VALUE);
//---
   return(ChartGetDouble(m_chart_id,CHART_POINTS_PER_BAR));
  }
//+------------------------------------------------------------------+
//| Set the property value "PointsPerBar".                           |
//| INPUT:  points_per_bar - new the property value "PointsPerBar".  |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CChart::PointsPerBar(double points_per_bar)
  {
//--- checking
   if(m_chart_id<=0) return(false);
//---
   return(ChartSetDouble(m_chart_id,CHART_POINTS_PER_BAR,points_per_bar));
  }
//+------------------------------------------------------------------+
//| Get the property value "ShowOHLC".                               |
//| INPUT:  no.                                                      |
//| OUTPUT: the property value "ShowOHLC".                           |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CChart::ShowOHLC() const
  {
//--- checking
   if(m_chart_id<=0) return(false);
//---
   return(ChartGetInteger(m_chart_id,CHART_SHOW_OHLC));
  }
//+------------------------------------------------------------------+
//| Set the property value "ShowOHLC".                               |
//| INPUT:  show - new the property value "ShowOHLC".                |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CChart::ShowOHLC(bool show)
  {
//--- checking
   if(m_chart_id<=0) return(false);
//---
   return(ChartSetInteger(m_chart_id,CHART_SHOW_OHLC,show));
  }
//+------------------------------------------------------------------+
//| Get the property value "ShowLineBid".                            |
//| INPUT:  no.                                                      |
//| OUTPUT: the property value "ShowLineBid".                        |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CChart::ShowLineBid() const
  {
//--- checking
   if(m_chart_id<=0) return(false);
//---
   return(ChartGetInteger(m_chart_id,CHART_SHOW_BID_LINE));
  }
//+------------------------------------------------------------------+
//| Set the property value "ShowLineBid".                            |
//| INPUT:  show - new the property value "ShowLineBid".             |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CChart::ShowLineBid(bool show)
  {
//--- checking
   if(m_chart_id<=0) return(false);
//---
   return(ChartSetInteger(m_chart_id,CHART_SHOW_BID_LINE,show));
  }
//+------------------------------------------------------------------+
//| Get the property value "ShowLineAsk".                            |
//| INPUT:  no.                                                      |
//| OUTPUT: the property value "ShowLineAsk".                        |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CChart::ShowLineAsk() const
  {
//--- checking
   if(m_chart_id<=0) return(false);
//---
   return(ChartGetInteger(m_chart_id,CHART_SHOW_ASK_LINE));
  }
//+------------------------------------------------------------------+
//| Set the property value "ShowLineAsk".                            |
//| INPUT:  show - new the property value "ShowLineAsk".             |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CChart::ShowLineAsk(bool show)
  {
//--- checking
   if(m_chart_id<=0) return(false);
//---
   return(ChartSetInteger(m_chart_id,CHART_SHOW_ASK_LINE,show));
  }
//+------------------------------------------------------------------+
//| Get the property value "ShowLastLine".                           |
//| INPUT:  no.                                                      |
//| OUTPUT: the property value "ShowLastLine".                       |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CChart::ShowLastLine() const
  {
//--- checking
   if(m_chart_id<=0) return(false);
//---
   return(ChartGetInteger(m_chart_id,CHART_SHOW_LAST_LINE));
  }
//+------------------------------------------------------------------+
//| Set the property value "ShowLastLine".                           |
//| INPUT:  show - new the property value "ShowLastLine".            |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CChart::ShowLastLine(bool show)
  {
//--- checking
   if(m_chart_id<=0) return(false);
//---
   return(ChartSetInteger(m_chart_id,CHART_SHOW_LAST_LINE,show));
  }
//+------------------------------------------------------------------+
//| Get the property value "ShowPeriodSep".                          |
//| INPUT:  no.                                                      |
//| OUTPUT: the property value "ShowPeriodSep".                      |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CChart::ShowPeriodSep() const
  {
//--- checking
   if(m_chart_id<=0) return(false);
//---
   return(ChartGetInteger(m_chart_id,CHART_SHOW_PERIOD_SEP));
  }
//+------------------------------------------------------------------+
//| Set the property value "ShowPeriodSep".                          |
//| INPUT:  show - new the property value "ShowPeriodSep".           |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CChart::ShowPeriodSep(bool show)
  {
//--- checking
   if(m_chart_id<=0) return(false);
//---
   return(ChartSetInteger(m_chart_id,CHART_SHOW_PERIOD_SEP,show));
  }
//+------------------------------------------------------------------+
//| Get the property value "ShowGrid".                               |
//| INPUT:  no.                                                      |
//| OUTPUT: the property value "ShowGrid".                           |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CChart::ShowGrid() const
  {
//--- checking
   if(m_chart_id<=0) return(false);
//---
   return(ChartGetInteger(m_chart_id,CHART_SHOW_GRID));
  }
//+------------------------------------------------------------------+
//| Set the property value "ShowGrid".                               |
//| INPUT:  show - new the property value "ShowGrid".                |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CChart::ShowGrid(bool show)
  {
//--- checking
   if(m_chart_id<=0) return(false);
//---
   return(ChartSetInteger(m_chart_id,CHART_SHOW_GRID,show));
  }
//+------------------------------------------------------------------+
//| Get the property value "ShowVolumes".                            |
//| INPUT:  no.                                                      |
//| OUTPUT: the property value "ShowVolumes".                        |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
ENUM_CHART_VOLUME_MODE CChart::ShowVolumes() const
  {
//--- checking
   if(m_chart_id<=0) return(WRONG_VALUE);
//---
   return((ENUM_CHART_VOLUME_MODE)ChartGetInteger(m_chart_id,CHART_SHOW_VOLUMES));
  }
//+------------------------------------------------------------------+
//| Set the property value "ShowVolumes".                            |
//| INPUT:  show - new the property value "ShowVolumes".             |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CChart::ShowVolumes(ENUM_CHART_VOLUME_MODE show)
  {
//--- checking
   if(m_chart_id<=0) return(false);
//---
   return(ChartSetInteger(m_chart_id,CHART_SHOW_VOLUMES,show));
  }
//+------------------------------------------------------------------+
//| Get the property value "ShowObjectDescr".                        |
//| INPUT:  no.                                                      |
//| OUTPUT: the property value "ShowObjectDescr".                    |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CChart::ShowObjectDescr() const
  {
//--- checking
   if(m_chart_id<=0) return(false);
//---
   return(ChartGetInteger(m_chart_id,CHART_SHOW_OBJECT_DESCR));
  }
//+------------------------------------------------------------------+
//| Set the property value "ShowObjectDescr".                        |
//| INPUT:  show - new the property value "ShowObjectDescr".         |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CChart::ShowObjectDescr(bool show)
  {
//--- checking
   if(m_chart_id<=0) return(false);
//---
   return(ChartSetInteger(m_chart_id,CHART_SHOW_OBJECT_DESCR,show));
  }
//+------------------------------------------------------------------+
//| Get the color value "Background".                                |
//| INPUT:  no.                                                      |
//| OUTPUT: the color value "Background".                            |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
color CChart::ColorBackground() const
  {
//--- checking
   if(m_chart_id<=0) return(CLR_NONE);
//---
   return((color)ChartGetInteger(m_chart_id,CHART_COLOR_BACKGROUND));
  }
//+------------------------------------------------------------------+
//| Set the color value "Background".                                |
//| INPUT:  new_color - new the color value "Background".            |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CChart::ColorBackground(color new_color)
  {
//--- checking
   if(m_chart_id<=0) return(false);
//---
   return(ChartSetInteger(m_chart_id,CHART_COLOR_BACKGROUND,new_color));
  }
//+------------------------------------------------------------------+
//| Get the color value "Foreground".                                |
//| INPUT:  no.                                                      |
//| OUTPUT: the color value "Foreground".                            |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
color CChart::ColorForeground() const
  {
//--- checking
   if(m_chart_id<=0) return(CLR_NONE);
//---
   return((color)ChartGetInteger(m_chart_id,CHART_COLOR_FOREGROUND));
  }
//+------------------------------------------------------------------+
//| Set the color value "Foreground".                                |
//| INPUT:  new_color - new the color value "Foreground".            |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CChart::ColorForeground(color new_color)
  {
//--- checking
   if(m_chart_id<=0) return(false);
//---
   return(ChartSetInteger(m_chart_id,CHART_COLOR_FOREGROUND,new_color));
  }
//+------------------------------------------------------------------+
//| Get the color value "Grid".                                      |
//| INPUT:  no.                                                      |
//| OUTPUT: the color value "Grid".                                  |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
color CChart::ColorGrid() const
  {
//--- checking
   if(m_chart_id<=0) return(CLR_NONE);
//---
   return((color)ChartGetInteger(m_chart_id,CHART_COLOR_GRID));
  }
//+------------------------------------------------------------------+
//| Set the color value "Grid".                                      |
//| INPUT:  new_color - new the color value "Grid".                  |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CChart::ColorGrid(color new_color)
  {
//--- checking
   if(m_chart_id<=0) return(false);
//---
   return(ChartSetInteger(m_chart_id,CHART_COLOR_GRID,new_color));
  }
//+------------------------------------------------------------------+
//| Get the color value "Bar Up".                                    |
//| INPUT:  no.                                                      |
//| OUTPUT: the color value "Bar Up".                                |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
color CChart::ColorBarUp() const
  {
//--- checking
   if(m_chart_id<=0) return(CLR_NONE);
//---
   return((color)ChartGetInteger(m_chart_id,CHART_COLOR_CHART_UP));
  }
//+------------------------------------------------------------------+
//| Set the color value "Bar Up".                                    |
//| INPUT:  new_color - new the color value "Bar Up".                |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CChart::ColorBarUp(color new_color)
  {
//--- checking
   if(m_chart_id<=0) return(false);
//---
   return(ChartSetInteger(m_chart_id,CHART_COLOR_CHART_UP,new_color));
  }
//+------------------------------------------------------------------+
//| Get the color value "Bar Down".                                  |
//| INPUT:  no.                                                      |
//| OUTPUT: the color value "Bar Down".                              |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
color CChart::ColorBarDown() const
  {
//--- checking
   if(m_chart_id<=0) return(CLR_NONE);
//---
   return((color)ChartGetInteger(m_chart_id,CHART_COLOR_CHART_DOWN));
  }
//+------------------------------------------------------------------+
//| Set the color value "Bar Down".                                  |
//| INPUT:  new_color - new the color value "Bar Down".              |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CChart::ColorBarDown(color new_color)
  {
//--- checking
   if(m_chart_id<=0) return(false);
//---
   return(ChartSetInteger(m_chart_id,CHART_COLOR_CHART_DOWN,new_color));
  }
//+------------------------------------------------------------------+
//| Get the color value "Candle Bull".                               |
//| INPUT:  no.                                                      |
//| OUTPUT: the color value "Candle Bull".                           |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
color CChart::ColorCandleBull() const
  {
//--- checking
   if(m_chart_id<=0) return(CLR_NONE);
//---
   return((color)ChartGetInteger(m_chart_id,CHART_COLOR_CANDLE_BULL));
  }
//+------------------------------------------------------------------+
//| Set the color value "Candle Bull".                               |
//| INPUT:  new_color - new the color value "Candle Bull".           |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CChart::ColorCandleBull(color new_color)
  {
//--- checking
   if(m_chart_id<=0) return(false);
//---
   return(ChartSetInteger(m_chart_id,CHART_COLOR_CANDLE_BULL,new_color));
  }
//+------------------------------------------------------------------+
//| Get the color value "Candle Bear".                               |
//| INPUT:  no.                                                      |
//| OUTPUT: the color value "Candle Bear".                           |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
color CChart::ColorCandleBear() const
  {
//--- checking
   if(m_chart_id<=0) return(CLR_NONE);
//---
   return((color)ChartGetInteger(m_chart_id,CHART_COLOR_CANDLE_BEAR));
  }
//+------------------------------------------------------------------+
//| Set the color value "Candle Bear".                               |
//| INPUT:  new_color - new the color value "Candle Bear".           |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CChart::ColorCandleBear(color new_color)
  {
//--- checking
   if(m_chart_id<=0) return(false);
//---
   return(ChartSetInteger(m_chart_id,CHART_COLOR_CANDLE_BEAR,new_color));
  }
//+------------------------------------------------------------------+
//| Get the color value "Chart Line".                                |
//| INPUT:  no.                                                      |
//| OUTPUT: the color value "Chart Line".                            |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
color CChart::ColorChartLine() const
  {
//--- checking
   if(m_chart_id<=0) return(CLR_NONE);
//---
   return((color)ChartGetInteger(m_chart_id,CHART_COLOR_CHART_LINE));
  }
//+------------------------------------------------------------------+
//| Set the color value "Chart Line".                                |
//| INPUT:  new_color - new the color value "Chart Line".            |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CChart::ColorChartLine(color new_color)
  {
//--- checking
   if(m_chart_id<=0) return(false);
//---
   return(ChartSetInteger(m_chart_id,CHART_COLOR_CHART_LINE,new_color));
  }
//+------------------------------------------------------------------+
//| Get the color value "Volumes".                                   |
//| INPUT:  no.                                                      |
//| OUTPUT: the color value "Volumes".                               |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
color CChart::ColorVolumes() const
  {
//--- checking
   if(m_chart_id<=0) return(CLR_NONE);
//---
   return((color)ChartGetInteger(m_chart_id,CHART_COLOR_VOLUME));
  }
//+------------------------------------------------------------------+
//| Set the color value "Volumes".                                   |
//| INPUT:  new_color - new the color value "Volumes".               |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CChart::ColorVolumes(color new_color)
  {
//--- checking
   if(m_chart_id<=0) return(false);
//---
   return(ChartSetInteger(m_chart_id,CHART_COLOR_VOLUME,new_color));
  }
//+------------------------------------------------------------------+
//| Get the color value "Line Bid".                                  |
//| INPUT:  no.                                                      |
//| OUTPUT: the color value "Line Bid".                              |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
color CChart::ColorLineBid() const
  {
//--- checking
   if(m_chart_id<=0) return(CLR_NONE);
//---
   return((color)ChartGetInteger(m_chart_id,CHART_COLOR_BID));
  }
//+------------------------------------------------------------------+
//| Set the color value "Line Bid".                                  |
//| INPUT:  new_color - new the color value "Line Bid".              |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CChart::ColorLineBid(color new_color)
  {
//--- checking
   if(m_chart_id<=0) return(false);
//---
   return(ChartSetInteger(m_chart_id,CHART_COLOR_BID,new_color));
  }
//+------------------------------------------------------------------+
//| Get the color value "Line Ask".                                  |
//| INPUT:  no.                                                      |
//| OUTPUT: the color value "Line Ask".                              |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
color CChart::ColorLineAsk() const
  {
//--- checking
   if(m_chart_id<=0) return(CLR_NONE);
//---
   return((color)ChartGetInteger(m_chart_id,CHART_COLOR_ASK));
  }
//+------------------------------------------------------------------+
//| Set the color value "Line Ask".                                  |
//| INPUT:  new_color - new the color value "Line Ask".              |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CChart::ColorLineAsk(color new_color)
  {
//--- checking
   if(m_chart_id<=0) return(false);
//---
   return(ChartSetInteger(m_chart_id,CHART_COLOR_ASK,new_color));
  }
//+------------------------------------------------------------------+
//| Get the color value "Line Last".                                 |
//| INPUT:  no.                                                      |
//| OUTPUT: the color value "Line Last".                             |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
color CChart::ColorLineLast() const
  {
//--- checking
   if(m_chart_id<=0) return(CLR_NONE);
//---
   return((color)ChartGetInteger(m_chart_id,CHART_COLOR_LAST));
  }
//+------------------------------------------------------------------+
//| Set the color value "Line Last".                                 |
//| INPUT:  new_color - new the color value "Line Last".             |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CChart::ColorLineLast(color new_color)
  {
//--- checking
   if(m_chart_id<=0) return(false);
//---
   return(ChartSetInteger(m_chart_id,CHART_COLOR_LAST,new_color));
  }
//+------------------------------------------------------------------+
//| Get the color value "Stop Levels".                               |
//| INPUT:  no.                                                      |
//| OUTPUT: the color value "Stop Levels".                           |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
color CChart::ColorStopLevels() const
  {
//--- checking
   if(m_chart_id<=0) return(CLR_NONE);
//---
   return((color)ChartGetInteger(m_chart_id,CHART_COLOR_STOP_LEVEL));
  }
//+------------------------------------------------------------------+
//| Set the color value "Stop Levels".                               |
//| INPUT:  new_color - new the color value "Stop Levels".           |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CChart::ColorStopLevels(color new_color)
  {
//--- checking
   if(m_chart_id<=0) return(false);
//---
   return(ChartSetInteger(m_chart_id,CHART_COLOR_STOP_LEVEL,new_color));
  }
//+------------------------------------------------------------------+
//| Get the property value "VisibleBars".                            |
//| INPUT:  no.                                                      |
//| OUTPUT: the property value "VisibleBars".                        |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
int CChart::VisibleBars() const
  {
//--- checking
   if(m_chart_id<=0) return(0);
//---
   return((int)ChartGetInteger(m_chart_id,CHART_WIDTH_IN_BARS));
  }
//+------------------------------------------------------------------+
//| Get the property value "WindowsTotal".                           |
//| INPUT:  no.                                                      |
//| OUTPUT: the property value "WindowsTotal".                       |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
int CChart::WindowsTotal() const
  {
//--- checking
   if(m_chart_id<=0) return(0);
//---
   return((int)ChartGetInteger(m_chart_id,CHART_WINDOWS_TOTAL));
  }
//+------------------------------------------------------------------+
//| Get the property value "WindowIsVisible".                        |
//| INPUT:  num -number of subwindows (0-main window).               |
//| OUTPUT: the property value "WindowIsVisible".                    |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CChart::WindowIsVisible(int num) const
  {
//--- checking
   if(m_chart_id<=0) return(false);
//---
   return(ChartGetInteger(m_chart_id,CHART_WINDOW_IS_VISIBLE,num));
  }
//+------------------------------------------------------------------+
//| Get the property value "WindowHandle".                           |
//| INPUT:  no.                                                      |
//| OUTPUT: the property value "WindowHandle".                       |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
int CChart::WindowHandle() const
  {
//--- checking
   if(m_chart_id<=0) return(INVALID_HANDLE);
//---
   return((int)ChartGetInteger(m_chart_id,CHART_WINDOW_HANDLE));
  }
//+------------------------------------------------------------------+
//| Get the property value "FirstVisibleBar".                        |
//| INPUT:  no.                                                      |
//| OUTPUT: the property value "FirstVisibleBar".                    |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
int CChart::FirstVisibleBar() const
  {
//--- checking
   if(m_chart_id<=0) return(-1);
//---
   return((int)ChartGetInteger(m_chart_id,CHART_FIRST_VISIBLE_BAR));
  }
//+------------------------------------------------------------------+
//| Get the property value "WidthInBars".                            |
//| INPUT:  no.                                                      |
//| OUTPUT: the property value "WidthInBars".                        |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
int CChart::WidthInBars() const
  {
//--- checking
   if(m_chart_id<=0) return(0);
//---
   return((int)ChartGetInteger(m_chart_id,CHART_WIDTH_IN_BARS));
  }
//+------------------------------------------------------------------+
//| Get the property value "WidthInPixels".                          |
//| INPUT:  no.                                                      |
//| OUTPUT: the property value "WidthInPixels".                      |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
int CChart::WidthInPixels() const
  {
//--- checking
   if(m_chart_id<=0) return(0);
//---
   return((int)ChartGetInteger(m_chart_id,CHART_WIDTH_IN_PIXELS));
  }
//+------------------------------------------------------------------+
//| Get the property value "HeightInPixels".                         |
//| INPUT:  num -number of subwindows (0-main window).               |
//| OUTPUT: the property value "HeightInPixels".                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
int CChart::HeightInPixels(int num) const
  {
//--- checking
   if(m_chart_id<=0) return(0);
//---
   return((int)ChartGetInteger(m_chart_id,CHART_HEIGHT_IN_PIXELS,num));
  }
//+------------------------------------------------------------------+
//| Get the property value "PriceMin".                               |
//| INPUT:  num -number of subwindows (0-main window).               |
//| OUTPUT: the property value "PriceMin".                           |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
double CChart::PriceMin(int num) const
  {
//--- checking
   if(m_chart_id<=0) return(EMPTY_VALUE);
//---
   return(ChartGetDouble(m_chart_id,CHART_PRICE_MIN,num));
  }
//+------------------------------------------------------------------+
//| Get the property value "PriceMax".                               |
//| INPUT:  num -number of subwindows (0-main window).               |
//| OUTPUT: the property value "PriceMax".                           |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
double CChart::PriceMax(int num) const
  {
//--- checking
   if(m_chart_id<=0) return(EMPTY_VALUE);
//---
   return(ChartGetDouble(m_chart_id,CHART_PRICE_MAX,num));
  }
//+------------------------------------------------------------------+
//| Chart close.                                                     |
//| INPUT:  no.                                                      |
//| OUTPUT: no.                                                      |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
void CChart::Close()
  {
   if(m_chart_id!=-1)
     {
      ChartClose(m_chart_id);
      m_chart_id=-1;
     }
  }
//+------------------------------------------------------------------+
//| Chart navigate.                                                  |
//| INPUT:  position -position of chart,                             |
//|                   which will be produced on offset,              |
//|         shift    -number of bars on which to shift the chart.    |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CChart::Navigate(ENUM_CHART_POSITION position,int shift)
  {
//--- checking
   if(m_chart_id<=0) return(false);
//---
   return(ChartNavigate(m_chart_id,position,shift));
  }
//+------------------------------------------------------------------+
//| Access functions long ChartGetInteger(...).                      |
//| INPUT:  prop_id   -identifier properties,                        |
//|         subwindow -subwindow.                                    |
//| OUTPUT: the property value.                                      |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
long CChart::GetInteger(ENUM_CHART_PROPERTY_INTEGER prop_id,int subwindow) const
  {
//--- checking
   if(m_chart_id<=0) return(0);
//---
   return(ChartGetInteger(m_chart_id,prop_id,subwindow));
  }
//+------------------------------------------------------------------+
//| Access functions bool ChartGetInteger(...).                      |
//| INPUT:  prop_id  -identifier properties,                         |
//|         modifier -subwindow,                                     |
//|         value    -variable for the property value.               |
//| OUTPUT: the property value.                                      |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CChart::GetInteger(ENUM_CHART_PROPERTY_INTEGER prop_id,int subwindow,long& value) const
  {
//--- checking
   if(m_chart_id<=0) return(false);
//---
   return(ChartGetInteger(m_chart_id,prop_id,subwindow,value));
  }
//+------------------------------------------------------------------+
//| Access functions ChartSetInteger(...).                           |
//| INPUT:  prop_id  -identifier properties,                         |
//|         value    -new the property value.                        |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CChart::SetInteger(ENUM_CHART_PROPERTY_INTEGER prop_id,long value)
  {
//--- checking
   if(m_chart_id<=0) return(false);
//---
   return(ChartSetInteger(m_chart_id,prop_id,value));
  }
//+------------------------------------------------------------------+
//| Access functions double ChartGetDouble(...).                     |
//| INPUT:  prop_id  -identifier properties,                         |
//|         modifier -subwindow.                                     |
//| OUTPUT: the property value.                                      |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
double CChart::GetDouble(ENUM_CHART_PROPERTY_DOUBLE prop_id,int subwindow) const
  {
//--- checking
   if(m_chart_id<=0) return(EMPTY_VALUE);
//---
   return(ChartGetDouble(m_chart_id,prop_id,subwindow));
  }
//+------------------------------------------------------------------+
//| Access functions bool ChartGetDouble(...).                       |
//| INPUT:  prop_id  -identifier properties,                         |
//|         modifier -subwindow,                                     |
//|         value    -variable for the property value.               |
//| OUTPUT: the property value.                                      |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CChart::GetDouble(ENUM_CHART_PROPERTY_DOUBLE prop_id,int subwindow,double& value) const
  {
//--- checking
   if(m_chart_id<=0) return(0);
//---
   return(ChartGetDouble(m_chart_id,prop_id,subwindow,value));
  }
//+------------------------------------------------------------------+
//| Access functions ChartSetDouble(...).                            |
//| INPUT:  prop_id  -identifier properties,                         |
//|         value    -new the property value.                        |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CChart::SetDouble(ENUM_CHART_PROPERTY_DOUBLE prop_id,double value)
  {
//--- checking
   if(m_chart_id<=0) return(false);
//---
   return(ChartSetDouble(m_chart_id,prop_id,value));
  }
//+------------------------------------------------------------------+
//| Access functions string ChartGetString(...).                     |
//| INPUT:  prop_id   -identifier properties.                        |
//| OUTPUT: the property value.                                      |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
string CChart::GetString(ENUM_CHART_PROPERTY_STRING prop_id) const
  {
//--- checking
   if(m_chart_id<=0) return("");
//---
   return(ChartGetString(m_chart_id,prop_id));
  }
//+------------------------------------------------------------------+
//| Access functions bool ChartGetString(...).                       |
//| INPUT:  prop_id  -identifier properties,                         |
//|         value    -variable for the property value.               |
//| OUTPUT: the property value.                                      |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CChart::GetString(ENUM_CHART_PROPERTY_STRING prop_id,string& value) const
  {
//--- checking
   if(m_chart_id<=0) return(false);
//---
   return(ChartGetString(m_chart_id,prop_id,value));
  }
//+------------------------------------------------------------------+
//| Access functions ChartSetString(...).                            |
//| INPUT:  prop_id  -identifier properties,                         |
//|         value    -new the property value.                        |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CChart::SetString(ENUM_CHART_PROPERTY_STRING prop_id,const string value)
  {
//--- checking
   if(m_chart_id<=0) return(false);
//---
   return(ChartSetString(m_chart_id,prop_id,value));
  }
//+------------------------------------------------------------------+
//| Access functions ChartSetSymbolPeriod(...).                      |
//| INPUT:  symbol  -new symbol,                                     |
//|         period  -new period.                                     |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CChart::SetSymbolPeriod(const string symbol,ENUM_TIMEFRAMES period)
  {
//--- checking
   if(m_chart_id<=0) return(false);
//---
   return(ChartSetSymbolPeriod(m_chart_id,symbol,period));
  }
//+------------------------------------------------------------------+
//| Access functions ChartApplyTemplate(...).                        |
//| INPUT:  filename -template filename.                             |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CChart::ApplyTemplate(const string filename)
  {
//--- checking
   if(m_chart_id<=0) return(false);
//---
   return(ChartApplyTemplate(m_chart_id,filename));
  }
//+------------------------------------------------------------------+
//| Access functions ChartScreenShot(...).                           |
//| INPUT:  filename  -filename for the screenshot,                  |
//|         width     -screenshot width in pixels,                   |
//|         height    -screenshot height in pixels,                  |
//|         align_mode-output mode narrow screenshot.                |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CChart::ScreenShot(const string filename,int width,int height,ENUM_ALIGN_MODE align_mode) const
  {
//--- checking
   if(m_chart_id<=0) return(false);
//---
   return(ChartScreenShot(m_chart_id,filename,width,height,align_mode));
  }
//+------------------------------------------------------------------+
//| Access functions WindowOnDropped().                              |
//| INPUT:  no.                                                      |
//| OUTPUT: number of subwindows (0-main window)).                   |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
int CChart::WindowOnDropped() const
  {
//--- checking
   if(m_chart_id<=0) return(0);
//---
   return(ChartWindowOnDropped());
  }
//+------------------------------------------------------------------+
//| Access functions PriceOnDropped().                               |
//| INPUT:  no.                                                      |
//| OUTPUT: value of the price (or in the unit oscillators).         |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
double CChart::PriceOnDropped() const
  {
//--- checking
   if(m_chart_id<=0) return(EMPTY_VALUE);
//---
   return(ChartPriceOnDropped());
  }
//+------------------------------------------------------------------+
//| Access functions TimeOnDropped().                                |
//| INPUT:  no.                                                      |
//| OUTPUT: time value.                                              |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
datetime CChart::TimeOnDropped() const
  {
//--- checking
   if(m_chart_id<=0) return(false);
//---
   return(ChartTimeOnDropped());
  }
//+------------------------------------------------------------------+
//| Access functions XOnDropped().                                   |
//| INPUT:  no.                                                      |
//| OUTPUT: the coordinate X.                                        |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
int CChart::XOnDropped() const
  {
//--- checking
   if(m_chart_id<=0) return(0);
//---
   return(ChartXOnDropped());
  }
//+------------------------------------------------------------------+
//| Access functions YOnDropped().                                   |
//| INPUT:  no.                                                      |
//| OUTPUT: the coordinate Y.                                        |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
int CChart::YOnDropped() const
  {
//--- checking
   if(m_chart_id<=0) return(0);
//---
   return(ChartYOnDropped());
  }
//+------------------------------------------------------------------+
//| Writing chart parameters to file.                                |
//| INPUT:  file_handle -handle previously opened for writing file.  |
//| OUTPUT: true if OK, else false.                                  |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CChart::Save(int file_handle)
  {
   bool   resutl;
   string work_str;
   int    work_int;
//--- checking
   if(file_handle<=0) return(false);
   if(m_chart_id<=0)  return(false);
//--- writing
//--- writing begin marker - 0xFFFFFFFFFFFFFFFF
   if(FileWriteLong(file_handle,-1)!=sizeof(long)) return(false);
//--- writing chart type
   if(FileWriteInteger(file_handle,Type(),INT_VALUE)!=INT_VALUE) return(false);
//--- writing chart symbol
   work_str=Symbol();
   work_int=StringLen(work_str);
   if(FileWriteInteger(file_handle,work_int,INT_VALUE)!=INT_VALUE) return(false);
   if(work_int!=0) if(FileWriteString(file_handle,work_str,work_int)!=work_int) return(false);
//--- writing chart period
   if(FileWriteInteger(file_handle,Period(),INT_VALUE)!=sizeof(int)) return(false);
//--- writing the property value "Mode"
   if(FileWriteInteger(file_handle,(int)ChartGetInteger(m_chart_id,CHART_MODE),INT_VALUE)!=sizeof(int)) return(false);
//--- writing the property value "Foreground"
   if(FileWriteInteger(file_handle,(int)ChartGetInteger(m_chart_id,CHART_FOREGROUND),CHAR_VALUE)!=sizeof(char)) return(false);
//--- writing the property value "Shift"
   if(FileWriteInteger(file_handle,(int)ChartGetInteger(m_chart_id,CHART_SHIFT),CHAR_VALUE)!=sizeof(char)) return(false);
//--- writing the property value "ShiftSize"
   if(FileWriteInteger(file_handle,(int)ChartGetInteger(m_chart_id,CHART_SHIFT),INT_VALUE)!=sizeof(int)) return(false);
//--- writing the property value "AutoScroll"
   if(FileWriteInteger(file_handle,(int)ChartGetInteger(m_chart_id,CHART_AUTOSCROLL),CHAR_VALUE)!=sizeof(char)) return(false);
//--- writing the property value "Scale"
   if(FileWriteInteger(file_handle,(int)ChartGetInteger(m_chart_id,CHART_SCALE),INT_VALUE)!=sizeof(int)) return(false);
//--- writing the property value "ScaleFix"
   if(FileWriteInteger(file_handle,(int)ChartGetInteger(m_chart_id,CHART_SCALEFIX),CHAR_VALUE)!=sizeof(char)) return(false);
//--- writing the property value "ScaleFix_11"
   if(FileWriteInteger(file_handle,(int)ChartGetInteger(m_chart_id,CHART_SCALEFIX_11),CHAR_VALUE)!=sizeof(char)) return(false);
//--- writing the property value "FixedMax"
   if(FileWriteDouble(file_handle,ChartGetDouble(m_chart_id,CHART_FIXED_MAX))!=sizeof(double)) return(false);
//--- writing the property value "FixedMin"
   if(FileWriteDouble(file_handle,ChartGetDouble(m_chart_id,CHART_FIXED_MIN))!=sizeof(double)) return(false);
//--- writing the property "ScalePPB"
   if(FileWriteInteger(file_handle,(int)ChartGetInteger(m_chart_id,CHART_SCALE_PT_PER_BAR),CHAR_VALUE)!=sizeof(char)) return(false);
//--- writing the property value "PointsPerBar"
   if(FileWriteDouble(file_handle,ChartGetDouble(m_chart_id,CHART_POINTS_PER_BAR))!=sizeof(double)) return(false);
//--- writing the property value "ShowOHLC"
   if(FileWriteInteger(file_handle,(int)ChartGetInteger(m_chart_id,CHART_SHOW_OHLC),CHAR_VALUE)!=sizeof(char)) return(false);
//--- writing the property value "ShowLineBid"
   if(FileWriteInteger(file_handle,(int)ChartGetInteger(m_chart_id,CHART_SHOW_BID_LINE),CHAR_VALUE)!=sizeof(char)) return(false);
//--- writing the property value "ShowLineAsk"
   if(FileWriteInteger(file_handle,(int)ChartGetInteger(m_chart_id,CHART_SHOW_ASK_LINE),CHAR_VALUE)!=sizeof(char)) return(false);
//--- writing the property value "ShowLastLine"
   if(FileWriteInteger(file_handle,(int)ChartGetInteger(m_chart_id,CHART_SHOW_LAST_LINE),CHAR_VALUE)!=sizeof(char)) return(false);
//--- writing the property value "ShowPeriodSep"
   if(FileWriteInteger(file_handle,(int)ChartGetInteger(m_chart_id,CHART_SHOW_PERIOD_SEP),CHAR_VALUE)!=sizeof(char)) return(false);
//--- writing the property value "ShowGrid"
   if(FileWriteInteger(file_handle,(int)ChartGetInteger(m_chart_id,CHART_SHOW_GRID),CHAR_VALUE)!=sizeof(char)) return(false);
//--- writing the property value "ShowVolumes"
   if(FileWriteInteger(file_handle,(int)ChartGetInteger(m_chart_id,CHART_SHOW_VOLUMES),INT_VALUE)!=sizeof(int)) return(false);
//--- writing the property value "ShowObjectDescr"
   if(FileWriteInteger(file_handle,(int)ChartGetInteger(m_chart_id,CHART_SHOW_OBJECT_DESCR),CHAR_VALUE)!=sizeof(char)) return(false);
//---
   return(true);
  }
//+------------------------------------------------------------------+
//| Reading chart parameters from file.                              |
//| INPUT:  file_handle -handle previously opened for reading file.  |
//| OUTPUT: true if OK, else false.                                  |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CChart::Load(int file_handle)
  {
   bool   resutl;
   string work_str;
   int    work_int;
//--- checking
   if(file_handle<=0) return(false);
   if(m_chart_id<=0)  return(false);
//--- reading
//--- reading and check begin marker - 0xFFFFFFFFFFFFFFFF
   if(FileReadLong(file_handle)!=-1) return(false);
//--- reading and check chart type
   if(FileReadInteger(file_handle,INT_VALUE)!=Type()) return(false);
//--- reading chart symbol
   work_int=FileReadInteger(file_handle);
   if(work_int!=0) work_str=FileReadString(file_handle,work_int);
   else            work_str="";
   SetSymbolPeriod(work_str,(ENUM_TIMEFRAMES)work_int);
//--- reading chart period
   work_int=FileReadInteger(file_handle);
//--- reading the property value "Mode"
   if(!ChartSetInteger(m_chart_id,CHART_MODE,FileReadInteger(file_handle,INT_VALUE))) return(false);
//--- reading the property value "Foreground"
   if(!ChartSetInteger(m_chart_id,CHART_FOREGROUND,FileReadInteger(file_handle,CHAR_VALUE))) return(false);
//--- reading the property value "Shift"
   if(!ChartSetInteger(m_chart_id,CHART_SHIFT,FileReadInteger(file_handle,CHAR_VALUE))) return(false);
//--- reading the property value "ShiftSize"
   if(!ChartSetInteger(m_chart_id,CHART_SHIFT,FileReadInteger(file_handle,INT_VALUE))) return(false);
//--- reading the property value "AutoScroll"
   if(!ChartSetInteger(m_chart_id,CHART_AUTOSCROLL,FileReadInteger(file_handle,CHAR_VALUE))) return(false);
//--- reading the property value "Scale"
   if(!ChartSetInteger(m_chart_id,CHART_SCALE,FileReadInteger(file_handle,INT_VALUE))) return(false);
//--- reading the property value "ScaleFix"
   if(!ChartSetInteger(m_chart_id,CHART_SCALEFIX,FileReadInteger(file_handle,CHAR_VALUE))) return(false);
//--- reading the property value "ScaleFix_11"
   if(!ChartSetInteger(m_chart_id,CHART_SCALEFIX_11,FileReadInteger(file_handle,CHAR_VALUE))) return(false);
//--- reading the property value "FixedMax"
   if(!ChartSetDouble(m_chart_id,CHART_FIXED_MAX,FileReadDatetime(file_handle))) return(false);
//--- reading the property value "FixedMin"
   if(!ChartSetDouble(m_chart_id,CHART_FIXED_MIN,FileReadDatetime(file_handle))) return(false);
//--- reading the property value "ScalePPB"
   if(!ChartSetInteger(m_chart_id,CHART_SCALE_PT_PER_BAR,FileReadInteger(file_handle,CHAR_VALUE))) return(false);
//--- reading the property value "PointsPerBar"
   if(!ChartSetDouble(m_chart_id,CHART_POINTS_PER_BAR,FileReadDatetime(file_handle))) return(false);
//--- reading the property value "ShowOHLC"
   if(!ChartSetInteger(m_chart_id,CHART_SHOW_OHLC,FileReadInteger(file_handle,CHAR_VALUE))) return(false);
//--- reading the property value "ShowLineBid"
   if(!ChartSetInteger(m_chart_id,CHART_SHOW_BID_LINE,FileReadInteger(file_handle,CHAR_VALUE))) return(false);
//--- reading the property value "ShowLineAsk"
   if(!ChartSetInteger(m_chart_id,CHART_SHOW_ASK_LINE,FileReadInteger(file_handle,CHAR_VALUE))) return(false);
//--- reading the property value "ShowLastLine"
   if(!ChartSetInteger(m_chart_id,CHART_SHOW_LAST_LINE,FileReadInteger(file_handle,CHAR_VALUE))) return(false);
//--- reading the property value "ShowPeriodSep"
   if(!ChartSetInteger(m_chart_id,CHART_SHOW_PERIOD_SEP,FileReadInteger(file_handle,CHAR_VALUE))) return(false);
//--- reading the property value "ShowGrid"
   if(!ChartSetInteger(m_chart_id,CHART_SHOW_GRID,FileReadInteger(file_handle,CHAR_VALUE))) return(false);
//--- reading the property value "ShowVolumes"
   if(!ChartSetInteger(m_chart_id,CHART_SHOW_VOLUMES,FileReadInteger(file_handle,INT_VALUE))) return(false);
//--- reading the property value "ShowObjectDescr"
   if(!ChartSetInteger(m_chart_id,CHART_SHOW_OBJECT_DESCR,FileReadInteger(file_handle,CHAR_VALUE))) return(false);
//---
   return(resutl);
  }
//+------------------------------------------------------------------+
