//+------------------------------------------------------------------+
//|                                                   Oscilators.mqh |
//|                        Copyright 2010, MetaQuotes Software Corp. |
//|                                        http://www.metaquotes.net |
//|                                              Revision 2010.02.22 |
//+------------------------------------------------------------------+
#include "Indicator.mqh"
//+------------------------------------------------------------------+
//| Class CiATR.                                                     |
//| Appointment: Class indicator "Average True Range".               |
//+------------------------------------------------------------------+
class CiATR : public CIndicator
  {
protected:
   int               m_ma_period;

public:
                     CiATR();
   //--- methods of access to protected data
   int               MaPeriod() const { return(m_ma_period); }
   //--- method create
   bool              Create(string symbol,ENUM_TIMEFRAMES period,int ma_period);
   //--- methods of access to indicator data
   double            Main(int index);
   //--- method of identifying
   virtual int       Type() const     { return(IND_ATR);     }

protected:
   //--- methods tune
   virtual bool      Initialize(string symbol,ENUM_TIMEFRAMES period,int num_params,MqlParam &params[]);
   bool              Initialize(string symbol,ENUM_TIMEFRAMES period,int ma_period);
  };
//+------------------------------------------------------------------+
//| Constructor CiATR.                                               |
//| INPUT:  no.                                                      |
//| OUTPUT: no.                                                      |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
CiATR::CiATR()
  {
//--- initialize protected data
   m_ma_period=-1;
  }
//+------------------------------------------------------------------+
//| Initialize indicator with the universal parameters.              |
//| INPUT:  symbol   -indicator symbol,                              |
//|         period   -indicator period,                              |
//|         num_param-number of parameters,                          |
//|         params   -array of parameters.                           |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CiATR::Initialize(string symbol,ENUM_TIMEFRAMES period,int num_params,MqlParam &params[])
  {
   return(Initialize(symbol,period,params[0].integer_value));
  }
//+------------------------------------------------------------------+
//| Initialize indicator with the special parameters.                |
//| INPUT:  symbol    -chart symbol,                                 |
//|         period    -chart period,                                 |
//|         ma_period -period MA.                                    |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CiATR::Initialize(string symbol,ENUM_TIMEFRAMES period,int ma_period)
  {
   bool result;
//--- formation status string
   m_name  ="ATR";
   m_status="("+symbol+","+PeriodStr(period)+","+IntegerToString(ma_period)+") H="+IntegerToString(m_handle);
//--- tuning
   result=CreateBuffers(symbol,period,1);
   if(result)
     {
      //--- save settings
      m_ma_period=ma_period;
      //--- create buffers
      ((CIndicatorBuffer*)At(0)).Name("MAIN_LINE");
     }
//---
   return(result);
  }
//+------------------------------------------------------------------+
//| Create indicator "Average True Range".                           |
//| INPUT:  symbol    -chart symbol,                                 |
//|         period    -chart period,                                 |
//|         ma_period -period MA.                                    |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CiATR::Create(string symbol,ENUM_TIMEFRAMES period,int ma_period)
  {
   m_handle=iATR(symbol,period,ma_period);
//---
   if(m_handle!=INVALID_HANDLE)
     {
      //--- idicator creating successful
      if(!Initialize(symbol,period,ma_period))
        {
         //--- initialization failed
         IndicatorRelease(m_handle);
         m_handle=INVALID_HANDLE;
         return(false);
        }
      return(true);
     }
//---
   return(false);
  }
//+------------------------------------------------------------------+
//| Access to buffer of "Average True Range".                        |
//| INPUT:  index -buffer index.                                     |
//| OUTPUT: value of buffer.                                         |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
double CiATR::Main(int index)
  {
   CIndicatorBuffer *buffer=At(0);
//--- checking
   if(buffer==NULL) return(EMPTY_VALUE);
//---
   return(buffer.At(index));
  }
//+------------------------------------------------------------------+
//| Class CiBearsPower.                                              |
//| Appointment: Class indicator "Bears Power".                      |
//+------------------------------------------------------------------+
class CiBearsPower : public CIndicator
  {
protected:
   int               m_ma_period;

public:
                     CiBearsPower();
   //--- methods of access to protected data
   int               MaPeriod() const { return(m_ma_period); }
   //--- method create
   bool              Create(string symbol,ENUM_TIMEFRAMES period,int ma_period);
   //--- methods of access to indicator data
   double            Main(int index);
   //--- method of identifying
   virtual int       Type() const     { return(IND_BEARS);   }

protected:
   //--- methods tune
   virtual bool      Initialize(string symbol,ENUM_TIMEFRAMES period,int num_params,MqlParam &params[]);
   bool              Initialize(string symbol,ENUM_TIMEFRAMES period,int ma_period);
  };
//+------------------------------------------------------------------+
//| Constructor CiBearsPower.                                        |
//| INPUT:  no.                                                      |
//| OUTPUT: no.                                                      |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
CiBearsPower::CiBearsPower()
  {
//--- initialize protected data
   m_ma_period=-1;
  }
//+------------------------------------------------------------------+
//| Initialize indicator with the universal parameters.              |
//| INPUT:  symbol   -indicator symbol,                              |
//|         period   -indicator period,                              |
//|         num_param-number of parameters,                          |
//|         params   -array of parameters.                           |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CiBearsPower::Initialize(string symbol,ENUM_TIMEFRAMES period,int num_params,MqlParam &params[])
  {
   return(Initialize(symbol,period,params[0].integer_value));
  }
//+------------------------------------------------------------------+
//| Initialize indicator with the special parameters.                |
//| INPUT:  symbol    -chart symbol,                                 |
//|         period    -chart period,                                 |
//|         ma_period -period MA.                                    |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CiBearsPower::Initialize(string symbol,ENUM_TIMEFRAMES period,int ma_period)
  {
   bool result;
//--- formation status string
   m_name  ="BearsPower";
   m_status="("+symbol+","+PeriodStr(period)+","+IntegerToString(ma_period)+") H="+IntegerToString(m_handle);
//--- tuning
   result=CreateBuffers(symbol,period,1);
   if(result)
     {
      //--- save settings
      m_ma_period=ma_period;
      //--- create buffers
      ((CIndicatorBuffer*)At(0)).Name("MAIN_LINE");
     }
//---
   return(result);
  }
//+------------------------------------------------------------------+
//| Create indicator "Bears Power".                                  |
//| INPUT:  symbol    -chart symbol,                                 |
//|         period    -chart period,                                 |
//|         ma_period -period MA.                                    |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CiBearsPower::Create(string symbol,ENUM_TIMEFRAMES period,int ma_period)
  {
   m_handle=iBearsPower(symbol,period,ma_period);
//---
   if(m_handle!=INVALID_HANDLE)
     {
      //--- idicator creating successful
      if(!Initialize(symbol,period,ma_period))
        {
         //--- initialization failed
         IndicatorRelease(m_handle);
         m_handle=INVALID_HANDLE;
         return(false);
        }
      return(true);
     }
//---
   return(false);
  }
//+------------------------------------------------------------------+
//| Access to buffer of "Bears Power".                               |
//| INPUT:  index -buffer index.                                     |
//| OUTPUT: value of buffer.                                         |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
double CiBearsPower::Main(int index)
  {
   CIndicatorBuffer *buffer=At(0);
//--- checking
   if(buffer==NULL) return(EMPTY_VALUE);
//---
   return(buffer.At(index));
  }
//+------------------------------------------------------------------+
//| Class CiBullsPower.                                              |
//| Appointment: Class indicator "Bulls Power".                      |
//+------------------------------------------------------------------+
class CiBullsPower : public CIndicator
  {
protected:
   int               m_ma_period;

public:
                     CiBullsPower();
   //--- methods of access to protected data
   int               MaPeriod() const { return(m_ma_period); }
   //--- method create
   bool              Create(string symbol,ENUM_TIMEFRAMES period,int ma_period);
   //--- methods of access to indicator data
   double            Main(int index);
   //--- method of identifying
   virtual int       Type() const     { return(IND_BULLS);   }

protected:
   //--- methods tune
   virtual bool      Initialize(string symbol,ENUM_TIMEFRAMES period,int num_params,MqlParam &params[]);
   bool              Initialize(string symbol,ENUM_TIMEFRAMES period,int ma_period);
  };
//+------------------------------------------------------------------+
//| Constructor CiBullsPower.                                        |
//| INPUT:  no.                                                      |
//| OUTPUT: no.                                                      |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
CiBullsPower::CiBullsPower()
  {
//--- initialize protected data
   m_ma_period=-1;
  }
//+------------------------------------------------------------------+
//| Initialize indicator with the universal parameters.              |
//| INPUT:  symbol   -indicator symbol,                              |
//|         period   -indicator period,                              |
//|         num_param-number of parameters,                          |
//|         params   -array of parameters.                           |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CiBullsPower::Initialize(string symbol,ENUM_TIMEFRAMES period,int num_params,MqlParam &params[])
  {
   return(Initialize(symbol,period,params[0].integer_value));
  }
//+------------------------------------------------------------------+
//| Initialize indicator with the special parameters.                |
//| INPUT:  symbol    -chart symbol,                                 |
//|         period    -chart period,                                 |
//|         ma_period -period MA.                                    |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CiBullsPower::Initialize(string symbol,ENUM_TIMEFRAMES period,int ma_period)
  {
   bool result;
//--- formation status string
   m_name  ="BullsPower";
   m_status="("+symbol+","+PeriodStr(period)+","+IntegerToString(ma_period)+") H="+IntegerToString(m_handle);
//--- tuning
   result=CreateBuffers(symbol,period,1);
   if(result)
     {
      //--- save settings
      m_ma_period=ma_period;
      //--- create buffers
      ((CIndicatorBuffer*)At(0)).Name("MAIN_LINE");
     }
//---
   return(result);
  }
//+------------------------------------------------------------------+
//| Create indicator "Bulls Power".                                  |
//| INPUT:  symbol    -chart symbol,                                 |
//|         period    -chart period,                                 |
//|         ma_period -period MA.                                    |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CiBullsPower::Create(string symbol,ENUM_TIMEFRAMES period,int ma_period)
  {
   m_handle=iBullsPower(symbol,period,ma_period);
//---
   if(m_handle!=INVALID_HANDLE)
     {
      //--- idicator creating successful
      if(!Initialize(symbol,period,ma_period))
        {
         //--- initialization failed
         IndicatorRelease(m_handle);
         m_handle=INVALID_HANDLE;
         return(false);
        }
      return(true);
     }
//---
   return(false);
  }
//+------------------------------------------------------------------+
//| Access to buffer of "Bulls Power".                               |
//| INPUT:  index -buffer index.                                     |
//| OUTPUT: value of buffer.                                         |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
double CiBullsPower::Main(int index)
  {
   CIndicatorBuffer *buffer=At(0);
//--- checking
   if(buffer==NULL) return(EMPTY_VALUE);
//---
   return(buffer.At(index));
  }
//+------------------------------------------------------------------+
//| Class CiCCI.                                                     |
//| Appointment: Class indicator "Commodity Channel Index".          |
//+------------------------------------------------------------------+
class CiCCI : public CIndicator
  {
protected:
   int               m_ma_period;
   int               m_applied;

public:
                     CiCCI();
   //--- methods of access to protected data
   int               MaPeriod() const { return(m_ma_period); }
   int               Applied() const  { return(m_applied);   }
   //--- method create
   bool              Create(string symbol,ENUM_TIMEFRAMES period,int ma_period,int applied);
   //--- methods of access to indicator data
   double            Main(int index);
   //--- method of identifying
   virtual int       Type() const     { return(IND_CCI);     }

protected:
   //--- methods tune
   virtual bool      Initialize(string symbol,ENUM_TIMEFRAMES period,int num_params,MqlParam &params[]);
   bool              Initialize(string symbol,ENUM_TIMEFRAMES period,int ma_period,int applied);
  };
//+------------------------------------------------------------------+
//| Constructor CiCCI.                                               |
//| INPUT:  no.                                                      |
//| OUTPUT: no.                                                      |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
CiCCI::CiCCI()
  {
//--- initialize protected data
   m_ma_period=-1;
   m_applied  =-1;
  }
//+------------------------------------------------------------------+
//| Initialize indicator with the universal parameters.              |
//| INPUT:  symbol   -indicator symbol,                              |
//|         period   -indicator period,                              |
//|         num_param-number of parameters,                          |
//|         params   -array of parameters.                           |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CiCCI::Initialize(string symbol,ENUM_TIMEFRAMES period,int num_params,MqlParam &params[])
  {
   return(Initialize(symbol,period,params[0].integer_value,params[1].integer_value));
  }
//+------------------------------------------------------------------+
//| Create indicator "Commodity Channel Index".                      |
//| INPUT:  symbol    -chart symbol,                                 |
//|         period    -chart period,                                 |
//|         ma_period -period MA,                                    |
//|         applied   -what used.                                    |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CiCCI::Initialize(string symbol,ENUM_TIMEFRAMES period,int ma_period,int applied)
  {
   bool result;
//--- formation status string
   m_name  ="CCI";
   m_status="("+symbol+","+PeriodStr(period)+","+
             IntegerToString(ma_period)+","+PriceStr(applied)+") H="+IntegerToString(m_handle);
//--- tuning
   result=CreateBuffers(symbol,period,1);
   if(result)
     {
      //--- save settings
      m_ma_period=ma_period;
      m_applied  =applied;
      //--- create buffers
      ((CIndicatorBuffer*)At(0)).Name("MAIN_LINE");
     }
//---
   return(result);
  }
//+------------------------------------------------------------------+
//| Create indicator "Commodity Channel Index".                      |
//| INPUT:  symbol    -chart symbol,                                 |
//|         period    -chart period,                                 |
//|         ma_period -period MA,                                    |
//|         applied   -what used.                                    |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CiCCI::Create(string symbol,ENUM_TIMEFRAMES period,int ma_period,int applied)
  {
   m_handle=iCCI(symbol,period,ma_period,applied);
//---
   if(m_handle!=INVALID_HANDLE)
     {
      //--- idicator creating successful
      if(!Initialize(symbol,period,ma_period,applied))
        {
         //--- initialization failed
         IndicatorRelease(m_handle);
         m_handle=INVALID_HANDLE;
         return(false);
        }
      return(true);
     }
//---
   return(false);
  }
//+------------------------------------------------------------------+
//| Access to buffer of "Commodity Channel Index".                   |
//| INPUT:  index -buffer index.                                     |
//| OUTPUT: value of buffer.                                         |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
double CiCCI::Main(int index)
  {
   CIndicatorBuffer *buffer=At(0);
//--- checking
   if(buffer==NULL) return(EMPTY_VALUE);
//---
   return(buffer.At(index));
  }
//+------------------------------------------------------------------+
//| Class CiChaikin.                                                 |
//| Appointment: Class indicator "Chaikin Oscillator".               |
//+------------------------------------------------------------------+
class CiChaikin : public CIndicator
  {
protected:
   int               m_fast_ma_period;
   int               m_slow_ma_period;
   ENUM_MA_METHOD    m_ma_method;
   ENUM_APPLIED_VOLUME m_applied;

public:
                     CiChaikin();
   //--- methods of access to protected data
   int               FastMaPeriod() const { return(m_fast_ma_period); }
   int               SlowMaPeriod() const { return(m_slow_ma_period); }
   ENUM_MA_METHOD    MaMethod() const     { return(m_ma_method);      }
   ENUM_APPLIED_VOLUME Applied() const    { return(m_applied);        }
   //--- method create
   bool              Create(string symbol,ENUM_TIMEFRAMES period,int fast_ma_period,int slow_ma_period,ENUM_MA_METHOD ma_method,ENUM_APPLIED_VOLUME applied);
   //--- methods of access to indicator data
   double            Main(int index);
   //--- method of identifying
   virtual int       Type() const         { return(IND_CHAIKIN);      }

protected:
   //--- methods tune
   virtual bool      Initialize(string symbol,ENUM_TIMEFRAMES period,int num_params,MqlParam &params[]);
   bool              Initialize(string symbol,ENUM_TIMEFRAMES period,int fast_ma_period,int slow_ma_period,ENUM_MA_METHOD ma_method,ENUM_APPLIED_VOLUME applied);
  };
//+------------------------------------------------------------------+
//| Constructor CiChaikin.                                           |
//| INPUT:  no.                                                      |
//| OUTPUT: no.                                                      |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
CiChaikin::CiChaikin()
  {
//--- initialize protected data
   m_fast_ma_period=-1;
   m_slow_ma_period=-1;
   m_ma_method     =WRONG_VALUE;
   m_applied       =WRONG_VALUE;
  }
//+------------------------------------------------------------------+
//| Initialize indicator with the universal parameters.              |
//| INPUT:  symbol   -indicator symbol,                              |
//|         period   -indicator period,                              |
//|         num_param-number of parameters,                          |
//|         params   -array of parameters.                           |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CiChaikin::Initialize(string symbol,ENUM_TIMEFRAMES period,int num_params,MqlParam &params[])
  {
   return(Initialize(symbol,period,params[0].integer_value,params[1].integer_value,
                   (ENUM_MA_METHOD)params[2].integer_value,(ENUM_APPLIED_VOLUME)params[3].integer_value));
  }
//+------------------------------------------------------------------+
//| Initialize indicator with the special parameters.                |
//| INPUT:  symbol         -chart symbol,                            |
//|         period         -chart period,                            |
//|         fast_ma_period -period fast MA,                          |
//|         slow_ma_period -period slow MA,                          |
//|         ma_method      -averaging method MA,                     |
//|         applied        -what used.                               |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CiChaikin::Initialize(string symbol,ENUM_TIMEFRAMES period,int fast_ma_period,int slow_ma_period,ENUM_MA_METHOD ma_method,ENUM_APPLIED_VOLUME applied)
  {
   bool result;
//--- formation status string
   m_name  ="Chaikin";
   m_status="("+symbol+","+PeriodStr(period)+","+
             IntegerToString(fast_ma_period)+","+IntegerToString(slow_ma_period)+","+
             MethodStr(ma_method)+","+PriceStr(applied)+") H="+IntegerToString(m_handle);
//--- tuning
   result=CreateBuffers(symbol,period,1);
   if(result)
     {
      //--- save settings
      m_fast_ma_period=fast_ma_period;
      m_slow_ma_period=slow_ma_period;
      m_ma_method     =ma_method;
      m_applied       =applied;
      //--- create buffers
      ((CIndicatorBuffer*)At(0)).Name("MAIN_LINE");
     }
//---
   return(result);
  }
//+------------------------------------------------------------------+
//| Create indicator "Chaikin Oscillator".                           |
//| INPUT:  symbol         -chart symbol,                            |
//|         period         -chart period,                            |
//|         fast_ma_period -period fast MA,                          |
//|         slow_ma_period -period slow MA,                          |
//|         ma_method      -averaging method MA,                     |
//|         applied        -what used.                               |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CiChaikin::Create(string symbol,ENUM_TIMEFRAMES period,int fast_ma_period,int slow_ma_period,ENUM_MA_METHOD ma_method,ENUM_APPLIED_VOLUME applied)
  {
   m_handle=iChaikin(symbol,period,fast_ma_period,slow_ma_period,ma_method,applied);
//---
   if(m_handle!=INVALID_HANDLE)
     {
      //--- idicator creating successful
      if(!Initialize(symbol,period,fast_ma_period,slow_ma_period,ma_method,applied))
        {
         //--- initialization failed
         IndicatorRelease(m_handle);
         m_handle=INVALID_HANDLE;
         return(false);
        }
      return(true);
     }
//---
   return(false);
  }
//+------------------------------------------------------------------+
//| Access to buffer of "Chaikin Oscillator".                        |
//| INPUT:  index -buffer index.                                     |
//| OUTPUT: value of buffer.                                         |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
double CiChaikin::Main(int index)
  {
   CIndicatorBuffer *buffer=At(0);
//--- checking
   if(buffer==NULL) return(EMPTY_VALUE);
//---
   return(buffer.At(index));
  }
//+------------------------------------------------------------------+
//| Class CiDeMarker.                                                |
//| Appointment: Class indicator "DeMarker".                         |
//+------------------------------------------------------------------+
class CiDeMarker : public CIndicator
  {
protected:
   int               m_ma_period;

public:
                     CiDeMarker();
   //--- methods of access to protected data
   int               MaPeriod() const { return(m_ma_period);  }
   //--- method create
   bool              Create(string symbol,ENUM_TIMEFRAMES period,int ma_period);
   //--- methods of access to indicator data
   double            Main(int index);
   //--- method of identifying
   virtual int       Type() const     { return(IND_DEMARKER); }

protected:
   //--- methods tune
   virtual bool      Initialize(string symbol,ENUM_TIMEFRAMES period,int num_params,MqlParam &params[]);
   bool              Initialize(string symbol,ENUM_TIMEFRAMES period,int ma_period);
  };
//+------------------------------------------------------------------+
//| Constructor CiDeMarker.                                          |
//| INPUT:  no.                                                      |
//| OUTPUT: no.                                                      |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
CiDeMarker::CiDeMarker()
  {
//--- initialize protected data
   m_ma_period=-1;
  }
//+------------------------------------------------------------------+
//| Initialize indicator with the universal parameters.              |
//| INPUT:  symbol   -indicator symbol,                              |
//|         period   -indicator period,                              |
//|         num_param-number of parameters,                          |
//|         params   -array of parameters.                           |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CiDeMarker::Initialize(string symbol,ENUM_TIMEFRAMES period,int num_params,MqlParam &params[])
  {
   return(Initialize(symbol,period,params[0].integer_value));
  }
//+------------------------------------------------------------------+
//| Initialize indicator with the special parameters.                |
//| INPUT:  symbol    -chart symbol,                                 |
//|         period    -chart period,                                 |
//|         ma_period -period MA.                                    |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CiDeMarker::Initialize(string symbol,ENUM_TIMEFRAMES period,int ma_period)
  {
   bool result;
//--- formation status string
   m_name  ="DeMarker";
   m_status="("+symbol+","+PeriodStr(period)+","+IntegerToString(ma_period)+") H="+IntegerToString(m_handle);
//--- tuning
   result=CreateBuffers(symbol,period,1);
   if(result)
     {
      //--- save settings
      m_ma_period=ma_period;
      //--- create buffers
      ((CIndicatorBuffer*)At(0)).Name("MAIN_LINE");
     }
//---
   return(result);
  }
//+------------------------------------------------------------------+
//| Create indicator "DeMarker".                                     |
//| INPUT:  symbol    -chart symbol,                                 |
//|         period    -chart period,                                 |
//|         ma_period -period MA.                                    |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CiDeMarker::Create(string symbol,ENUM_TIMEFRAMES period,int ma_period)
  {
   m_handle=iDeMarker(symbol,period,ma_period);
//---
   if(m_handle!=INVALID_HANDLE)
     {
      //--- idicator creating successful
      if(!Initialize(symbol,period,ma_period))
        {
         //--- initialization failed
         IndicatorRelease(m_handle);
         m_handle=INVALID_HANDLE;
         return(false);
        }
      return(true);
     }
//---
   return(false);
  }
//+------------------------------------------------------------------+
//| Access to buffer of "DeMarker".                                  |
//| INPUT:  index -buffer index.                                     |
//| OUTPUT: value of buffer.                                         |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
double CiDeMarker::Main(int index)
  {
   CIndicatorBuffer *buffer=At(0);
//--- checking
   if(buffer==NULL) return(EMPTY_VALUE);
//---
   return(buffer.At(index));
  }
//+------------------------------------------------------------------+
//| Class CiForce.                                                   |
//| Appointment: Class indicator "Force Index".                      |
//+------------------------------------------------------------------+
class CiForce : public CIndicator
  {
protected:
   int               m_ma_period;
   ENUM_MA_METHOD    m_ma_method;
   ENUM_APPLIED_VOLUME m_applied;

public:
                     CiForce();
   //--- methods of access to protected data
   int               MaPeriod() const  { return(m_ma_period); }
   ENUM_MA_METHOD    MaMethod() const  { return(m_ma_method); }
   ENUM_APPLIED_VOLUME Applied() const { return(m_applied);   }
   //--- method create
   bool              Create(string symbol,ENUM_TIMEFRAMES period,int ma_period,ENUM_MA_METHOD ma_method,ENUM_APPLIED_VOLUME applied);
   //--- methods of access to indicator data
   double            Main(int index);
   //--- method of identifying
   virtual int       Type() const      { return(IND_FORCE);   }

protected:
   //--- methods tune
   virtual bool      Initialize(string symbol,ENUM_TIMEFRAMES period,int num_params,MqlParam &params[]);
   bool              Initialize(string symbol,ENUM_TIMEFRAMES period,int ma_period,ENUM_MA_METHOD ma_method,ENUM_APPLIED_VOLUME applied);
  };
//+------------------------------------------------------------------+
//| Constructor CiForce.                                             |
//| INPUT:  no.                                                      |
//| OUTPUT: no.                                                      |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
CiForce::CiForce()
  {
//--- initialize protected data
   m_ma_period=-1;
   m_ma_method=WRONG_VALUE;
   m_applied  =WRONG_VALUE;
  }
//+------------------------------------------------------------------+
//| Initialize indicator with the universal parameters.              |
//| INPUT:  symbol   -indicator symbol,                              |
//|         period   -indicator period,                              |
//|         num_param-number of parameters,                          |
//|         params   -array of parameters.                           |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CiForce::Initialize(string symbol,ENUM_TIMEFRAMES period,int num_params,MqlParam &params[])
  {
   return(Initialize(symbol,period,params[0].integer_value,(ENUM_MA_METHOD)params[1].integer_value,
              (ENUM_APPLIED_VOLUME)params[2].integer_value));
  }
//+------------------------------------------------------------------+
//| Initialize indicator with the special parameters.                |
//| INPUT:  symbol    -chart symbol,                                 |
//|         period    -chart period,                                 |
//|         ma_period -period MA,                                    |
//|         ma_method -averaging method MA,                          |
//|         applied   -what used.                                    |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CiForce::Initialize(string symbol,ENUM_TIMEFRAMES period,int ma_period,ENUM_MA_METHOD ma_method,ENUM_APPLIED_VOLUME applied)
  {
   bool result;
//--- formation status string
   m_name  ="Force";
   m_status="("+symbol+","+PeriodStr(period)+","+
             IntegerToString(ma_period)+","+MethodStr(ma_method)+","+
             VolumeStr(applied)+","+") H="+IntegerToString(m_handle);
//--- tuning
   result=CreateBuffers(symbol,period,1);
   if(result)
     {
      //--- save settings
      m_ma_period=ma_period;
      m_ma_method=ma_method;
      m_applied  =applied;
      //--- create buffers
      ((CIndicatorBuffer*)At(0)).Name("MAIN_LINE");
     }
//---
   return(result);
  }
//+------------------------------------------------------------------+
//| Create indicator "Force Index".                                  |
//| INPUT:  symbol    -chart symbol,                                 |
//|         period    -chart period,                                 |
//|         ma_period -period MA,                                    |
//|         ma_method -averaging method MA,                          |
//|         applied   -what used.                                    |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CiForce::Create(string symbol,ENUM_TIMEFRAMES period,int ma_period,ENUM_MA_METHOD ma_method,ENUM_APPLIED_VOLUME applied)
  {
   m_handle=iForce(symbol,period,ma_period,ma_method,applied);
//---
   if(m_handle!=INVALID_HANDLE)
     {
      //--- idicator creating successful
      if(!Initialize(symbol,period,ma_period,ma_method,applied))
        {
         //--- initialization failed
         IndicatorRelease(m_handle);
         m_handle=INVALID_HANDLE;
         return(false);
        }
      return(true);
     }
//---
   return(false);
  }
//+------------------------------------------------------------------+
//| Access to buffer of "Force Index".                               |
//| INPUT:  index -buffer index.                                     |
//| OUTPUT: value of buffer.                                         |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
double CiForce::Main(int index)
  {
   CIndicatorBuffer *buffer=At(0);
//--- checking
   if(buffer==NULL) return(EMPTY_VALUE);
//---
   return(buffer.At(index));
  }
//+------------------------------------------------------------------+
//| Class CiMACD.                                                    |
//| Appointment: Class indicator "Moving Averages                    |
//|             Convergence-Divergence".                             |
//+------------------------------------------------------------------+
class CiMACD : public CIndicator
  {
protected:
   int               m_fast_ema_period;
   int               m_slow_ema_period;
   int               m_signal_period;
   int               m_applied;

public:
                     CiMACD();
   //--- methods of access to protected data
   int               FastEmaPeriod() const { return(m_fast_ema_period); }
   int               SlowEmaPeriod() const { return(m_slow_ema_period); }
   int               SignalPeriod() const  { return(m_signal_period);   }
   int               Applied() const       { return(m_applied);         }
   //--- method create
   bool              Create(string symbol,ENUM_TIMEFRAMES period,int fast_ema_period,int slow_ema_period,int signal_period,int applied);
   //--- methods of access to indicator data
   double            Main(int index);
   double            Signal(int index);
   //--- method of identifying
   virtual int       Type() const          { return(IND_MACD);          }

protected:
   //--- methods tune
   virtual bool      Initialize(string symbol,ENUM_TIMEFRAMES period,int num_params,MqlParam &params[]);
   bool              Initialize(string symbol,ENUM_TIMEFRAMES period,int fast_ema_period,int slow_ema_period,int signal_period,int applied);
  };
//+------------------------------------------------------------------+
//| Constructor CiMACD.                                              |
//| INPUT:  no.                                                      |
//| OUTPUT: no.                                                      |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
CiMACD::CiMACD()
  {
//--- initialize protected data
   m_fast_ema_period=-1;
   m_slow_ema_period=-1;
   m_signal_period  =-1;
   m_applied        =-1;
  }
//+------------------------------------------------------------------+
//| Initialize indicator with the universal parameters.              |
//| INPUT:  symbol   -indicator symbol,                              |
//|         period   -indicator period,                              |
//|         num_param-number of parameters,                          |
//|         params   -array of parameters.                           |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CiMACD::Initialize(string symbol,ENUM_TIMEFRAMES period,int num_params,MqlParam &params[])
  {
   return(Initialize(symbol,period,params[0].integer_value,params[1].integer_value,
                                   params[2].integer_value,params[3].integer_value));
  }
//+------------------------------------------------------------------+
//| Initialize indicator with the special parameters.                |
//| INPUT:  symbol          -chart symbol,                           |
//|         period          -chart period,                           |
//|         fast_ema_period -period fast EMA,                        |
//|         slow_ema_period -period slow EMA,                        |
//|         signal_period   -period signal MA,                       |
//|         applied         -what used.                              |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CiMACD::Initialize(string symbol,ENUM_TIMEFRAMES period,int fast_ema_period,int slow_ema_period,int signal_period,int applied)
  {
   bool result;
//--- formation status string
   m_name  ="MACD";
   m_status="("+symbol+","+PeriodStr(period)+","+
             IntegerToString(fast_ema_period)+","+IntegerToString(slow_ema_period)+","+
             IntegerToString(signal_period)+","+PriceStr(applied)+","+") H="+IntegerToString(m_handle);
//--- tuning
   result=CreateBuffers(symbol,period,2);
   if(result)
     {
      //--- save settings
      m_fast_ema_period=fast_ema_period;
      m_slow_ema_period=slow_ema_period;
      m_signal_period  =signal_period;
      m_applied        =applied;
      //--- create buffers
      ((CIndicatorBuffer*)At(0)).Name("MAIN_LINE");
      ((CIndicatorBuffer*)At(1)).Name("SIGNAL_LINE");
     }
//---
   return(result);
  }
//+------------------------------------------------------------------+
//| Create indicator "Moving Averages Convergence-Divergence".       |
//| INPUT:  symbol          -chart symbol,                           |
//|         period          -chart period,                           |
//|         fast_ema_period -period fast EMA,                        |
//|         slow_ema_period -period slow EMA,                        |
//|         signal_period   -period signal MA,                       |
//|         applied         -what used.                              |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CiMACD::Create(string symbol,ENUM_TIMEFRAMES period,int fast_ema_period,int slow_ema_period,int signal_period,int applied)
  {   
   m_handle=iMACD(symbol,period,fast_ema_period,slow_ema_period,signal_period,applied);
//---
   if(m_handle!=INVALID_HANDLE)
     {
      //--- idicator creating successful
      if(!Initialize(symbol,period,fast_ema_period,slow_ema_period,signal_period,applied))
        {
         //--- initialization failed
         IndicatorRelease(m_handle);
         m_handle=INVALID_HANDLE;
         return(false);
        }
      return(true);
     }
//---
   return(false);
  }
//+------------------------------------------------------------------+
//| Access to Main buffer of "Moving Averages                        |
//|                           Convergence-Divergence".               |
//| INPUT:  index -buffer index.                                     |
//| OUTPUT: value of buffer.                                         |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
double CiMACD::Main(int index)
  {
   CIndicatorBuffer *buffer=At(0);
//--- checking
   if(buffer==NULL) return(EMPTY_VALUE);
//---
   return(buffer.At(index));
  }
//+------------------------------------------------------------------+
//| Access to Signel buffer of "Moving Averages                      |
//|                             Convergence-Divergence".             |
//| INPUT:  index -buffer index.                                     |
//| OUTPUT: value of buffer.                                         |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
double CiMACD::Signal(int index)
  {
   CIndicatorBuffer *buffer=At(1);
//--- checking
   if(buffer==NULL) return(EMPTY_VALUE);
//---
   return(buffer.At(index));
  }
//+------------------------------------------------------------------+
//| Class CiMomentum.                                                |
//| Appointment: Class indicator "Momentum".                         |
//+------------------------------------------------------------------+
class CiMomentum : public CIndicator
  {
protected:
   int               m_ma_period;
   int               m_applied;

public:
                     CiMomentum();
   //--- methods of access to protected data
   int               MaPeriod() const { return(m_ma_period);  }
   int               Applied() const  { return(m_applied);    }
   //--- method create
   bool              Create(string symbol,ENUM_TIMEFRAMES period,int ma_period,int applied);
   //--- methods of access to indicator data
   double            Main(int index);
   //--- method of identifying
   virtual int       Type() const     { return(IND_MOMENTUM); }

protected:
   //--- methods tune
   virtual bool      Initialize(string symbol,ENUM_TIMEFRAMES period,int num_params,MqlParam &params[]);
   bool              Initialize(string symbol,ENUM_TIMEFRAMES period,int ma_period,int applied);
  };
//+------------------------------------------------------------------+
//| Constructor CiMomentum.                                          |
//| INPUT:  no.                                                      |
//| OUTPUT: no.                                                      |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
CiMomentum::CiMomentum()
  {
//--- initialize protected data
   m_ma_period=-1;
   m_applied  =-1;
  }
//+------------------------------------------------------------------+
//| Initialize indicator with the universal parameters.              |
//| INPUT:  symbol   -indicator symbol,                              |
//|         period   -indicator period,                              |
//|         num_param-number of parameters,                          |
//|         params   -array of parameters.                           |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CiMomentum::Initialize(string symbol,ENUM_TIMEFRAMES period,int num_params,MqlParam &params[])
  {
   return(Initialize(symbol,period,params[0].integer_value,params[1].integer_value));
  }
//+------------------------------------------------------------------+
//| Initialize indicator with the special parameters.                |
//| INPUT:  symbol    -chart symbol,                                 |
//|         period    -chart period,                                 |
//|         ma_period -period MA,                                    |
//|         applied   -what used.                                    |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CiMomentum::Initialize(string symbol,ENUM_TIMEFRAMES period,int ma_period,int applied)
  {
   bool result;
//--- formation status string
   m_name="Momentum";
   m_status="("+symbol+","+PeriodStr(period)+","+
             IntegerToString(ma_period)+","+PriceStr(applied)+","+") H="+IntegerToString(m_handle);
//--- tuning
   result=CreateBuffers(symbol,period,1);
   if(result)
     {
      //--- save settings
      m_ma_period=ma_period;
      m_applied  =applied;
      //--- create buffers
      ((CIndicatorBuffer*)At(0)).Name("MAIN_LINE");
     }
//---
   return(result);
  }
//+------------------------------------------------------------------+
//| Create indicator "Momentum".                                     |
//| INPUT:  symbol    -chart symbol,                                 |
//|         period    -chart period,                                 |
//|         ma_period -period MA,                                    |
//|         applied   -what used.                                    |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CiMomentum::Create(string symbol,ENUM_TIMEFRAMES period,int ma_period,int applied)
  {
   m_handle=iMomentum(symbol,period,ma_period,applied);
//---
   if(m_handle!=INVALID_HANDLE)
     {
      //--- idicator creating successful
      if(!Initialize(symbol,period,ma_period,applied))
        {
         //--- initialization failed
         IndicatorRelease(m_handle);
         m_handle=INVALID_HANDLE;
         return(false);
        }
      return(true);
     }
//---
   return(false);
  }
//+------------------------------------------------------------------+
//| Access to buffer of "Momentum".                                  |
//| INPUT:  index -buffer index.                                     |
//| OUTPUT: value of buffer.                                         |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
double CiMomentum::Main(int index)
  {
   CIndicatorBuffer *buffer=At(0);
//--- checking
   if(buffer==NULL) return(EMPTY_VALUE);
//---
   return(buffer.At(index));
  }
//+------------------------------------------------------------------+
//| Class CiOsMA.                                                    |
//| Appointment: Class indicator "Moving Average of Oscillator       |
//|             (MACD histogram)".                                   |
//+------------------------------------------------------------------+
class CiOsMA : public CIndicator
  {
protected:
   int               m_fast_ema_period;
   int               m_slow_ema_period;
   int               m_signal_period;
   int               m_applied;

public:
                     CiOsMA();
   //--- methods of access to protected data
   int               FastEmaPeriod() const { return(m_fast_ema_period); }
   int               SlowEmaPeriod() const { return(m_slow_ema_period); }
   int               SignalPeriod() const  { return(m_signal_period);   }
   int               Applied() const       { return(m_applied);         }
   //--- method create
   bool              Create(string symbol,ENUM_TIMEFRAMES period,int fast_ema_period,int slow_ema_period,int signal_period,int applied);
   //--- methods of access to indicator data
   double            Main(int index);
   //--- method of identifying
   virtual int       Type() const          { return(IND_OSMA);          }

protected:
   //--- methods tune
   virtual bool      Initialize(string symbol,ENUM_TIMEFRAMES period,int num_params,MqlParam &params[]);
   bool              Initialize(string symbol,ENUM_TIMEFRAMES period,int fast_ema_period,int slow_ema_period,int signal_period,int applied);
  };
//+------------------------------------------------------------------+
//| Constructor CiOsMA.                                              |
//| INPUT:  no.                                                      |
//| OUTPUT: no.                                                      |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
CiOsMA::CiOsMA()
  {
//--- initialize protected data
   m_fast_ema_period=-1;
   m_slow_ema_period=-1;
   m_signal_period  =-1;
   m_applied        =-1;
  }
//+------------------------------------------------------------------+
//| Initialize indicator with the universal parameters.              |
//| INPUT:  symbol   -indicator symbol,                              |
//|         period   -indicator period,                              |
//|         num_param-number of parameters,                          |
//|         params   -array of parameters.                           |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CiOsMA::Initialize(string symbol,ENUM_TIMEFRAMES period,int num_params,MqlParam &params[])
  {
   return(Initialize(symbol,period,params[0].integer_value,params[1].integer_value,
                                   params[2].integer_value,params[3].integer_value));
  }
//+------------------------------------------------------------------+
//| Initialize indicator with the special parameters.                |
//| INPUT:  symbol          -chart symbol,                           |
//|         period          -chart period,                           |
//|         fast_ema_period -period fast EMA,                        |
//|         slow_ema_period -period slow EMA,                        |
//|         signal_period   -period signal MA,                       |
//|         applied         -what used.                              |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CiOsMA::Initialize(string symbol,ENUM_TIMEFRAMES period,int fast_ema_period,int slow_ema_period,int signal_period,int applied)
  {
   bool result;
//--- formation status string
   m_name  ="OsMA";
   m_status="("+symbol+","+PeriodStr(period)+","+
             IntegerToString(fast_ema_period)+","+IntegerToString(slow_ema_period)+","+
             IntegerToString(signal_period)+","+PriceStr(applied)+","+") H="+IntegerToString(m_handle);
//--- tuning
   result=CreateBuffers(symbol,period,1);
   if(result)
     {
      //--- save settings
      m_fast_ema_period=fast_ema_period;
      m_slow_ema_period=slow_ema_period;
      m_signal_period  =signal_period;
      m_applied        =applied;
      //--- create buffers
      ((CIndicatorBuffer*)At(0)).Name("MAIN_LINE");
     }
//---
   return(result);
  }
//+------------------------------------------------------------------+
//| Create indicator "Moving Average of Oscillator                   |
//| (MACD histogram)".                                               |
//| INPUT:  symbol          -chart symbol,                           |
//|         period          -chart period,                           |
//|         fast_ema_period -period fast EMA,                        |
//|         slow_ema_period -period slow EMA,                        |
//|         signal_period   -period signal MA,                       |
//|         applied         -what used.                              |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CiOsMA::Create(string symbol,ENUM_TIMEFRAMES period,int fast_ema_period,int slow_ema_period,int signal_period,int applied)
  {
   m_handle=iOsMA(symbol,period,fast_ema_period,slow_ema_period,signal_period,applied);
//---
   if(m_handle!=INVALID_HANDLE)
     {
      //--- idicator creating successful
      if(!Initialize(symbol,period,fast_ema_period,slow_ema_period,signal_period,applied))
        {
         //--- initialization failed
         IndicatorRelease(m_handle);
         m_handle=INVALID_HANDLE;
         return(false);
        }
      return(true);
     }
//---
   return(false);
  }
//+------------------------------------------------------------------+
//| Access to buffer of "Moving Average of Oscillator                |
//|                     (MACD histogram)".                           |
//| INPUT:  index -buffer index.                                     |
//| OUTPUT: value of buffer.                                         |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
double CiOsMA::Main(int index)
  {
   CIndicatorBuffer *buffer=At(0);
//--- checking
   if(buffer==NULL) return(EMPTY_VALUE);
//---
   return(buffer.At(index));
  }
//+------------------------------------------------------------------+
//| Class CiRSI.                                                     |
//| Appointment: Class indicator "Relative Strength Index".          |
//+------------------------------------------------------------------+
class CiRSI : public CIndicator
  {
protected:
   int               m_ma_period;
   int               m_applied;

public:
                     CiRSI();
   //--- methods of access to protected data
   int               MaPeriod() const { return(m_ma_period); }
   int               Applied() const  { return(m_applied);   }
   //--- method create
   bool              Create(string symbol,ENUM_TIMEFRAMES period,int ma_period,int applied);
   //--- methods of access to indicator data
   double            Main(int index);
   //--- method of identifying
   virtual int       Type() const     { return(IND_RSI);     }

protected:
   //--- methods tune
   virtual bool      Initialize(string symbol,ENUM_TIMEFRAMES period,int num_params,MqlParam &params[]);
   bool              Initialize(string symbol,ENUM_TIMEFRAMES period,int ma_period,int applied);
  };
//+------------------------------------------------------------------+
//| Constructor CiRSI.                                               |
//| INPUT:  no.                                                      |
//| OUTPUT: no.                                                      |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
CiRSI::CiRSI()
  {
//--- initialize protected data
   m_ma_period=-1;
   m_applied  =-1;
  }
//+------------------------------------------------------------------+
//| Initialize indicator with the universal parameters.              |
//| INPUT:  symbol   -indicator symbol,                              |
//|         period   -indicator period,                              |
//|         num_param-number of parameters,                          |
//|         params   -array of parameters.                           |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CiRSI::Initialize(string symbol,ENUM_TIMEFRAMES period,int num_params,MqlParam &params[])
  {
   return(Initialize(symbol,period,params[0].integer_value,params[1].integer_value));
  }
//+------------------------------------------------------------------+
//| Initialize indicator with the special parameters.                |
//| INPUT:  symbol    -chart symbol,                                 |
//|         period    -chart period,                                 |
//|         ma_period -period MA,                                    |
//|         applied   -what used.                                    |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CiRSI::Initialize(string symbol,ENUM_TIMEFRAMES period,int ma_period,int applied)
  {
   bool result;
//--- formation status string
   m_name  ="RSI";
   m_status="("+symbol+","+PeriodStr(period)+","+
             IntegerToString(ma_period)+","+PriceStr(applied)+","+") H="+IntegerToString(m_handle);
//--- tuning
   result=CreateBuffers(symbol,period,1);
   if(result)
     {
      //--- save settings
      m_ma_period=ma_period;
      m_applied  =applied;
      //--- create buffers
      ((CIndicatorBuffer*)At(0)).Name("MAIN_LINE");
     }
//---
   return(result);
  }
//+------------------------------------------------------------------+
//| Create indicator "Relative Strength Index".                      |
//| INPUT:  symbol    -chart symbol,                                 |
//|         period    -chart period,                                 |
//|         ma_period -period MA,                                    |
//|         applied   -what used.                                    |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CiRSI::Create(string symbol,ENUM_TIMEFRAMES period,int ma_period,int applied)
  {
   m_handle=iRSI(symbol,period,ma_period,applied);
//---
   if(m_handle!=INVALID_HANDLE)
     {
      //--- idicator creating successful
      if(!Initialize(symbol,period,ma_period,applied))
        {
         //--- initialization failed
         IndicatorRelease(m_handle);
         m_handle=INVALID_HANDLE;
         return(false);
        }
      return(true);
     }
//---
   return(false);
  }
//+------------------------------------------------------------------+
//| Access to buffer of "Relative Strength Index".                   |
//| INPUT:  index -buffer index.                                     |
//| OUTPUT: value of buffer.                                         |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
double CiRSI::Main(int index)
  {
   CIndicatorBuffer *buffer=At(0);
//--- checking
   if(buffer==NULL) return(EMPTY_VALUE);
//---
   return(buffer.At(index));
  }
//+------------------------------------------------------------------+
//| Class CiRVI.                                                     |
//| Appointment: Class indicator "Relative Vigor Index".             |
//+------------------------------------------------------------------+
class CiRVI : public CIndicator
  {
protected:
   int               m_ma_period;

public:
                     CiRVI();
   //--- methods of access to protected data
   int               MaPeriod() const { return(m_ma_period); }
   //--- method create
   bool              Create(string symbol,ENUM_TIMEFRAMES period,int ma_period);
   //--- methods of access to indicator data
   double            Main(int index);
   double            Signal(int index);
   //--- method of identifying
   virtual int       Type() const     { return(IND_RVI);     }

protected:
   //--- methods tune
   virtual bool      Initialize(string symbol,ENUM_TIMEFRAMES period,int num_params,MqlParam &params[]);
   bool              Initialize(string symbol,ENUM_TIMEFRAMES period,int ma_period);
  };
//+------------------------------------------------------------------+
//| Constructor CiRVI.                                               |
//| INPUT:  no.                                                      |
//| OUTPUT: no.                                                      |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
CiRVI::CiRVI()
  {
//--- initialize protected data
   m_ma_period=-1;
  }
//+------------------------------------------------------------------+
//| Initialize indicator with the universal parameters.              |
//| INPUT:  symbol   -indicator symbol,                              |
//|         period   -indicator period,                              |
//|         num_param-number of parameters,                          |
//|         params   -array of parameters.                           |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CiRVI::Initialize(string symbol,ENUM_TIMEFRAMES period,int num_params,MqlParam &params[])
  {
   return(Initialize(symbol,period,params[0].integer_value));
  }
//+------------------------------------------------------------------+
//| Initialize indicator with the special parameters.                |
//| INPUT:  symbol    -chart symbol,                                 |
//|         period    -chart period,                                 |
//|         ma_period -period MA.                                    |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CiRVI::Initialize(string symbol,ENUM_TIMEFRAMES period,int ma_period)
  {
   bool result;
//--- formation status string
   m_name  ="RVI";
   m_status="("+symbol+","+PeriodStr(period)+","+
             IntegerToString(ma_period)+") H="+IntegerToString(m_handle);
//--- tuning
   result=CreateBuffers(symbol,period,2);
   if(result)
     {
      //--- save settings
      m_ma_period=ma_period;
      //--- create buffers
      ((CIndicatorBuffer*)At(0)).Name("MAIN_LINE");
      ((CIndicatorBuffer*)At(1)).Name("SIGNAL_LINE");
     }
//---
   return(result);
  }
//+------------------------------------------------------------------+
//| Create indicator "Relative Vigor Index".                         |
//| INPUT:  symbol    -chart symbol,                                 |
//|         period    -chart period,                                 |
//|         ma_period -period MA.                                    |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CiRVI::Create(string symbol,ENUM_TIMEFRAMES period,int ma_period)
  {
   m_handle=iRVI(symbol,period,ma_period);
//---
   if(m_handle!=INVALID_HANDLE)
     {
      //--- idicator creating successful
      if(!Initialize(symbol,period,ma_period))
        {
         //--- initialization failed
         IndicatorRelease(m_handle);
         m_handle=INVALID_HANDLE;
         return(false);
        }
      return(true);
     }
//---
   return(false);
  }
//+------------------------------------------------------------------+
//| Access to Main buffer of "Relative Vigor Index".                 |
//| INPUT:  index -buffer index.                                     |
//| OUTPUT: value of buffer.                                         |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
double CiRVI::Main(int index)
  {
   CIndicatorBuffer *buffer=At(0);
//--- checking
   if(buffer==NULL) return(EMPTY_VALUE);
//---
   return(buffer.At(index));
  }
//+------------------------------------------------------------------+
//| Access to Signal buffer of "Relative Vigor Index".               |
//| INPUT:  index -buffer index.                                     |
//| OUTPUT: value of buffer.                                         |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
double CiRVI::Signal(int index)
  {
   CIndicatorBuffer *buffer=At(1);
//--- checking
   if(buffer==NULL) return(EMPTY_VALUE);
//---
   return(buffer.At(index));
  }
//+------------------------------------------------------------------+
//| Class CiStochastic.                                              |
//| Appointment: Class indicator "Stochastic Oscillator".            |
//+------------------------------------------------------------------+
class CiStochastic : public CIndicator
  {
protected:
   int               m_Kperiod;
   int               m_Dperiod;
   int               m_slowing;
   ENUM_MA_METHOD    m_ma_method;
   ENUM_STO_PRICE    m_price_field;

public:
                     CiStochastic();
   //--- methods of access to protected data
   int               Kperiod() const    { return(m_Kperiod);      }
   int               Dperiod() const    { return(m_Dperiod);      }
   int               Slowing() const    { return(m_slowing);      }
   ENUM_MA_METHOD    MaMethod() const   { return(m_ma_method);    }
   ENUM_STO_PRICE    PriceField() const { return(m_price_field);  }
   //--- method create
   bool              Create(string symbol,ENUM_TIMEFRAMES period,int Kperiod,int Dperiod,int slowing,ENUM_MA_METHOD ma_method,ENUM_STO_PRICE price_field);
   //--- methods of access to indicator data
   double            Main(int index);
   double            Signal(int index);
   //--- method of identifying
   virtual int       Type() const       { return(IND_STOCHASTIC); }

protected:
   //--- methods tune
   virtual bool      Initialize(string symbol,ENUM_TIMEFRAMES period,int num_params,MqlParam &params[]);
   bool              Initialize(string symbol,ENUM_TIMEFRAMES period,int Kperiod,int Dperiod,int slowing,ENUM_MA_METHOD ma_method,ENUM_STO_PRICE price_field);
  };
//+------------------------------------------------------------------+
//| Constructor CiStochastic.                                        |
//| INPUT:  no.                                                      |
//| OUTPUT: no.                                                      |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
CiStochastic::CiStochastic()
  {
//--- initialize protected data
   m_Kperiod    =-1;
   m_Dperiod    =-1;
   m_slowing    =-1;
   m_ma_method  =WRONG_VALUE;
   m_price_field=WRONG_VALUE;
  }
//+------------------------------------------------------------------+
//| Initialize indicator with the universal parameters.              |
//| INPUT:  symbol   -indicator symbol,                              |
//|         period   -indicator period,                              |
//|         num_param-number of parameters,                          |
//|         params   -array of parameters.                           |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CiStochastic::Initialize(string symbol,ENUM_TIMEFRAMES period,int num_params,MqlParam &params[])
  {
   return(Initialize(symbol,period,params[0].integer_value,params[1].integer_value,
                                   params[2].integer_value,(ENUM_MA_METHOD)params[3].integer_value,
                   (ENUM_STO_PRICE)params[4].integer_value));
  }
//+------------------------------------------------------------------+
//| Initialize indicator with the special parameters.                |
//| INPUT:  symbol    -chart symbol,                                 |
//|         period    -chart period,                                 |
//|         Kperiod   -period %K,                                    |
//|         Dperiod   -period %D,                                    |
//|         slowing   -period slowing,                               |
//|         ma_method -averaging method,                             |
//|         price_field -what used.                                  |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CiStochastic::Initialize(string symbol,ENUM_TIMEFRAMES period,int Kperiod,int Dperiod,int slowing,ENUM_MA_METHOD ma_method,ENUM_STO_PRICE price_field)
  {
   bool result;
//--- formation status string
   m_name  ="Stochastic";
   m_status="("+symbol+","+PeriodStr(period)+","+
             IntegerToString(Kperiod)+","+IntegerToString(Dperiod)+","+IntegerToString(slowing)+","+
             MethodStr(ma_method)+","+IntegerToString(price_field)+") H="+IntegerToString(m_handle);
//--- tuning
   result=CreateBuffers(symbol,period,2);
   if(result)
     {
      //--- save settings
      m_Kperiod    =Kperiod;
      m_Dperiod    =Dperiod;
      m_slowing    =slowing;
      m_ma_method  =ma_method;
      m_price_field=price_field;
      //--- create buffers
      ((CIndicatorBuffer*)At(0)).Name("MAIN_LINE");
      ((CIndicatorBuffer*)At(1)).Name("SIGNAL_LINE");
     }
//---
   return(result);
  }
//+------------------------------------------------------------------+
//| Create indicator "Stochastic Oscillator".                        |
//| INPUT:  symbol    -chart symbol,                                 |
//|         period    -chart period,                                 |
//|         Kperiod   -period %K,                                    |
//|         Dperiod   -period %D,                                    |
//|         slowing   -period slowing,                               |
//|         ma_method -averaging method,                             |
//|         price_field -what used.                                  |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CiStochastic::Create(string symbol,ENUM_TIMEFRAMES period,int Kperiod,int Dperiod,int slowing,ENUM_MA_METHOD ma_method,ENUM_STO_PRICE price_field)
  {
   m_handle=iStochastic(symbol,period,Kperiod,Dperiod,slowing,ma_method,price_field);
//---
   if(m_handle!=INVALID_HANDLE)
     {
      //--- idicator creating successful
      if(!Initialize(symbol,period,Kperiod,Dperiod,slowing,ma_method,price_field))
        {
         //--- initialization failed
         IndicatorRelease(m_handle);
         m_handle=INVALID_HANDLE;
         return(false);
        }
      return(true);
     }
//---
   return(false);
  }
//+------------------------------------------------------------------+
//| Access to Main buffer of "Stochastic Oscillator".                |
//| INPUT:  index -buffer index.                                     |
//| OUTPUT: value of buffer.                                         |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
double CiStochastic::Main(int index)
  {
   CIndicatorBuffer *buffer=At(0);
//--- checking
   if(buffer==NULL) return(EMPTY_VALUE);
//---
   return(buffer.At(index));
  }
//+------------------------------------------------------------------+
//| Access to Signal buffer of "Stochastic Oscillator".              |
//| INPUT:  index -buffer index.                                     |
//| OUTPUT: value of buffer.                                         |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
double CiStochastic::Signal(int index)
  {
   CIndicatorBuffer *buffer=At(1);
//--- checking
   if(buffer==NULL) return(EMPTY_VALUE);
//---
   return(buffer.At(index));
  }
//+------------------------------------------------------------------+
//| Class CiWPR.                                                     |
//| Appointment: Class indicator "Williams' Percent Range".          |
//+------------------------------------------------------------------+
class CiWPR : public CIndicator
  {
protected:
   int               m_calc_period;

public:
                     CiWPR();
   //--- methods of access to protected data
   int               CalcPeriod() const { return(m_calc_period); }
   //--- method create
   bool              Create(string symbol,ENUM_TIMEFRAMES period,int calc_period);
   //--- methods of access to indicator data
   double            Main(int index);
   //--- method of identifying
   virtual int       Type() const       { return(IND_WPR);       }

protected:
   //--- methods tune
   virtual bool      Initialize(string symbol,ENUM_TIMEFRAMES period,int num_params,MqlParam &params[]);
   bool              Initialize(string symbol,ENUM_TIMEFRAMES period,int calc_period);
  };
//+------------------------------------------------------------------+
//| Constructor CiWPR.                                               |
//| INPUT:  no.                                                      |
//| OUTPUT: no.                                                      |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
CiWPR::CiWPR()
  {
//--- initialize protected data
   m_calc_period=-1;
  }
//+------------------------------------------------------------------+
//| Initialize indicator with the universal parameters.              |
//| INPUT:  symbol   -indicator symbol,                              |
//|         period   -indicator period,                              |
//|         num_param-number of parameters,                          |
//|         params   -array of parameters.                           |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CiWPR::Initialize(string symbol,ENUM_TIMEFRAMES period,int num_params,MqlParam &params[])
  {
   return(Initialize(symbol,period,params[0].integer_value));
  }
//+------------------------------------------------------------------+
//| Initialize indicator with the cpecial parameters.                |
//| INPUT:  symbol      -chart symbol,                               |
//|         period      -chart period,                               |
//|         calc_period -period.                                     |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CiWPR::Initialize(string symbol,ENUM_TIMEFRAMES period,int calc_period)
  {
   bool result;
//--- formation status string
   m_name  ="WPR";
   m_status="("+symbol+","+PeriodStr(period)+","+
             IntegerToString(calc_period)+") H="+IntegerToString(m_handle);
//--- tuning
   result=CreateBuffers(symbol,period,1);
   if(result)
     {
      //--- save settings
      m_calc_period=calc_period;
      //--- create buffers
      ((CIndicatorBuffer*)At(0)).Name("MAIN_LINE");
     }
//---
   return(result);
  }
//+------------------------------------------------------------------+
//| Create indicator "Williams' Percent Range".                      |
//| INPUT:  symbol      -chart symbol,                               |
//|         period      -chart period,                               |
//|         calc_period -period.                                     |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CiWPR::Create(string symbol,ENUM_TIMEFRAMES period,int calc_period)
  {
   m_handle=iWPR(symbol,period,calc_period);
//---
   if(m_handle!=INVALID_HANDLE)
     {
      //--- idicator creating successful
      if(!Initialize(symbol,period,calc_period))
        {
         //--- initialization failed
         IndicatorRelease(m_handle);
         m_handle=INVALID_HANDLE;
         return(false);
        }
      return(true);
     }
//---
   return(false);
  }
//+------------------------------------------------------------------+
//| Access to buffer of "Williams' Percent Range".                   |
//| INPUT:  index -buffer index.                                     |
//| OUTPUT: value of buffer.                                         |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
double CiWPR::Main(int index)
  {
   CIndicatorBuffer *buffer=At(0);
//--- checking
   if(buffer==NULL) return(EMPTY_VALUE);
//---
   return(buffer.At(index));
  }
//+------------------------------------------------------------------+
//| Class CiTriX.                                                    |
//| Appointment: Class indicator "Triple Exponential Moving Averages |
//|             Oscillator".                                         |
//+------------------------------------------------------------------+
class CiTriX : public CIndicator
  {
protected:
   int               m_ma_period;
   int               m_applied;

public:
                     CiTriX();
   //--- methods of access to protected data
   int               MaPeriod() const { return(m_ma_period); }
   int               Applied() const  { return(m_applied);   }
   //--- method create
   bool              Create(string symbol,ENUM_TIMEFRAMES period,int ma_period,int applied);
   //--- methods of access to indicator data
   double            Main(int index);
   //--- method of identifying
   virtual int       Type() const     { return(IND_TRIX);    }

protected:
   //--- methods tune
   virtual bool      Initialize(string symbol,ENUM_TIMEFRAMES period,int num_params,MqlParam &params[]);
   bool              Initialize(string symbol,ENUM_TIMEFRAMES period,int ma_period,int applied);
  };
//+------------------------------------------------------------------+
//| Constructor CiTriX.                                              |
//| INPUT:  no.                                                      |
//| OUTPUT: no.                                                      |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
CiTriX::CiTriX()
  {
//--- initialize protected data
   m_ma_period=-1;
   m_applied  =-1;
  }
//+------------------------------------------------------------------+
//| Initialize indicator with the universal parameters.              |
//| INPUT:  symbol   -indicator symbol,                              |
//|         period   -indicator period,                              |
//|         num_param-number of parameters,                          |
//|         params   -array of parameters.                           |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CiTriX::Initialize(string symbol,ENUM_TIMEFRAMES period,int num_params,MqlParam &params[])
  {
   return(Initialize(symbol,period,params[0].integer_value,params[1].integer_value));
  }
//+------------------------------------------------------------------+
//| Create indicator "Triple Exponential Moving Averages             |
//| Oscillator".                                                     |
//| INPUT:  symbol    -chart symbol,                                 |
//|         period    -chart period,                                 |
//|         ma_period -period MA,                                    |
//|         applied   -what used.                                    |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CiTriX::Initialize(string symbol,ENUM_TIMEFRAMES period,int ma_period,int applied)
  {
   bool result;
//--- formation status string
   m_name  ="TriX";
   m_status="("+symbol+","+PeriodStr(period)+","+
             IntegerToString(ma_period)+","+PriceStr(applied)+") H="+IntegerToString(m_handle);
//--- tuning
   result=CreateBuffers(symbol,period,1);
   if(result)
     {
      //--- save settings
      m_ma_period=ma_period;
      m_applied  =applied;
      //--- create buffers
      ((CIndicatorBuffer*)At(0)).Name("MAIN_LINE");
     }
//---
   return(result);
  }
//+------------------------------------------------------------------+
//| Create indicator "Triple Exponential Moving Averages             |
//| Oscillator".                                                     |
//| INPUT:  symbol    -chart symbol,                                 |
//|         period    -chart period,                                 |
//|         ma_period -period MA,                                    |
//|         applied   -what used.                                    |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CiTriX::Create(string symbol,ENUM_TIMEFRAMES period,int ma_period,int applied)
  {
   m_handle=iTriX(symbol,period,ma_period,applied);
//---
   if(m_handle!=INVALID_HANDLE)
     {
      //--- idicator creating successful
      if(!Initialize(symbol,period,ma_period,applied))
        {
         //--- initialization failed
         IndicatorRelease(m_handle);
         m_handle=INVALID_HANDLE;
         return(false);
        }
      return(true);
     }
//---
   return(false);
  }
//+------------------------------------------------------------------+
//| Access to buffer of "Triple Exponential Moving Averages          |
//|                      Oscillator".                                |
//| INPUT:  index -buffer index.                                     |
//| OUTPUT: value of buffer.                                         |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
double CiTriX::Main(int index)
  {
   CIndicatorBuffer *buffer=At(0);
//--- checking
   if(buffer==NULL) return(EMPTY_VALUE);
//---
   return(buffer.At(index));
  }
//+------------------------------------------------------------------+
