//+------------------------------------------------------------------+
//|                                                      Volumes.mqh |
//|                        Copyright 2010, MetaQuotes Software Corp. |
//|                                        http://www.metaquotes.net |
//|                                              Revision 2010.02.22 |
//+------------------------------------------------------------------+
#include "Indicator.mqh"
//+------------------------------------------------------------------+
//| Class CiAD.                                                      |
//| Appointment: Class indicator "Accumulation/Distribution".        |
//+------------------------------------------------------------------+
class CiAD : public CIndicator
  {
protected:
   ENUM_APPLIED_VOLUME m_applied;        // applied volume

public:
                     CiAD();
   //--- methods of access to protected data
   ENUM_APPLIED_VOLUME Applied() const { return(m_applied); }
   //--- method create
   bool              Create(string symbol,ENUM_TIMEFRAMES period,ENUM_APPLIED_VOLUME applied);
   //--- methods of access to indicator data
   double            Main(int index);
   //--- method of identifying
   virtual int       Type() const      { return(IND_AD);    }

protected:
   //--- methods tune
   virtual bool      Initialize(string symbol,ENUM_TIMEFRAMES period,int num_params,MqlParam &params[]);
   bool              Initialize(string symbol,ENUM_TIMEFRAMES period,ENUM_APPLIED_VOLUME applied);
  };
//+------------------------------------------------------------------+
//| Constructor CiAD.                                                |
//| INPUT:  no.                                                      |
//| OUTPUT: no.                                                      |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
CiAD::CiAD()
  {
//--- initialize protected data
   m_applied=WRONG_VALUE;
  }
//+------------------------------------------------------------------+
//| Initialize indicator with the universal parameters.              |
//| INPUT:  symbol   -indicator symbol,                              |
//|         period   -indicator period,                              |
//|         num_param-number of parameters,                          |
//|         params   -array of parameters.                           |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CiAD::Initialize(string symbol,ENUM_TIMEFRAMES period,int num_params,MqlParam &params[])
  {
   return(Initialize(symbol,period,(ENUM_APPLIED_VOLUME)params[0].integer_value));
  }
//+------------------------------------------------------------------+
//| Initialize indicator with the special parameters.                |
//| INPUT:  symbol  -chart symbol,                                   |
//|         period  -chart period,                                   |
//|         applied -what used.                                      |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CiAD::Initialize(string symbol,ENUM_TIMEFRAMES period,ENUM_APPLIED_VOLUME applied)
  {
   bool result;
//--- formation status string
   m_name  ="AD";
   m_status="("+symbol+","+PeriodStr(period)+","+VolumeStr(applied)+") H="+IntegerToString(m_handle);
//--- tuning
   result=CreateBuffers(symbol,period,1);
   if(result)
     {
      //--- save settings
      m_applied=applied;
      //--- create buffers
      ((CIndicatorBuffer*)At(0)).Name("MAIN_LINE");
     }
//---
   return(result);
  }
//+------------------------------------------------------------------+
//| Create indicator "Accumulation/Distribution".                    |
//| INPUT:  symbol  -chart symbol,                                   |
//|         period  -chart period,                                   |
//|         applied -what used.                                      |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CiAD::Create(string symbol,ENUM_TIMEFRAMES period,ENUM_APPLIED_VOLUME applied)
  {
   m_handle=iAD(symbol,period,applied);
//---
   if(m_handle!=INVALID_HANDLE)
     {
      //--- idicator creating successful
      if(!Initialize(symbol,period,applied))
        {
         //--- initialization failed
         IndicatorRelease(m_handle);
         m_handle=INVALID_HANDLE;
         return(false);
        }
      return(true);
     }
//---
   return(false);
  }
//+------------------------------------------------------------------+
//| Access to buffer of "Accumulation/Distribution".                 |
//| INPUT:  index -buffer index.                                     |
//| OUTPUT: value of buffer.                                         |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
double CiAD::Main(int index)
  {
   CIndicatorBuffer *buffer=At(0);
//--- checking
   if(buffer==NULL) return(EMPTY_VALUE);
//---
   return(buffer.At(index));
  }
//+------------------------------------------------------------------+
//| Class CiMFI.                                                     |
//| Appointment: Class indicator "Money Flow Index".                 |
//+------------------------------------------------------------------+
class CiMFI : public CIndicator
  {
protected:
   int               m_ma_period;
   ENUM_APPLIED_VOLUME m_applied;

public:
                     CiMFI();
   //--- methods of access to protected data
   int               MaPeriod() const  { return(m_ma_period); }
   ENUM_APPLIED_VOLUME Applied() const { return(m_applied);   }
   //--- method create
   bool              Create(string symbol,ENUM_TIMEFRAMES period,int ma_period,ENUM_APPLIED_VOLUME applied);
   //--- methods of access to indicator data
   double            Main(int index);
   //--- method of identifying
   virtual int       Type() const      { return(IND_MFI);     }

protected:
   //--- methods tune
   virtual bool      Initialize(string symbol,ENUM_TIMEFRAMES period,int num_params,MqlParam &params[]);
   bool              Initialize(string symbol,ENUM_TIMEFRAMES period,int ma_period,ENUM_APPLIED_VOLUME applied);
  };
//+------------------------------------------------------------------+
//| Constructor CiMFI.                                               |
//| INPUT:  no.                                                      |
//| OUTPUT: no.                                                      |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
CiMFI::CiMFI()
  {
//--- initialize protected data
   m_ma_period=-1;
   m_applied  =WRONG_VALUE;
  }
//+------------------------------------------------------------------+
//| Initialize indicator with the universal parameters.              |
//| INPUT:  symbol   -indicator symbol,                              |
//|         period   -indicator period,                              |
//|         num_param-number of parameters,                          |
//|         params   -array of parameters.                           |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CiMFI::Initialize(string symbol,ENUM_TIMEFRAMES period,int num_params,MqlParam &params[])
  {
   return(Initialize(symbol,period,params[0].integer_value,(ENUM_APPLIED_VOLUME)params[1].integer_value));
  }
//+------------------------------------------------------------------+
//| Initialize indicator with the special parameters.                |
//| INPUT:  symbol    -chart symbol,                                 |
//|         period    -chart period,                                 |
//|         ma_period -period MA,                                    |
//|         applied   -what used.                                    |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CiMFI::Initialize(string symbol,ENUM_TIMEFRAMES period,int ma_period,ENUM_APPLIED_VOLUME applied)
  {
   bool result;
//--- formation status string
   m_name  ="MFI";
   m_status="("+symbol+","+PeriodStr(period)+","+
             IntegerToString(ma_period)+","+VolumeStr(applied)+","+") H="+IntegerToString(m_handle);
//--- tuning
   result=CreateBuffers(symbol,period,1);
   if(result)
     {
      //--- save settings
      m_ma_period=ma_period;
      m_applied  =applied;
      //--- create buffers
      ((CIndicatorBuffer*)At(0)).Name("MAIN_LINE");
     }
//---
   return(result);
  }
//+------------------------------------------------------------------+
//| Create indicator "Money Flow Index".                             |
//| INPUT:  symbol    -chart symbol,                                 |
//|         period    -chart period,                                 |
//|         ma_period -period MA,                                    |
//|         applied   -what used.                                    |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CiMFI::Create(string symbol,ENUM_TIMEFRAMES period,int ma_period,ENUM_APPLIED_VOLUME applied)
  {
   m_handle=iMFI(symbol,period,ma_period,applied);
//---
   if(m_handle!=INVALID_HANDLE)
     {
      //--- idicator creating successful
      if(!Initialize(symbol,period,ma_period,applied))
        {
         //--- initialization failed
         IndicatorRelease(m_handle);
         m_handle=INVALID_HANDLE;
         return(false);
        }
      return(true);
     }
//---
   return(false);
  }
//+------------------------------------------------------------------+
//| Access to buffer of "Money Flow Index".                          |
//| INPUT:  index -buffer index.                                     |
//| OUTPUT: value of buffer.                                         |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
double CiMFI::Main(int index)
  {
   CIndicatorBuffer *buffer=At(0);
//--- checking
   if(buffer==NULL) return(EMPTY_VALUE);
//---
   return(buffer.At(index));
  }
//+------------------------------------------------------------------+
//| Class CiOBV.                                                     |
//| Appointment: Class indicator "On Balance Volume".                |
//+------------------------------------------------------------------+
class CiOBV : public CIndicator
  {
protected:
   ENUM_APPLIED_VOLUME m_applied;

public:
                     CiOBV();
   //--- methods of access to protected data
   ENUM_APPLIED_VOLUME Applied() const { return(m_applied); }
   //--- method create
   bool              Create(string symbol,ENUM_TIMEFRAMES period,ENUM_APPLIED_VOLUME applied);
   //--- methods of access to indicator data
   double            Main(int index);
   //--- method of identifying
   virtual int       Type() const      { return(IND_OBV);   }

protected:
   //--- methods tune
   virtual bool      Initialize(string symbol,ENUM_TIMEFRAMES period,int num_params,MqlParam &params[]);
   bool              Initialize(string symbol,ENUM_TIMEFRAMES period,ENUM_APPLIED_VOLUME applied);
  };
//+------------------------------------------------------------------+
//| Constructor CiOBV.                                               |
//| INPUT:  no.                                                      |
//| OUTPUT: no.                                                      |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
CiOBV::CiOBV()
  {
//--- initialize protected data
   m_applied=WRONG_VALUE;
  }
//+------------------------------------------------------------------+
//| Initialize indicator with the universal parameters.              |
//| INPUT:  symbol   -indicator symbol,                              |
//|         period   -indicator period,                              |
//|         num_param-number of parameters,                          |
//|         params   -array of parameters.                           |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CiOBV::Initialize(string symbol,ENUM_TIMEFRAMES period,int num_params,MqlParam &params[])
  {
   return(Initialize(symbol,period,(ENUM_APPLIED_VOLUME)params[0].integer_value));
  }
//+------------------------------------------------------------------+
//| Initialize indicator with the special parameters.                |
//| INPUT:  symbol  -chart symbol,                                   |
//|         period  -chart period,                                   |
//|         applied -what used.                                      |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CiOBV::Initialize(string symbol,ENUM_TIMEFRAMES period,ENUM_APPLIED_VOLUME applied)
  {
   bool result;
//--- formation status string
   m_name  ="OBV";
   m_status="("+symbol+","+PeriodStr(period)+","+VolumeStr(applied)+","+") H="+IntegerToString(m_handle);
//--- tuning
   result=CreateBuffers(symbol,period,1);
   if(result)
     {
      //--- save settings
      m_applied=applied;
      //--- create buffers
      ((CIndicatorBuffer*)At(0)).Name("MAIN_LINE");
     }
//---
   return(result);
  }
//+------------------------------------------------------------------+
//| Create indicator "On Balance Volume".                            |
//| INPUT:  symbol  -chart symbol,                                   |
//|         period  -chart period,                                   |
//|         applied -what used.                                      |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CiOBV::Create(string symbol,ENUM_TIMEFRAMES period,ENUM_APPLIED_VOLUME applied)
  {
   m_handle=iOBV(symbol,period,applied);
//---
   if(m_handle!=INVALID_HANDLE)
     {
      //--- idicator creating successful
      if(!Initialize(symbol,period,applied))
        {
         //--- initialization failed
         IndicatorRelease(m_handle);
         m_handle=INVALID_HANDLE;
         return(false);
        }
      return(true);
     }
//---
   return(false);
  }
//+------------------------------------------------------------------+
//| Access to buffer of "On Balance Volume".                         |
//| INPUT:  index -buffer index.                                     |
//| OUTPUT: value of buffer.                                         |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
double CiOBV::Main(int index)
  {
   CIndicatorBuffer *buffer=At(0);
//--- checking
   if(buffer==NULL) return(EMPTY_VALUE);
//---
   return(buffer.At(index));
  }
//+------------------------------------------------------------------+
//| Class CiVolumes.                                                 |
//| Appointment: Class indicator "Volumes".                          |
//+------------------------------------------------------------------+
class CiVolumes : public CIndicator
  {
protected:
   ENUM_APPLIED_VOLUME m_applied;

public:
                     CiVolumes();
   //--- methods of access to protected data
   ENUM_APPLIED_VOLUME Applied() const { return(m_applied);   }
   //--- method create
   bool              Create(string symbol,ENUM_TIMEFRAMES period,ENUM_APPLIED_VOLUME applied);
   //--- methods of access to indicator data
   double            Main(int index);
   //--- method of identifying
   virtual int       Type() const      { return(IND_VOLUMES); }

protected:
   //--- methods tune
   virtual bool      Initialize(string symbol,ENUM_TIMEFRAMES period,int num_params,MqlParam &params[]);
   bool              Initialize(string symbol,ENUM_TIMEFRAMES period,ENUM_APPLIED_VOLUME applied);
  };
//+------------------------------------------------------------------+
//| Constructor CiVolumes.                                           |
//| INPUT:  no.                                                      |
//| OUTPUT: no.                                                      |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
CiVolumes::CiVolumes()
  {
//--- initialize protected data
   m_applied=WRONG_VALUE;
  }
//+------------------------------------------------------------------+
//| Initialize indicator with the universal parameters.              |
//| INPUT:  symbol   -indicator symbol,                              |
//|         period   -indicator period,                              |
//|         num_param-number of parameters,                          |
//|         params   -array of parameters.                           |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CiVolumes::Initialize(string symbol,ENUM_TIMEFRAMES period,int num_params,MqlParam &params[])
  {
   return(Initialize(symbol,period,(ENUM_APPLIED_VOLUME)params[0].integer_value));
  }
//+------------------------------------------------------------------+
//| Initialize indicator with the special parameters.                |
//| INPUT:  symbol  -chart symbol,                                   |
//|         period  -chart period,                                   |
//|         applied -what used.                                      |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CiVolumes::Initialize(string symbol,ENUM_TIMEFRAMES period,ENUM_APPLIED_VOLUME applied)
  {
   bool result;
//--- formation status string
   m_name  ="Volumes";
   m_status="("+symbol+","+PeriodStr(period)+","+VolumeStr(applied)+","+") H="+IntegerToString(m_handle);
//--- tuning
   result=CreateBuffers(symbol,period,1);
   if(result)
     {
      //--- save settings
      m_applied=applied;
      //--- create buffers
      ((CIndicatorBuffer*)At(0)).Name("MAIN_LINE");
     }
//---
   return(result);
  }
//+------------------------------------------------------------------+
//| Create indicator "Volumes".                                      |
//| INPUT:  symbol  -chart symbol,                                   |
//|         period  -chart period,                                   |
//|         applied -what used.                                      |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CiVolumes::Create(string symbol,ENUM_TIMEFRAMES period,ENUM_APPLIED_VOLUME applied)
  {
   m_handle=iVolumes(symbol,period,applied);
//---
   if(m_handle!=INVALID_HANDLE)
     {
      //--- idicator creating successful
      if(!Initialize(symbol,period,applied))
        {
         //--- initialization failed
         IndicatorRelease(m_handle);
         m_handle=INVALID_HANDLE;
         return(false);
        }
      return(true);
     }
//---
   return(false);
  }
//+------------------------------------------------------------------+
//| Access to buffer of "Volumes".                                   |
//| INPUT:  index -buffer index.                                     |
//| OUTPUT: value of buffer.                                         |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
double CiVolumes::Main(int index)
  {
   CIndicatorBuffer *buffer=At(0);
//--- checking
   if(buffer==NULL) return(EMPTY_VALUE);
//---
   return(buffer.At(index));
  }
//+------------------------------------------------------------------+
